@@ -2,9 +2,19 @@
  * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh and
  *                Radu Serban @ LLNL
  * --------------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * --------------------------------------------------------------------
  * Demonstration program for CVODE - Krylov linear solver.
  * ODE system from ns-species interaction PDE in 2 dimensions.
- * 
+ *
  * This program solves a stiff ODE system that arises from a system
  * of partial differential equations. The PDE system is a food web
  * population model, with predator-prey interaction and diffusion on
@@ -17,14 +27,14 @@
  *
  *    i               i      i
  *  dc /dt  =  d(i)*(c    + c   )  +  f (x,y,c)  (i=1,...,ns)
- *                    xx     yy        i   
+ *                    xx     yy        i
  *
  * where
  *
  *                 i          ns         j
  *  f (x,y,c)  =  c *(b(i) + sum a(i,j)*c )
- *   i                       j=1                                         
- *                                                                       
+ *   i                       j=1
+ *
  * The number of species is ns = 2*np, with the first np being prey
  * and the last np being predators. The coefficients a(i,j), b(i),
  * d(i) are:
@@ -45,7 +55,7 @@
  *
  * The resulting ODE system is stiff.
  *
- * The ODE system is solved using Newton iteration and the SUNSPGMR
+ * The ODE system is solved using Newton iteration and the SUNLinSol_SPGMR
  * linear solver (scaled preconditioned GMRES).
  *
  * The preconditioner matrix used is the product of two matrices:
@@ -59,10 +69,10 @@
  * The product preconditoner is applied on the left and on the
  * right. In each case, both the modified and classical Gram-Schmidt
  * options are tested.
- * In the series of runs, CVodeInit, SUNSPGMR, and 
- * CVDlsSetLinearSolver are called only for the first run, whereas 
- * CVodeReInit, SUNSPGMRSetPrecType, and SUNSPGMRSetGSType are called
- * for each of the remaining three runs.
+ * In the series of runs, CVodeInit, SUNLinSol_SPGMR, and
+ * CVSetLinearSolver are called only for the first run, whereas
+ * CVodeReInit, SUNLinSol_SPGMRSetPrecType, and SUNLinSol_SPGMRSetGSType
+ * are called for each of the remaining three runs.
  *
  * A problem description, performance statistics at selected output
  * times, and final statistics are written to standard output.
@@ -71,7 +81,7 @@
  * but there should be no such messages.
  *
  * Note: This program requires the dense linear solver functions
- * newDenseMat, newIndexArray, denseAddIdentity, denseGETRF, denseGETRS, 
+ * newDenseMat, newIndexArray, denseAddIdentity, denseGETRF, denseGETRS,
  * destroyMat and destroyArray.
  *
  * Note: This program assumes the sequential implementation for the
@@ -82,7 +92,8 @@
  * Matrix Methods in Stiff ODE Systems, J. Appl. Math. & Comp., 31
  * (1989), pp. 40-91.  Also available as Lawrence Livermore National
  * Laboratory Report UCRL-95088, Rev. 1, June 1987.
- * --------------------------------------------------------------------*/
+ * --------------------------------------------------------------------
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -90,11 +101,19 @@
 
 #include <cvode/cvode.h>                /* main integrator header file                 */
 #include <sunlinsol/sunlinsol_spgmr.h>  /* access to SPGMR SUNLinearSolver             */
-#include <cvode/cvode_spils.h>          /* access to CVSpils interface                 */
 #include <nvector/nvector_serial.h>     /* serial N_Vector types, fct. and macros      */
 #include <sundials/sundials_dense.h>    /* use generic DENSE solver in preconditioning */
 #include <sundials/sundials_types.h>    /* definition of realtype                      */
-#include <sundials/sundials_math.h>     /* contains the macros ABS and SUNSQR          */
+
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
+#ifndef SQR
+#define SQR(A) ((A)*(A))
+#endif
 
 /* Constants */
 
@@ -137,7 +156,7 @@
 #define RTOL RCONST(1.0e-5)
 #define ATOL RCONST(1.0e-5)
 
-/* Spgmr/SPILS Constants */
+/* Spgmr/CVLS Constants */
 
 #define MAXL 0     /* => use default = MIN(NEQ, 5)            */
 #define DELT ZERO  /* => use default = 0.05                   */
@@ -200,7 +219,7 @@ static void v_zero(realtype u[], int n);
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
-static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok, 
+static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data);
 
 static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
@@ -208,7 +227,7 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
 
 /* Private function to check function return values */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /* Implementation */
 
@@ -220,7 +239,7 @@ int main()
   SUNLinearSolver LS;
   void *cvode_mem;
   booleantype firstrun;
-  int jpre, gstype, flag;
+  int jpre, gstype, retval;
   int ns, mxns, iout;
 
   c = NULL;
@@ -230,9 +249,9 @@ int main()
 
   /* Initializations */
   c = N_VNew_Serial(NEQ);
-  if(check_flag((void *)c, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)c, "N_VNew_Serial", 0)) return(1);
   wdata = AllocUserData();
-  if(check_flag((void *)wdata, "AllocUserData", 2)) return(1);
+  if(check_retval((void *)wdata, "AllocUserData", 2)) return(1);
   InitUserData(wdata);
   ns = wdata->ns;
   mxns = wdata->mxns;
@@ -243,72 +262,72 @@ int main()
   /* Loop over jpre and gstype (four cases) */
   for (jpre = PREC_LEFT; jpre <= PREC_RIGHT; jpre++) {
     for (gstype = MODIFIED_GS; gstype <= CLASSICAL_GS; gstype++) {
-      
+
       /* Initialize c and print heading */
       CInit(c, wdata);
       PrintHeader(jpre, gstype);
 
-      /* Call CVodeInit or CVodeReInit, then SUNSPGMR to set up problem */
-      
+      /* Call CVodeInit or CVodeReInit, then SUNLinSol_SPGMR to set up problem */
+
       firstrun = (jpre == PREC_LEFT) && (gstype == MODIFIED_GS);
       if (firstrun) {
-        cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
-        if(check_flag((void *)cvode_mem, "CVodeCreate", 0)) return(1);
+        cvode_mem = CVodeCreate(CV_BDF);
+        if(check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
 
         wdata->cvode_mem = cvode_mem;
 
-        flag = CVodeSetUserData(cvode_mem, wdata);
-        if(check_flag(&flag, "CVodeSetUserData", 1)) return(1);
+        retval = CVodeSetUserData(cvode_mem, wdata);
+        if(check_retval(&retval, "CVodeSetUserData", 1)) return(1);
 
-        flag = CVodeInit(cvode_mem, f, T0, c);
-        if(check_flag(&flag, "CVodeInit", 1)) return(1);
+        retval = CVodeInit(cvode_mem, f, T0, c);
+        if(check_retval(&retval, "CVodeInit", 1)) return(1);
 
-        flag = CVodeSStolerances(cvode_mem, reltol, abstol);
-        if(check_flag(&flag, "CVodeSStolerances", 1)) return(1);
+        retval = CVodeSStolerances(cvode_mem, reltol, abstol);
+        if(check_retval(&retval, "CVodeSStolerances", 1)) return(1);
 
-        LS = SUNSPGMR(c, jpre, MAXL);
-        if(check_flag((void *)LS, "SUNSPGMR", 0)) return(1);
+        LS = SUNLinSol_SPGMR(c, jpre, MAXL);
+        if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
-        flag = CVSpilsSetLinearSolver(cvode_mem, LS);
-        if(check_flag(&flag, "CVSpilsSetLinearSolver", 1)) return 1;
+        retval = CVodeSetLinearSolver(cvode_mem, LS, NULL);
+        if(check_retval(&retval, "CVodeSetLinearSolver", 1)) return 1;
 
-        flag = SUNSPGMRSetGSType(LS, gstype);
-        if(check_flag(&flag, "SUNSPGMRSetGSType", 1)) return(1);
+        retval = SUNLinSol_SPGMRSetGSType(LS, gstype);
+        if(check_retval(&retval, "SUNLinSol_SPGMRSetGSType", 1)) return(1);
 
-        flag = CVSpilsSetEpsLin(cvode_mem, DELT);
-        if(check_flag(&flag, "CVSpilsSetEpsLin", 1)) return(1);
+        retval = CVodeSetEpsLin(cvode_mem, DELT);
+        if(check_retval(&retval, "CVodeSetEpsLin", 1)) return(1);
 
-        flag = CVSpilsSetPreconditioner(cvode_mem, Precond, PSolve);
-        if(check_flag(&flag, "CVSpilsSetPreconditioner", 1)) return(1);
+        retval = CVodeSetPreconditioner(cvode_mem, Precond, PSolve);
+        if(check_retval(&retval, "CVodeSetPreconditioner", 1)) return(1);
 
       } else {
 
-        flag = CVodeReInit(cvode_mem, T0, c);
-        if(check_flag(&flag, "CVodeReInit", 1)) return(1);
+        retval = CVodeReInit(cvode_mem, T0, c);
+        if(check_retval(&retval, "CVodeReInit", 1)) return(1);
 
-        flag = SUNSPGMRSetPrecType(LS, jpre);
-        check_flag(&flag, "SUNSPGMRSetPrecType", 1);
-        flag = SUNSPGMRSetGSType(LS, gstype);
-        if(check_flag(&flag, "SUNSPGMRSetGSType", 1)) return(1);
+        retval = SUNLinSol_SPGMRSetPrecType(LS, jpre);
+        check_retval(&retval, "SUNLinSol_SPGMRSetPrecType", 1);
+        retval = SUNLinSol_SPGMRSetGSType(LS, gstype);
+        if(check_retval(&retval, "SUNLinSol_SPGMRSetGSType", 1)) return(1);
 
       }
-      
+
       /* Print initial values */
       if (firstrun) PrintAllSpecies(c, ns, mxns, T0);
-      
+
       /* Loop over output points, call CVode, print sample solution values. */
       tout = T1;
       for (iout = 1; iout <= NOUT; iout++) {
-        flag = CVode(cvode_mem, tout, c, &t, CV_NORMAL);
+        retval = CVode(cvode_mem, tout, c, &t, CV_NORMAL);
         PrintOutput(cvode_mem, t);
         if (firstrun && (iout % 3 == 0)) PrintAllSpecies(c, ns, mxns, t);
-        if(check_flag(&flag, "CVode", 1)) break;
-        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT; 
+        if(check_retval(&retval, "CVode", 1)) break;
+        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT;
       }
-      
+
       /* Print final statistics, and loop for next case */
       PrintFinalStats(cvode_mem);
-      
+
     }
   }
 
@@ -326,7 +345,7 @@ static WebData AllocUserData(void)
   int i, ngrp = NGRP;
   sunindextype ns = NS;
   WebData wdata;
-  
+
   wdata = (WebData) malloc(sizeof *wdata);
   for(i=0; i < ngrp; i++) {
     (wdata->P)[i] = newDenseMat(ns, ns);
@@ -342,14 +361,14 @@ static void InitUserData(WebData wdata)
   int i, j, ns;
   realtype *bcoef, *diff, *cox, *coy, dx, dy;
   realtype (*acoef)[NS];
-  
+
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
   diff = wdata->diff;
   cox = wdata->cox;
   coy = wdata->coy;
   ns = wdata->ns = NS;
-  
+
   for (j = 0; j < NS; j++) { for (i = 0; i < NS; i++) acoef[i][j] = 0.; }
   for (j = 0; j < NP; j++) {
     for (i = 0; i < NP; i++) {
@@ -370,8 +389,8 @@ static void InitUserData(WebData wdata)
   dx = wdata->dx = DX;
   dy = wdata->dy = DY;
   for (i = 0; i < ns; i++) {
-    cox[i] = diff[i]/SUNSQR(dx);
-    coy[i] = diff[i]/SUNSQR(dy);
+    cox[i] = diff[i]/SQR(dx);
+    coy[i] = diff[i]/SQR(dy);
   }
 
   /* Set remaining method parameters */
@@ -380,7 +399,7 @@ static void InitUserData(WebData wdata)
   wdata->mq = MQ;
   wdata->mx = MX;
   wdata->my = MY;
-  wdata->srur = SUNRsqrt(UNIT_ROUNDOFF);
+  wdata->srur = sqrt(UNIT_ROUNDOFF);
   wdata->mxmp = MXMP;
   wdata->ngrp = NGRP;
   wdata->ngx = NGX;
@@ -407,7 +426,7 @@ static void SetGroups(int m, int ng, int jg[], int jig[], int jr[])
   mper = m/ng; /* does integer division */
   for (ig=0; ig < ng; ig++) jg[ig] = ig*mper;
   jg[ng] = m;
-  
+
   ngm1 = ng - 1;
   len1 = ngm1*mper;
   for (j = 0; j < len1; j++) jig[j] = j/mper;
@@ -422,22 +441,22 @@ static void CInit(N_Vector c, WebData wdata)
 {
   int jx, jy, ns, mxns, ioff, iyoff, i, ici;
   realtype argx, argy, x, y, dx, dy, x_factor, y_factor, *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
   ns = wdata->ns;
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
 
-  x_factor = RCONST(4.0)/SUNSQR(AX);
-  y_factor = RCONST(4.0)/SUNSQR(AY);
+  x_factor = RCONST(4.0)/SQR(AX);
+  y_factor = RCONST(4.0)/SQR(AY);
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
-    argy = SUNSQR(y_factor*y*(AY-y));
+    argy = SQR(y_factor*y*(AY-y));
     iyoff = mxns*jy;
     for (jx = 0; jx < MX; jx++) {
       x = jx*dx;
-      argx = SUNSQR(x_factor*x*(AX-x));
+      argx = SQR(x_factor*x*(AX-x));
       ioff = iyoff + ns*jx;
       for (i = 1; i <= ns; i++) {
         ici = ioff + i-1;
@@ -514,7 +533,7 @@ static void PrintAllSpecies(N_Vector c, int ns, int mxns, realtype t)
 {
   int i, jx ,jy;
   realtype *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("c values at t = %Lg:\n\n", t);
@@ -544,19 +563,19 @@ static void PrintAllSpecies(N_Vector c, int ns, int mxns, realtype t)
 static void PrintOutput(void *cvode_mem, realtype t)
 {
   long int nst, nfe, nni;
-  int qu, flag;
+  int qu, retval;
   realtype hu;
 
-  flag = CVodeGetNumSteps(cvode_mem, &nst);
-  check_flag(&flag, "CVodeGetNumSteps", 1);
-  flag = CVodeGetNumRhsEvals(cvode_mem, &nfe);
-  check_flag(&flag, "CVodeGetNumRhsEvals", 1);
-  flag = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
-  check_flag(&flag, "CVodeGetNumNonlinSolvIters", 1);
-  flag = CVodeGetLastOrder(cvode_mem, &qu);
-  check_flag(&flag, "CVodeGetLastOrder", 1);
-  flag = CVodeGetLastStep(cvode_mem, &hu);
-  check_flag(&flag, "CVodeGetLastStep", 1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst);
+  check_retval(&retval, "CVodeGetNumSteps", 1);
+  retval = CVodeGetNumRhsEvals(cvode_mem, &nfe);
+  check_retval(&retval, "CVodeGetNumRhsEvals", 1);
+  retval = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
+  check_retval(&retval, "CVodeGetNumNonlinSolvIters", 1);
+  retval = CVodeGetLastOrder(cvode_mem, &qu);
+  check_retval(&retval, "CVodeGetLastOrder", 1);
+  retval = CVodeGetLastStep(cvode_mem, &hu);
+  check_retval(&retval, "CVodeGetLastStep", 1);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("t = %10.2Le  nst = %ld  nfe = %ld  nni = %ld", t, nst, nfe, nni);
@@ -576,42 +595,42 @@ static void PrintFinalStats(void *cvode_mem)
   long int lenrwLS, leniwLS;
   long int nst, nfe, nsetups, nni, ncfn, netf;
   long int nli, npe, nps, ncfl, nfeLS;
-  int flag;
+  int retval;
   realtype avdim;
-  
-  flag = CVodeGetWorkSpace(cvode_mem, &lenrw, &leniw);
-  check_flag(&flag, "CVodeGetWorkSpace", 1);
-  flag = CVodeGetNumSteps(cvode_mem, &nst);
-  check_flag(&flag, "CVodeGetNumSteps", 1);
-  flag = CVodeGetNumRhsEvals(cvode_mem, &nfe);
-  check_flag(&flag, "CVodeGetNumRhsEvals", 1);
-  flag = CVodeGetNumLinSolvSetups(cvode_mem, &nsetups);
-  check_flag(&flag, "CVodeGetNumLinSolvSetups", 1);
-  flag = CVodeGetNumErrTestFails(cvode_mem, &netf);
-  check_flag(&flag, "CVodeGetNumErrTestFails", 1);
-  flag = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
-  check_flag(&flag, "CVodeGetNumNonlinSolvIters", 1);
-  flag = CVodeGetNumNonlinSolvConvFails(cvode_mem, &ncfn);
-  check_flag(&flag, "CVodeGetNumNonlinSolvConvFails", 1);
 
-  flag = CVSpilsGetWorkSpace(cvode_mem, &lenrwLS, &leniwLS);
-  check_flag(&flag, "CVSpilsGetWorkSpace", 1);
-  flag = CVSpilsGetNumLinIters(cvode_mem, &nli);
-  check_flag(&flag, "CVSpilsGetNumLinIters", 1);
-  flag = CVSpilsGetNumPrecEvals(cvode_mem, &npe);
-  check_flag(&flag, "CVSpilsGetNumPrecEvals", 1);
-  flag = CVSpilsGetNumPrecSolves(cvode_mem, &nps);
-  check_flag(&flag, "CVSpilsGetNumPrecSolves", 1);
-  flag = CVSpilsGetNumConvFails(cvode_mem, &ncfl);
-  check_flag(&flag, "CVSpilsGetNumConvFails", 1);
-  flag = CVSpilsGetNumRhsEvals(cvode_mem, &nfeLS);
-  check_flag(&flag, "CVSpilsGetNumRhsEvals", 1);
+  retval = CVodeGetWorkSpace(cvode_mem, &lenrw, &leniw);
+  check_retval(&retval, "CVodeGetWorkSpace", 1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst);
+  check_retval(&retval, "CVodeGetNumSteps", 1);
+  retval = CVodeGetNumRhsEvals(cvode_mem, &nfe);
+  check_retval(&retval, "CVodeGetNumRhsEvals", 1);
+  retval = CVodeGetNumLinSolvSetups(cvode_mem, &nsetups);
+  check_retval(&retval, "CVodeGetNumLinSolvSetups", 1);
+  retval = CVodeGetNumErrTestFails(cvode_mem, &netf);
+  check_retval(&retval, "CVodeGetNumErrTestFails", 1);
+  retval = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
+  check_retval(&retval, "CVodeGetNumNonlinSolvIters", 1);
+  retval = CVodeGetNumNonlinSolvConvFails(cvode_mem, &ncfn);
+  check_retval(&retval, "CVodeGetNumNonlinSolvConvFails", 1);
+
+  retval = CVodeGetLinWorkSpace(cvode_mem, &lenrwLS, &leniwLS);
+  check_retval(&retval, "CVodeGetLinWorkSpace", 1);
+  retval = CVodeGetNumLinIters(cvode_mem, &nli);
+  check_retval(&retval, "CVodeGetNumLinIters", 1);
+  retval = CVodeGetNumPrecEvals(cvode_mem, &npe);
+  check_retval(&retval, "CVodeGetNumPrecEvals", 1);
+  retval = CVodeGetNumPrecSolves(cvode_mem, &nps);
+  check_retval(&retval, "CVodeGetNumPrecSolves", 1);
+  retval = CVodeGetNumLinConvFails(cvode_mem, &ncfl);
+  check_retval(&retval, "CVodeGetNumLinConvFails", 1);
+  retval = CVodeGetNumLinRhsEvals(cvode_mem, &nfeLS);
+  check_retval(&retval, "CVodeGetNumLinRhsEvals", 1);
 
   printf("\n\n Final statistics for this run:\n\n");
   printf(" CVode real workspace length           = %4ld \n", lenrw);
   printf(" CVode integer workspace length        = %4ld \n", leniw);
-  printf(" CVSPILS real workspace length         = %4ld \n", lenrwLS);
-  printf(" CVSPILS integer workspace length      = %4ld \n", leniwLS);
+  printf(" CVLS real workspace length            = %4ld \n", lenrwLS);
+  printf(" CVLS integer workspace length         = %4ld \n", leniwLS);
   printf(" Number of steps                       = %4ld \n", nst);
   printf(" Number of f-s                         = %4ld \n", nfe);
   printf(" Number of f-s (SPGMR)                 = %4ld \n", nfeLS);
@@ -661,11 +680,11 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   realtype dcxli, dcxui, dcyli, dcyui, x, y, *cox, *coy, *fsave, dx, dy;
   realtype *cdata, *cdotdata;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
   cdata = N_VGetArrayPointer(c);
   cdotdata = N_VGetArrayPointer(cdot);
-  
+
   mxns = wdata->mxns;
   ns = wdata->ns;
   fsave = wdata->fsave;
@@ -674,7 +693,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
-  
+
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
     iyoff = mxns*jy;
@@ -707,7 +726,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
 
 /*
   This routine computes the interaction rates for the species
-  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point 
+  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point
   and at time t.
 */
 static void WebRates(realtype x, realtype y, realtype t, realtype c[],
@@ -716,20 +735,20 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
   int i, j, ns;
   realtype fac, *bcoef;
   realtype (*acoef)[NS];
-  
+
   ns = wdata->ns;
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
-  
+
   for (i = 0; i < ns; i++)
     rate[i] = ZERO;
-  
-  for (j = 0; j < ns; j++) 
-    for (i = 0; i < ns; i++) 
+
+  for (j = 0; j < ns; j++)
+    for (i = 0; i < ns; i++)
       rate[i] += c[j] * acoef[i][j];
-  
+
   fac = ONE + ALPH*x*y;
-  for (i = 0; i < ns; i++) 
+  for (i = 0; i < ns; i++)
     rate[i] = c[i]*(bcoef[i]*fac + rate[i]);
 }
 
@@ -745,28 +764,27 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
  This routine can be regarded as a prototype for the general case
  of a block-diagonal preconditioner. The blocks are of size mp, and
  there are ngrp=ngx*ngy blocks computed in the block-grouping scheme.
-*/ 
+*/
 static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data)
 {
   realtype ***P;
-  int ier;
+  sunindextype ier;
   sunindextype **pivot;
   int i, if0, if00, ig, igx, igy, j, jj, jx, jy;
-  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, flag;
-  sunindextype mp;
+  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, mp, retval;
   realtype uround, fac, r, r0, save, srur;
   realtype *f1, *fsave, *cdata, *rewtdata;
   WebData wdata;
   void *cvode_mem;
   N_Vector rewt;
-  
+
   wdata = (WebData) user_data;
   cvode_mem = wdata->cvode_mem;
   cdata = N_VGetArrayPointer(c);
   rewt = wdata->rewt;
-  flag = CVodeGetErrWeights(cvode_mem, rewt);
-  if(check_flag(&flag, "CVodeGetErrWeights", 1)) return(1);
+  retval = CVodeGetErrWeights(cvode_mem, rewt);
+  if(check_retval(&retval, "CVodeGetErrWeights", 1)) return(1);
   rewtdata = N_VGetArrayPointer(rewt);
 
   uround = UNIT_ROUNDOFF;
@@ -782,30 +800,30 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
   ngy = wdata->ngy;
   mxmp = wdata->mxmp;
   fsave = wdata->fsave;
-  
+
   /* Make mp calls to fblock to approximate each diagonal block of Jacobian.
-     Here, fsave contains the base value of the rate vector and 
+     Here, fsave contains the base value of the rate vector and
      r0 is a minimum increment factor for the difference quotient. */
-  
+
   f1 = N_VGetArrayPointer(wdata->tmp);
-  
+
   fac = N_VWrmsNorm (fc, rewt);
-  r0 = RCONST(1000.0)*SUNRabs(gamma)*uround*NEQ*fac;
+  r0 = RCONST(1000.0)*fabs(gamma)*uround*NEQ*fac;
   if (r0 == ZERO) r0 = ONE;
-  
+
   for (igy = 0; igy < ngy; igy++) {
     jy = jyr[igy];
     if00 = jy*mxmp;
-    for (igx = 0; igx < ngx; igx++) { 
+    for (igx = 0; igx < ngx; igx++) {
       jx = jxr[igx];
       if0 = if00 + jx*mp;
-      ig = igx + igy*ngx; 
+      ig = igx + igy*ngx;
       /* Generate ig-th diagonal block */
       for (j = 0; j < mp; j++) {
         /* Generate the jth column as a difference quotient */
-        jj = if0 + j; 
+        jj = if0 + j;
         save = cdata[jj];
-        r = SUNMAX(srur*SUNRabs(save),r0/rewtdata[jj]);
+        r = MAX(srur*fabs(save),r0/rewtdata[jj]);
         cdata[jj] += r;
         fac = -gamma/r;
         fblock (t, cdata, jx, jy, f1, wdata);
@@ -816,15 +834,15 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
       }
     }
   }
-  
+
   /* Add identity matrix and do LU decompositions on blocks. */
-  
+
   for (ig = 0; ig < ngrp; ig++) {
     denseAddIdentity(P[ig], mp);
     ier = denseGETRF(P[ig], mp, mp, pivot[ig]);
     if (ier != 0) return(1);
   }
-  
+
   *jcurPtr = SUNTRUE;
   return(0);
 }
@@ -839,7 +857,7 @@ static void fblock(realtype t, realtype cdata[], int jx, int jy,
 {
   int iblok, ic;
   realtype x, y;
-  
+
   iblok = jx + jy*(wdata->mx);
   y = jy*(wdata->dy);
   x = jx*(wdata->dx);
@@ -862,20 +880,19 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
 {
   realtype   ***P;
   sunindextype **pivot;
-  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx;
-  sunindextype mp;
+  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx, mp;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
-  
+
   N_VScale(ONE, r, z);
-  
+
   /* call GSIter for Gauss-Seidel iterations */
-  
+
   GSIter(gamma, z, wdata->tmp, wdata);
-  
+
   /* Do backsolves for inverse of block-diagonal preconditioner factor */
-  
+
   P = wdata->P;
   pivot = wdata->pivot;
   mx = wdata->mx;
@@ -884,7 +901,7 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
   mp = wdata->mp;
   jigx = wdata->jigx;
   jigy = wdata->jigy;
-  
+
   iv = 0;
   for (jy = 0; jy < my; jy++) {
     igy = jigy[jy];
@@ -895,7 +912,7 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
       iv += mp;
     }
   }
-  
+
   return(0);
 }
 
@@ -914,7 +931,7 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   int ns, mxns, i, iyoff, ic, iter;
   realtype beta[NS], beta2[NS], cof1[NS], gam[NS], gam2[NS];
   realtype temp, *cox, *coy, *xd, *zd;
-  
+
   xd = N_VGetArrayPointer(x);
   zd = N_VGetArrayPointer(z);
   ns = wdata->ns;
@@ -923,10 +940,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   mxns = wdata->mxns;
   cox = wdata->cox;
   coy = wdata->coy;
-  
+
   /* Write matrix as P = D - L - U.
      Load local arrays beta, beta2, gam, gam2, and cof1. */
-  
+
   for (i = 0; i < ns; i++) {
     temp = ONE/(ONE + RCONST(2.0)*gamma*(cox[i] + coy[i]));
     beta[i] = gamma*cox[i]*temp;
@@ -935,10 +952,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     gam2[i] = RCONST(2.0)*gam[i];
     cof1[i] = temp;
   }
-  
+
   /* Begin iteration loop.
      Load vector x with (D-inverse)*z for first iteration. */
-  
+
   for (jy = 0; jy < my; jy++) {
     iyoff = mxns*jy;
     for (jx = 0; jx < mx; jx++) {
@@ -947,13 +964,13 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     }
   }
   N_VConst(ZERO, z);
-  
+
   /* Looping point for iterations. */
-  
+
   for (iter=1; iter <= ITMAX; iter++) {
-    
+
     /* Calculate (D-inverse)*U*x if not the first iteration. */
-    
+
     if (iter > 1) {
       for (jy=0; jy < my; jy++) {
         iyoff = mxns*jy;
@@ -962,47 +979,47 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
           x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
           y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
           switch (3*y_loc+x_loc) {
-          case 0 : 
+          case 0 :
             /* jx == 0, jy == 0 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 1 : 
+          case 1 :
             /* 1 <= jx <= mx-2, jy == 0 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 2 : 
+          case 2 :
             /* jx == mx-1, jy == 0 */
             /* x[ic+i] = gam2[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam2, xd+ic+mxns, ns);
             break;
-          case 3 : 
+          case 3 :
             /* jx == 0, 1 <= jy <= my-2 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 4 : 
+          case 4 :
             /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 5 : 
+          case 5 :
             /* jx == mx-1, 1 <= jy <= my-2 */
             /* x[ic+i] = gam[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam, xd+ic+mxns, ns);
             break;
-          case 6 : 
+          case 6 :
             /* jx == 0, jy == my-1 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] */
             v_prod(xd+ic, beta2, xd+ic+ns, ns);
             break;
-          case 7 : 
+          case 7 :
             /* 1 <= jx <= mx-2, jy == my-1 */
             /* x[ic+i] = beta[i]x[ic+ns+i] */
             v_prod(xd+ic, beta, xd+ic+ns, ns);
             break;
-          case 8 : 
+          case 8 :
             /* jx == mx-1, jy == my-1 */
             /* x[ic+i] = 0.0 */
             v_zero(xd+ic, ns);
@@ -1011,9 +1028,9 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }  /* end if (iter > 1) */
-    
+
     /* Overwrite x with [(I - (D-inverse)*L)-inverse]*x. */
-    
+
     for (jy=0; jy < my; jy++) {
       iyoff = mxns*jy;
       for (jx=0; jx < mx; jx++) { /* order of loops matters */
@@ -1021,48 +1038,48 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
         y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
         switch (3*y_loc+x_loc) {
-        case 0 : 
+        case 0 :
           /* jx == 0, jy == 0 */
           break;
-        case 1 : 
+        case 1 :
           /* 1 <= jx <= mx-2, jy == 0 */
           /* x[ic+i] += beta[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           break;
-        case 2 : 
+        case 2 :
           /* jx == mx-1, jy == 0 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           break;
-        case 3 : 
+        case 3 :
           /* jx == 0, 1 <= jy <= my-2 */
           /* x[ic+i] += gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 4 : 
+        case 4 :
           /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 5 : 
+        case 5 :
           /* jx == mx-1, 1 <= jy <= my-2 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 6 : 
+        case 6 :
           /* jx == 0, jy == my-1 */
           /* x[ic+i] += gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 7 : 
+        case 7 :
           /* 1 <= jx <= mx-2, jy == my-1 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 8 : 
+        case 8 :
           /* jx == mx-1, jy == my-1 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
@@ -1071,67 +1088,67 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }
-    
+
     /* Add increment x to z : z <- z+x */
-    
+
     N_VLinearSum(ONE, z, ONE, x, z);
-    
+
   }
 }
 
 static void v_inc_by_prod(realtype u[], realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] += v[i]*w[i];
 }
 
 static void v_sum_prods(realtype u[], realtype p[], realtype q[],
                         realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = p[i]*q[i] + v[i]*w[i];
 }
 
 static void v_prod(realtype u[], realtype v[], realtype w[], int n)
-{ 
+{
   int i;
   for (i=0; i < n; i++) u[i] = v[i]*w[i];
 }
 
 static void v_zero(realtype u[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = ZERO;
 }
 
 /* Check function return value...
      opt == 0 means SUNDIALS function allocates memory so check if
               returned NULL pointer
-     opt == 1 means SUNDIALS function returns a flag so check if
-              flag >= 0
+     opt == 1 means SUNDIALS function returns an integer value so check if
+              retval < 0
      opt == 2 means function allocates memory so check if returned
               NULL pointer */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && returnvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1); }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-              funcname, *errflag);
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
       return(1); }}
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && returnvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1); }
