@@ -4,7 +4,7 @@
  *                and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -63,7 +63,6 @@
 #include <cvodes/cvodes.h>
 #include <nvector/nvector_parallel.h>
 #include <sundials/sundials_types.h>
-#include <sundials/sundials_math.h>
 #include "sunnonlinsol/sunnonlinsol_fixedpoint.h" /* access to the fixed point SUNNonlinearSolver */
 
 #include <mpi.h>
@@ -83,7 +82,7 @@
 
 #define ZERO  RCONST(0.0)
 
-/* Type : UserData 
+/* Type : UserData
    contains problem parameters, grid constants, work array. */
 
 typedef struct {
@@ -142,6 +141,8 @@ int main(int argc, char *argv[])
   pbar = NULL;
   plist = NULL;
   uS = NULL;
+  NLS = NULL;
+  NLSsens = NULL;
 
   /* Get processor number, total number of pe's, and my_pe. */
   MPI_Init(&argc, &argv);
@@ -200,7 +201,7 @@ int main(int argc, char *argv[])
   /* attach nonlinear solver object to CVode */
   retval = CVodeSetNonlinearSolver(cvode_mem, NLS);
   if(check_retval(&retval, "CVodeSetNonlinearSolver", 1, my_pe)) MPI_Abort(comm, 1);
- 
+
   if (my_pe == 0) {
     printf("\n1-D advection-diffusion equation, mesh size =%3d \n", MX);
     printf("\nNumber of PEs = %3d \n",npes);
@@ -218,7 +219,7 @@ int main(int argc, char *argv[])
     for(is=0; is<NS; is++) pbar[is] = data->p[plist[is]];
 
     uS = N_VCloneVectorArray_Parallel(NS, u);
-    if(check_retval((void *)uS, "N_VCloneVectorArray_Parallel", 0, my_pe)) 
+    if(check_retval((void *)uS, "N_VCloneVectorArray_Parallel", 0, my_pe))
       MPI_Abort(comm, 1);
     for(is=0;is<NS;is++)
       N_VConst(ZERO,uS[is]);
@@ -258,11 +259,11 @@ int main(int argc, char *argv[])
 
     if(my_pe == 0) {
       printf("Sensitivity: YES ");
-      if(sensi_meth == CV_SIMULTANEOUS)   
+      if(sensi_meth == CV_SIMULTANEOUS)
         printf("( SIMULTANEOUS +");
-      else 
+      else
         if(sensi_meth == CV_STAGGERED) printf("( STAGGERED +");
-        else                           printf("( STAGGERED1 +");   
+        else                           printf("( STAGGERED1 +");
       if(err_con) printf(" FULL ERROR CONTROL )");
       else        printf(" PARTIAL ERROR CONTROL )");
     }
@@ -298,12 +299,12 @@ int main(int argc, char *argv[])
   }
 
   /* Print final statistics */
-  if (my_pe == 0) 
+  if (my_pe == 0)
     PrintFinalStats(cvode_mem, sensi, err_con, sensi_meth);
 
   /* Free memory */
   N_VDestroy(u);                   /* Free the u vector              */
-  if (sensi) 
+  if (sensi)
     N_VDestroyVectorArray(uS, NS); /* Free the uS vectors            */
   free(data->p);                   /* Free the p vector              */
   free(data);                      /* Free block of UserData         */
@@ -325,7 +326,7 @@ int main(int argc, char *argv[])
  */
 
 /*
- * f routine. Compute f(t,u). 
+ * f routine. Compute f(t,u).
  */
 
 static int f(realtype t, N_Vector u, N_Vector udot, void *user_data)
@@ -333,8 +334,8 @@ static int f(realtype t, N_Vector u, N_Vector udot, void *user_data)
   realtype ui, ult, urt, hordc, horac, hdiff, hadv;
   realtype *udata, *dudata, *z;
   realtype dx;
-  int i;
-  int npes, my_pe, my_length, my_pe_m1, my_pe_p1, last_pe;
+  int npes, my_pe, my_pe_m1, my_pe_p1, last_pe;
+  sunindextype i, my_length;
   UserData data;
   MPI_Status status;
   MPI_Comm comm;
@@ -344,15 +345,15 @@ static int f(realtype t, N_Vector u, N_Vector udot, void *user_data)
 
   /* Extract needed problem constants from data */
   data  = (UserData) user_data;
-  dx    = data->dx; 
+  dx    = data->dx;
   hordc = data->p[0]/(dx*dx);
   horac = data->p[1]/(RCONST(2.0)*dx);
 
   /* Extract parameters for parallel computation. */
   comm = data->comm;
-  npes = data->npes;           /* Number of processes. */ 
+  npes = data->npes;           /* Number of processes. */
   my_pe = data->my_pe;         /* Current process number. */
-  my_length = N_VGetLocalLength_Parallel(u); /* Number of local elements of u. */ 
+  my_length = N_VGetLocalLength_Parallel(u); /* Number of local elements of u. */
   z = data->z;
 
   /* Compute related parameters. */
@@ -366,17 +367,17 @@ static int f(realtype t, N_Vector u, N_Vector udot, void *user_data)
 
   /* Pass needed data to processes before and after current process. */
    if (my_pe != 0)
-     MPI_Send(&z[1], 1, PVEC_REAL_MPI_TYPE, my_pe_m1, 0, comm);
+     MPI_Send(&z[1], 1, MPI_SUNREALTYPE, my_pe_m1, 0, comm);
    if (my_pe != last_pe)
-     MPI_Send(&z[my_length], 1, PVEC_REAL_MPI_TYPE, my_pe_p1, 0, comm);   
+     MPI_Send(&z[my_length], 1, MPI_SUNREALTYPE, my_pe_p1, 0, comm);
 
   /* Receive needed data from processes before and after current process. */
    if (my_pe != 0)
-     MPI_Recv(&z[0], 1, PVEC_REAL_MPI_TYPE, my_pe_m1, 0, comm, &status);
+     MPI_Recv(&z[0], 1, MPI_SUNREALTYPE, my_pe_m1, 0, comm, &status);
    else z[0] = ZERO;
    if (my_pe != last_pe)
-     MPI_Recv(&z[my_length+1], 1, PVEC_REAL_MPI_TYPE, my_pe_p1, 0, comm,
-              &status);   
+     MPI_Recv(&z[my_length+1], 1, MPI_SUNREALTYPE, my_pe_p1, 0, comm,
+              &status);
    else z[my_length + 1] = ZERO;
 
   /* Loop over all grid points in current process. */
@@ -402,7 +403,7 @@ static int f(realtype t, N_Vector u, N_Vector udot, void *user_data)
  *--------------------------------------------------------------------
  */
 
-/* 
+/*
  * Process and verify arguments to cvsfwdnonx_p.
  */
 
@@ -421,7 +422,7 @@ static void ProcessArgs(int argc, char *argv[], int my_pe,
     *sensi = SUNTRUE;
   else
     WrongArgs(my_pe, argv[0]);
-  
+
   if (*sensi) {
 
     if (argc != 4)
@@ -433,7 +434,7 @@ static void ProcessArgs(int argc, char *argv[], int my_pe,
       *sensi_meth = CV_STAGGERED;
     else if (strcmp(argv[2],"stg1") == 0)
       *sensi_meth = CV_STAGGERED1;
-    else 
+    else
       WrongArgs(my_pe, argv[0]);
 
     if (strcmp(argv[3],"t") == 0)
@@ -452,16 +453,16 @@ static void WrongArgs(int my_pe, char *name)
     printf("\nUsage: %s [-nosensi] [-sensi sensi_meth err_con]\n",name);
     printf("         sensi_meth = sim, stg, or stg1\n");
     printf("         err_con    = t or f\n");
-  }  
+  }
   MPI_Finalize();
   exit(0);
 }
 
 /*
- * Set initial conditions in u vector 
+ * Set initial conditions in u vector
  */
 
-static void SetIC(N_Vector u, realtype dx, sunindextype my_length, 
+static void SetIC(N_Vector u, realtype dx, sunindextype my_length,
                   sunindextype my_base)
 {
   int i;
@@ -477,12 +478,12 @@ static void SetIC(N_Vector u, realtype dx, sunindextype my_length,
   for (i=1; i<=my_length; i++) {
     iglobal = my_base + i;
     x = iglobal*dx;
-    udata[i-1] = x*(XMAX - x)*SUNRexp(2.0*x);
-  }  
+    udata[i-1] = x*(XMAX - x)*exp(2.0*x);
+  }
 }
 
 /*
- * Print current t, step count, order, stepsize, and max norm of solution  
+ * Print current t, step count, order, stepsize, and max norm of solution
  */
 
 static void PrintOutput(void *cvode_mem, int my_pe, realtype t, N_Vector u)
@@ -520,12 +521,12 @@ static void PrintOutput(void *cvode_mem, int my_pe, realtype t, N_Vector u)
     printf("%12.4e \n", umax);
 #endif
 
-  }  
+  }
 
 }
 
 /*
- * Print max norm of sensitivities 
+ * Print max norm of sensitivities
  */
 
 static void PrintOutputS(int my_pe, N_Vector *uS)
@@ -559,7 +560,7 @@ static void PrintOutputS(int my_pe, N_Vector *uS)
 }
 
 /*
- * Print some final statistics located in the iopt array 
+ * Print some final statistics located in the iopt array
  */
 
 static void PrintFinalStats(void *cvode_mem, booleantype sensi,
@@ -623,14 +624,14 @@ static void PrintFinalStats(void *cvode_mem, booleantype sensi,
 
 }
 
-/* 
+/*
  * Check function return value...
  *   opt == 0 means SUNDIALS function allocates memory so check if
  *            returned NULL pointer
  *   opt == 1 means SUNDIALS function returns an integer value so check if
  *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt, int id)
