@@ -4,7 +4,7 @@
  *      Based on idaFoodWeb_bnd.c and parallelized with OpenMP
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -15,7 +15,7 @@
  * -----------------------------------------------------------------
  * Example program for IDAS: Food web problem.
  *
- * This example program (OpenMP version) uses the SUNBAND linear 
+ * This example program (OpenMP version) uses the SUNBAND linear
  * solver, and IDACalcIC for initial condition calculation.
  *
  * The mathematical problem solved in this example is a DAE system
@@ -72,19 +72,19 @@
  * The DAE system is solved by IDAS using the SUNBAND linear solver.
  * Output is printed at t = 0, .001, .01, .1, .4, .7, 1.
  *
- * Optionally, we can set the number of threads from environment 
+ * Optionally, we can set the number of threads from environment
  * variable or command line. To check the current value for number
  * of threads from environment:
  *      % echo $OMP_NUM_THREADS
  *
  * Execution:
  *
- * To use the default value for the number of threads from 
+ * To use the default value for the number of threads from
  * the OMP_NUM_THREADS environment value:
- *      % ./idasFoodWeb_bnd_omp 
+ *      % ./idasFoodWeb_bnd_omp
  * To specify the number of threads at the command line, use
  *      % ./idasFoodWeb_bnd_omp num_threads
- * where num_threads is the desired number of threads. 
+ * where num_threads is the desired number of threads.
  *
  * -----------------------------------------------------------------
  * References:
@@ -147,15 +147,15 @@
 #define NOUT        6              /* Number of output times            */
 #define TMULT       RCONST(10.0)   /* Multiplier for tout values        */
 #define TADD        RCONST(0.3)    /* Increment for tout values         */
-#define ZERO        RCONST(0.)     
-#define ONE         RCONST(1.0)    
+#define ZERO        RCONST(0.)
+#define ONE         RCONST(1.0)
 
-/* 
+/*
  * User-defined vector and accessor macro: IJ_Vptr.
- * IJ_Vptr is defined in order to express the underlying 3-D structure of 
+ * IJ_Vptr is defined in order to express the underlying 3-D structure of
  * the dependent variable vector from its underlying 1-D storage (an N_Vector).
- * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
- * species index is = 0, x-index ix = i, and y-index jy = j.                
+ * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
+ * species index is = 0, x-index ix = i, and y-index jy = j.
  */
 
 #define IJ_Vptr(vv,i,j) (&NV_Ith_OMP(vv, (i)*NUM_SPECIES + (j)*NSMX))
@@ -172,7 +172,7 @@ typedef struct {
 
 /* Prototypes for functions called by the IDA Solver. */
 
-static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval, 
+static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval,
                   void *user_data);
 
 /* Prototypes for private Helper Functions. */
@@ -184,7 +184,7 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
 static void PrintOutput(void *ida_mem, N_Vector c, realtype t);
 static void PrintFinalStats(void *ida_mem);
 static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate, UserData webdata);
-static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                      UserData webdata);
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2);
 static int check_retval(void *returnvalue, char *funcname, int opt);
@@ -196,7 +196,7 @@ static int check_retval(void *returnvalue, char *funcname, int opt);
  */
 
 int main(int argc, char *argv[])
-{ 
+{
   void *ida_mem;
   SUNMatrix A;
   SUNLinearSolver LS;
@@ -219,7 +219,7 @@ int main(int argc, char *argv[])
   num_threads = omp_get_max_threads();  /* overwrite with OMP_NUM_THREADS enviroment variable */
 #endif
   if (argc > 1)      /* overwrite with command line value, if supplied */
-    num_threads = strtol(argv[1], NULL, 0);
+    num_threads = (int) strtol(argv[1], NULL, 0);
 
   /* Allocate and initialize user data block webdata. */
 
@@ -240,17 +240,17 @@ int main(int argc, char *argv[])
 
   id  = N_VNew_OpenMP(NEQ, num_threads);
   if(check_retval((void *)id, "N_VNew_OpenMP", 0)) return(1);
-  
+
   SetInitialProfiles(cc, cp, id, webdata);
-  
+
   /* Set remaining inputs to IDAMalloc. */
-  
+
   t0 = ZERO;
-  rtol = RTOL; 
+  rtol = RTOL;
   atol = ATOL;
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
-  
+
   ida_mem = IDACreate();
   if(check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
@@ -281,27 +281,27 @@ int main(int argc, char *argv[])
   tout = RCONST(0.001);
   retval = IDACalcIC(ida_mem, IDA_YA_YDP_INIT, tout);
   if(check_retval(&retval, "IDACalcIC", 1)) return(1);
-  
+
   /* Print heading, basic parameters, and initial values. */
 
   PrintHeader(mu, ml, rtol, atol);
   PrintOutput(ida_mem, cc, ZERO);
-  
+
   /* Loop over iout, call IDASolve (normal mode), print selected output. */
-  
+
   for (iout = 1; iout <= NOUT; iout++) {
-    
+
     retval = IDASolve(ida_mem, tout, &tret, cc, cp, IDA_NORMAL);
     if(check_retval(&retval, "IDASolve", 1)) return(retval);
-    
+
     PrintOutput(ida_mem, cc, tret);
-    
+
     if (iout < 3) tout *= TMULT; else tout += TADD;
-    
+
   }
-  
-  /* Print final statistics and free memory. */  
-  
+
+  /* Print final statistics and free memory. */
+
   PrintFinalStats(ida_mem);
   printf("num_threads = %i\n\n", num_threads);
 
@@ -323,7 +323,7 @@ int main(int argc, char *argv[])
   return(0);
 }
 
-/* Define lines for readability in later routines */ 
+/* Define lines for readability in later routines */
 
 #define acoef  (webdata->acoef)
 #define bcoef  (webdata->bcoef)
@@ -336,29 +336,31 @@ int main(int argc, char *argv[])
  *--------------------------------------------------------------------
  */
 
-/* 
+/*
  * resweb: System residual function for predator-prey system.
  * This routine calls Fweb to get all the right-hand sides of the
  * equations, then loads the residual vector accordingly,
- * using cp in the case of prey species.                 
+ * using cp in the case of prey species.
  */
 
-static int resweb(realtype tt, N_Vector cc, N_Vector cp, 
+static int resweb(realtype tt, N_Vector cc, N_Vector cp,
                   N_Vector res,  void *user_data)
 {
   sunindextype jx, jy, is, yloc, loc, np;
   realtype *resv, *cpv;
   UserData webdata;
-  
+
+  jx = jy = is = 0;
+
   webdata = (UserData)user_data;
-  
-  cpv = NV_DATA_OMP(cp);
+
+  cpv  = NV_DATA_OMP(cp);
   resv = NV_DATA_OMP(res);
-  np = webdata->np;
-  
+  np   = webdata->np;
+
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
-  
+
   /* Loop over all grid points, setting residual values appropriately
      for differential or algebraic components.                        */
 #pragma omp parallel for default(shared) private(jy, yloc, jx, loc, is) schedule(static) num_threads(webdata->nthreads)
@@ -374,9 +376,9 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
       }
     }
   }
-  
+
   return(0);
-  
+
 }
 
 /*
@@ -386,12 +388,12 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
  */
 
 /*
- * InitUserData: Load problem constants in webdata (of type UserData).   
+ * InitUserData: Load problem constants in webdata (of type UserData).
  */
 
 static void InitUserData(UserData webdata)
 {
-  int i, j, np;
+  sunindextype i, j, np;
   realtype *a1,*a2, *a3, *a4, dx2, dy2;
 
   webdata->mx = MX;
@@ -401,11 +403,11 @@ static void InitUserData(UserData webdata)
   webdata->dx = AX/(MX-1);
   webdata->dy = AY/(MY-1);
   webdata->Neq= NEQ;
-  
+
   /* Set up the coefficients a and b, and others found in the equations. */
   np = webdata->np;
   dx2 = (webdata->dx)*(webdata->dx); dy2 = (webdata->dy)*(webdata->dy);
-  
+
   for (i = 0; i < np; i++) {
     a1 = &(acoef[i][np]);
     a2 = &(acoef[i+np][0]);
@@ -418,25 +420,25 @@ static void InitUserData(UserData webdata)
       *a3++ = ZERO;
       *a4++ = ZERO;
     }
-    
+
     /* Reset the diagonal elements of acoef to -AA. */
     acoef[i][i] = -AA; acoef[i+np][i+np] = -AA;
-    
+
     /* Set coefficients for b and diffusion terms. */
     bcoef[i] = BB; bcoef[i+np] = -BB;
     cox[i] = DPREY/dx2; cox[i+np] = DPRED/dx2;
     coy[i] = DPREY/dy2; coy[i+np] = DPRED/dy2;
   }
-  
+
 }
 
-/* 
+/*
  * SetInitialProfiles: Set initial conditions in cc, cp, and id.
  * A polynomial profile is used for the prey cc values, and a constant
  * (1.0e5) is loaded as the initial guess for the predator cc values.
  * The id values are set to 1 for the prey and 0 for the predators.
  * The prey cp values are set according to the given system, and
- * the predator cp values are set to zero.                               
+ * the predator cp values are set to zero.
  */
 
 static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
@@ -445,12 +447,12 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
   sunindextype loc, yloc, is, jx, jy, np;
   realtype xx, yy, xyfactor;
   realtype *ccv, *cpv, *idv;
-  
+
   ccv = NV_DATA_OMP(cc);
   cpv = NV_DATA_OMP(cp);
   idv = NV_DATA_OMP(id);
   np = webdata->np;
-  
+
   /* Loop over grid, load cc values and id values. */
   for (jy = 0; jy < MY; jy++) {
     yy = jy * webdata->dy;
@@ -460,7 +462,7 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       xyfactor = RCONST(16.0)*xx*(ONE-xx)*yy*(ONE-yy);
       xyfactor *= xyfactor;
       loc = yloc + NUM_SPECIES*jx;
-       
+
       for (is = 0; is < NUM_SPECIES; is++) {
         if (is < np) {
           ccv[loc+is] = RCONST(10.0) + (realtype)(is+1) * xyfactor;
@@ -473,10 +475,10 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       }
     }
   }
-  
+
   /* Set c' for the prey by calling the function Fweb. */
   Fweb(ZERO, cc, cp, webdata);
-  
+
   /* Set c' for predators to 0. */
   for (jy = 0; jy < MY; jy++) {
     yloc = NSMX * jy;
@@ -499,9 +501,9 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
   printf("Number of species ns: %d", NUM_SPECIES);
   printf("     Mesh dimensions: %d x %d", MX, MY);
   printf("     System size: %d\n", NEQ);
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  rtol = %Lg   atol = %Lg\n", rtol, atol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #else
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
@@ -513,13 +515,13 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
   printf("  t        bottom-left  top-right");
   printf("    | nst  k      h\n");
   printf("-----------------------------------------------------------\n\n");
-  
+
 }
 
-/* 
+/*
  * PrintOutput: Print output values at output time t = tt.
  * Selected run statistics are printed.  Then values of the concentrations
- * are printed for the bottom left and top right grid points only.  
+ * are printed for the bottom left and top right grid points only.
  */
 
 static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
@@ -534,22 +536,22 @@ static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
   check_retval(&retval, "IDAGetNumSteps", 1);
   retval = IDAGetLastStep(ida_mem, &hused);
   check_retval(&retval, "IDAGetLastStep", 1);
-  
+
   c_bl = IJ_Vptr(c,0,0);
   c_tr = IJ_Vptr(c,MX-1,MY-1);
 
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
-  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n", 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4Le %12.4Le   |\n",c_bl[i],c_tr[i]);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
 #else
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
@@ -558,12 +560,12 @@ static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
   printf("\n");
 }
 
-/* 
- * PrintFinalStats: Print final run data contained in iopt.              
+/*
+ * PrintFinalStats: Print final run data contained in iopt.
  */
 
 static void PrintFinalStats(void *ida_mem)
-{ 
+{
   long int nst, nre, nreLS, nni, nje, netf, ncfn;
   int retval;
 
@@ -593,27 +595,29 @@ static void PrintFinalStats(void *ida_mem)
 
 }
 
-/* 
- * Fweb: Rate function for the food-web problem.                        
- * This routine computes the right-hand sides of the system equations,   
- * consisting of the diffusion term and interaction term.                
- * The interaction term is computed by the function WebRates.            
+/*
+ * Fweb: Rate function for the food-web problem.
+ * This routine computes the right-hand sides of the system equations,
+ * consisting of the diffusion term and interaction term.
+ * The interaction term is computed by the function WebRates.
  */
 
-static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,  
+static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
                  UserData webdata)
-{ 
+{
   sunindextype jx, jy, is, idyu, idyl, idxu, idxl;
   realtype xx, yy, *cxy, *ratesxy, *cratexy, dcyli, dcyui, dcxli, dcxui;
-  
+
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
-  
+
+  jx = jy = is = 0;
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
     idyl = (jy!= 0  ) ? NSMX : -NSMX;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = (webdata->dx) * jx;
       idxu = (jx!= MX-1) ?  NUM_SPECIES : -NUM_SPECIES;
@@ -621,33 +625,33 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
       cxy = IJ_Vptr(cc,jx,jy);
       ratesxy = IJ_Vptr(webdata->rates,jx,jy);
       cratexy = IJ_Vptr(crate,jx,jy);
-      
+
       /* Get interaction vector at this grid point. */
       WebRates(xx, yy, cxy, ratesxy, webdata);
-      
+
       /* Loop over species, do differencing, load crate segment. */
 #pragma omp parallel for default(shared) private(is, dcyli, dcyui, dcxli, dcxui) schedule(static) num_threads(webdata->nthreads)
       for (is = 0; is < NUM_SPECIES; is++) {
-        
+
         /* Differencing in y. */
         dcyli = *(cxy+is) - *(cxy - idyl + is) ;
         dcyui = *(cxy + idyu + is) - *(cxy+is);
-        
+
         /* Differencing in x. */
         dcxli = *(cxy+is) - *(cxy - idxl + is);
         dcxui = *(cxy + idxu +is) - *(cxy+is);
-        
+
         /* Compute the crate values at (xx,yy). */
         cratexy[is] = coy[is] * (dcyui - dcyli) +
           cox[is] * (dcxui - dcxli) + ratesxy[is];
-        
+
       } /* End is loop */
     } /* End of jx loop */
   } /* End of jy loop */
-  
+
 }
 
-/* 
+/*
  * WebRates: Evaluate reaction rates at a given spatial point.
  * At a given (x,y), evaluate the array of ns reaction terms R.
  */
@@ -657,30 +661,30 @@ static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
 {
   int is;
   realtype fac;
-  
+
   for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = dotprod(NUM_SPECIES, cxy, acoef[is]);
-  
+
   fac = ONE + ALPHA*xx*yy + BETA*sin(FOURPI*xx)*sin(FOURPI*yy);
-   
-  for (is = 0; is < NUM_SPECIES; is++)  
+
+  for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = cxy[is]*( bcoef[is]*fac + ratesxy[is] );
-  
+
 }
 
 /*
- * dotprod: dot product routine for realtype arrays, for use by WebRates.    
+ * dotprod: dot product routine for realtype arrays, for use by WebRates.
  */
 
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2)
 {
   sunindextype i;
   realtype *xx1, *xx2, temp = ZERO;
-  
+
   xx1 = x1; xx2 = x2;
   for (i = 0; i < size; i++) temp += (*xx1++) * (*xx2++);
   return(temp);
-  
+
 }
 
 /*
@@ -690,7 +694,7 @@ static realtype dotprod(sunindextype size, realtype *x1, realtype *x2)
  *   opt == 1 means SUNDIALS function returns an integer value so check if
  *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, char *funcname, int opt)
@@ -699,25 +703,25 @@ static int check_retval(void *returnvalue, char *funcname, int opt)
 
   if (opt == 0 && returnvalue == NULL) {
     /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   } else if (opt == 1) {
     /* Check if retval < 0 */
     retval = (int *) returnvalue;
     if (*retval < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n", 
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
               funcname, *retval);
-      return(1); 
+      return(1);
     }
   } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
-    return(1); 
+    return(1);
   }
 
   return(0);
