@@ -3,7 +3,7 @@
  * Programmer(s): Daniel R. Reynolds and Ting Yan @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -12,10 +12,10 @@
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * Example program for IDAS: Food web problem, OpenMP, GMRES, 
+ * Example program for IDAS: Food web problem, OpenMP, GMRES,
  * user-supplied preconditioner
  *
- * This example program uses SUNLinSol_SPGMR as the linear 
+ * This example program uses SUNLinSol_SPGMR as the linear
  * solver, and IDACalcIC for initial condition calculation.
  *
  * The mathematical problem solved in this example is a DAE system
@@ -72,19 +72,19 @@
  * The DAE system is solved by IDAS using the SUNLinSol_SPGMR linear solver.
  * Output is printed at t = 0, .001, .01, .1, .4, .7, 1.
  *
- * Optionally, we can set the number of threads from environment 
+ * Optionally, we can set the number of threads from environment
  * variable or command line. To check the current value for number
  * of threads from environment:
  *      % echo $OMP_NUM_THREADS
  *
  * Execution:
  *
- * To use the default value for the number of threads from 
+ * To use the default value for the number of threads from
  * the OMP_NUM_THREADS environment value:
- *      % ./idasFoodWeb_kry_omp 
+ *      % ./idasFoodWeb_kry_omp
  * To specify the number of threads at the command line, use
  *      % ./idasFoodWeb_kry_omp num_threads
- * where num_threads is the desired number of threads. 
+ * where num_threads is the desired number of threads.
  *
  * -----------------------------------------------------------------
  * References:
@@ -114,12 +114,17 @@
 #include <nvector/nvector_openmp.h>
 #include <sundials/sundials_dense.h>
 #include <sundials/sundials_types.h>
-#include <sundials/sundials_math.h>
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
 /* Problem Constants. */
 
 #define NPREY       1              /* No. of prey (= no. of predators). */
@@ -147,15 +152,15 @@
 #define NOUT        6              /* Number of output times            */
 #define TMULT       RCONST(10.0)   /* Multiplier for tout values        */
 #define TADD        RCONST(0.3)    /* Increment for tout values         */
-#define ZERO        RCONST(0.)     
-#define ONE         RCONST(1.0)    
+#define ZERO        RCONST(0.)
+#define ONE         RCONST(1.0)
 
-/* 
+/*
  * User-defined vector and accessor macro: IJ_Vptr.
- * IJ_Vptr is defined in order to express the underlying 3-D structure of 
+ * IJ_Vptr is defined in order to express the underlying 3-D structure of
  * the dependent variable vector from its underlying 1-D storage (an N_Vector).
- * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
- * species index is = 0, x-index ix = i, and y-index jy = j.                
+ * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
+ * species index is = 0, x-index ix = i, and y-index jy = j.
  */
 
 #define IJ_Vptr(vv,i,j) (&NV_Ith_OMP(vv, (i)*NUM_SPECIES + (j)*NSMX))
@@ -176,7 +181,7 @@ typedef struct {
 
 /* Prototypes for functions called by the IDA Solver. */
 
-static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval, 
+static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval,
                   void *user_data);
 
 static int Precond(realtype tt, N_Vector cc, N_Vector cp,
@@ -191,11 +196,11 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
 static void InitUserData(UserData webdata);
 static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
                                UserData webdata);
-static void PrintHeader(sunindextype maxl, realtype rtol, realtype atol);
+static void PrintHeader(int maxl, realtype rtol, realtype atol);
 static void PrintOutput(void *ida_mem, N_Vector c, realtype t);
 static void PrintFinalStats(void *ida_mem);
 static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate, UserData webdata);
-static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                      UserData webdata);
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2);
 static int check_retval(void *returnvalue, char *funcname, int opt);
@@ -207,13 +212,13 @@ static int check_retval(void *returnvalue, char *funcname, int opt);
  */
 
 int main(int argc, char *argv[])
-{ 
+{
   void *ida_mem;
   SUNLinearSolver LS;
   UserData webdata;
   N_Vector cc, cp, id;
   int iout, jx, jy, retval;
-  sunindextype maxl;
+  int maxl;
   realtype rtol, atol, t0, tout, tret;
   int num_threads;
 
@@ -228,7 +233,7 @@ int main(int argc, char *argv[])
   num_threads = omp_get_max_threads();    /* overwrite with OMP_NUM_THREADS */
 #endif
   if (argc > 1)
-    num_threads = strtol(argv[1], NULL, 0);
+    num_threads = (int) strtol(argv[1], NULL, 0);
 
   /* Allocate and initialize user data block webdata. */
 
@@ -256,17 +261,17 @@ int main(int argc, char *argv[])
 
   id  = N_VNew_OpenMP(NEQ, num_threads);
   if(check_retval((void *)id, "N_VNew_OpenMP", 0)) return(1);
-  
+
   SetInitialProfiles(cc, cp, id, webdata);
-  
+
   /* Set remaining inputs to IDAMalloc. */
-  
+
   t0 = ZERO;
-  rtol = RTOL; 
+  rtol = RTOL;
   atol = ATOL;
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
-  
+
   ida_mem = IDACreate();
   if(check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
@@ -284,7 +289,7 @@ int main(int argc, char *argv[])
 
   webdata->ida_mem = ida_mem;
 
-  /* Create SUNLinSol_SPGMR linear solver, attach to IDA, and set 
+  /* Create SUNLinSol_SPGMR linear solver, attach to IDA, and set
      preconditioning routines. */
 
   maxl = 16;                                      /* max dimension of the Krylov subspace */
@@ -302,27 +307,27 @@ int main(int argc, char *argv[])
   tout = RCONST(0.001);
   retval = IDACalcIC(ida_mem, IDA_YA_YDP_INIT, tout);
   if(check_retval(&retval, "IDACalcIC", 1)) return(1);
-  
+
   /* Print heading, basic parameters, and initial values. */
 
   PrintHeader(maxl, rtol, atol);
   PrintOutput(ida_mem, cc, ZERO);
-  
+
   /* Loop over iout, call IDASolve (normal mode), print selected output. */
-  
+
   for (iout = 1; iout <= NOUT; iout++) {
-    
+
     retval = IDASolve(ida_mem, tout, &tret, cc, cp, IDA_NORMAL);
     if(check_retval(&retval, "IDASolve", 1)) return(retval);
-    
+
     PrintOutput(ida_mem, cc, tret);
-    
+
     if (iout < 3) tout *= TMULT; else tout += TADD;
-    
+
   }
-  
-  /* Print final statistics and free memory. */  
-  
+
+  /* Print final statistics and free memory. */
+
   PrintFinalStats(ida_mem);
   printf("num_threads = %i\n\n", num_threads);
 
@@ -331,14 +336,14 @@ int main(int argc, char *argv[])
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
 
-  N_VDestroy_OpenMP(cc);
-  N_VDestroy_OpenMP(cp);
-  N_VDestroy_OpenMP(id);
+  N_VDestroy(cc);
+  N_VDestroy(cp);
+  N_VDestroy(id);
 
 
   destroyMat(webdata->acoef);
-  N_VDestroy_OpenMP(webdata->rates);
-  N_VDestroy_OpenMP(webdata->ewt);
+  N_VDestroy(webdata->rates);
+  N_VDestroy(webdata->ewt);
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy < MY; jy ++) {
       destroyArray((webdata->pivot)[jx][jy]);
@@ -350,7 +355,7 @@ int main(int argc, char *argv[])
   return(0);
 }
 
-/* Define lines for readability in later routines */ 
+/* Define lines for readability in later routines */
 
 #define acoef  (webdata->acoef)
 #define bcoef  (webdata->bcoef)
@@ -363,32 +368,34 @@ int main(int argc, char *argv[])
  *--------------------------------------------------------------------
  */
 
-/* 
+/*
  * resweb: System residual function for predator-prey system.
  * This routine calls Fweb to get all the right-hand sides of the
  * equations, then loads the residual vector accordingly,
- * using cp in the case of prey species.                 
+ * using cp in the case of prey species.
  */
 
-static int resweb(realtype tt, N_Vector cc, N_Vector cp, 
+static int resweb(realtype tt, N_Vector cc, N_Vector cp,
                   N_Vector res,  void *user_data)
 {
   sunindextype jx, jy, is, yloc, loc, np;
   realtype *resv, *cpv;
   UserData webdata;
-  
+
+  jx = jy = is = 0;
+
   webdata = (UserData)user_data;
-  
-  cpv = NV_DATA_OMP(cp);
+
+  cpv  = NV_DATA_OMP(cp);
   resv = NV_DATA_OMP(res);
-  np = webdata->np;
-  
+  np   = webdata->np;
+
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
-  
+
   /* Loop over all grid points, setting residual values appropriately
      for differential or algebraic components.                        */
-#pragma omp parallel for default(shared) private(jy, jx, is, yloc, loc) schedule(static) num_threads(webdata->nthreads)  
+#pragma omp parallel for default(shared) private(jy, jx, is, yloc, loc) schedule(static) num_threads(webdata->nthreads)
   for (jy = 0; jy < MY; jy++) {
     yloc = NSMX * jy;
     for (jx = 0; jx < MX; jx++) {
@@ -401,9 +408,9 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
       }
     }
   }
-  
+
   return(0);
-  
+
 }
 
 
@@ -411,10 +418,11 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
                    N_Vector rr, realtype cj, void *user_data)
 {
   int retval;
+  sunindextype ret;
   realtype uround, xx, yy, del_x, del_y;
   realtype **Pxy, *ratesxy, *Pxycol, *cxy, *cpxy, *ewtxy, cctmp;
   realtype inc, fac, sqru, perturb_rates[NUM_SPECIES];
-  int is, js, jx, jy, ret;
+  int is, js, jx, jy;
   void *ida_mem;
   N_Vector ewt;
   realtype hh;
@@ -425,7 +433,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
   del_y = webdata->dy;
 
   uround = UNIT_ROUNDOFF;
-  sqru = SUNRsqrt(uround);
+  sqru = sqrt(uround);
 
   ida_mem = webdata->ida_mem;
   ewt = webdata->ewt;
@@ -436,7 +444,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
 
   for (jy = 0; jy < MY; jy++) {
     yy = jy * del_y;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = jx * del_x;
       Pxy = (webdata->PP)[jx][jy];
@@ -446,7 +454,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
       ratesxy = IJ_Vptr((webdata->rates), jx, jy);
 
       for (js = 0; js < NUM_SPECIES; js++) {
-	inc = sqru*(SUNMAX(SUNRabs(cxy[js]), SUNMAX(hh*SUNRabs(cpxy[js]), ONE/ewtxy[js])));
+	inc = sqru*(MAX(fabs(cxy[js]), MAX(hh*fabs(cpxy[js]), ONE/ewtxy[js])));
 	cctmp = cxy[js];
 	cxy[js] += inc;
 	fac = -ONE/inc;
@@ -454,12 +462,12 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
 	WebRates(xx, yy, cxy, perturb_rates, webdata);
 
 	Pxycol = Pxy[js];
-	
+
 	for (is = 0; is < NUM_SPECIES; is++)
 	  Pxycol[is] = (perturb_rates[is] - ratesxy[is])*fac;
 
 	if (js < 1) Pxycol[js] += cj;
-	
+
 	cxy[js] = cctmp;
       }
 
@@ -476,13 +484,15 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
 
 static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
                   N_Vector rr, N_Vector rvec, N_Vector zvec,
-		  realtype cj, realtype dalta, void *user_data) 
+		  realtype cj, realtype dalta, void *user_data)
 {
   realtype **Pxy, *zxy;
   sunindextype *pivot;
-  int jx, jy;
+  sunindextype jx, jy;
   UserData webdata;
-  
+
+  jx = jy = 0;
+
   webdata = (UserData) user_data;
 
   N_VScale(ONE, rvec, zvec);
@@ -490,14 +500,14 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
 #pragma omp parallel for collapse(2) default(shared) private(jx, jy, zxy, Pxy, pivot) schedule(static) num_threads(webdata->nthreads)
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy <MY; jy++) {
-      
+
       zxy = IJ_Vptr(zvec, jx, jy);
       Pxy = (webdata->PP)[jx][jy];
       pivot = (webdata->pivot)[jx][jy];
       denseGETRS(Pxy, NUM_SPECIES, pivot, zxy);
     }
   }
-  
+
   return(0);
 
 }
@@ -510,12 +520,12 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
  */
 
 /*
- * InitUserData: Load problem constants in webdata (of type UserData).   
+ * InitUserData: Load problem constants in webdata (of type UserData).
  */
 
 static void InitUserData(UserData webdata)
 {
-  int i, j, np;
+  sunindextype i, j, np;
   realtype *a1,*a2, *a3, *a4, dx2, dy2;
 
   webdata->mx = MX;
@@ -525,11 +535,11 @@ static void InitUserData(UserData webdata)
   webdata->dx = AX/(MX-1);
   webdata->dy = AY/(MY-1);
   webdata->Neq= NEQ;
-  
+
   /* Set up the coefficients a and b, and others found in the equations. */
   np = webdata->np;
   dx2 = (webdata->dx)*(webdata->dx); dy2 = (webdata->dy)*(webdata->dy);
-  
+
   for (i = 0; i < np; i++) {
     a1 = &(acoef[i][np]);
     a2 = &(acoef[i+np][0]);
@@ -542,25 +552,25 @@ static void InitUserData(UserData webdata)
       *a3++ = ZERO;
       *a4++ = ZERO;
     }
-    
+
     /* Reset the diagonal elements of acoef to -AA. */
     acoef[i][i] = -AA; acoef[i+np][i+np] = -AA;
-    
+
     /* Set coefficients for b and diffusion terms. */
     bcoef[i] = BB; bcoef[i+np] = -BB;
     cox[i] = DPREY/dx2; cox[i+np] = DPRED/dx2;
     coy[i] = DPREY/dy2; coy[i+np] = DPRED/dy2;
   }
-  
+
 }
 
-/* 
+/*
  * SetInitialProfiles: Set initial conditions in cc, cp, and id.
  * A polynomial profile is used for the prey cc values, and a constant
  * (1.0e5) is loaded as the initial guess for the predator cc values.
  * The id values are set to 1 for the prey and 0 for the predators.
  * The prey cp values are set according to the given system, and
- * the predator cp values are set to zero.                               
+ * the predator cp values are set to zero.
  */
 
 static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
@@ -569,12 +579,12 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
   sunindextype loc, yloc, is, jx, jy, np;
   realtype xx, yy, xyfactor;
   realtype *ccv, *cpv, *idv;
-  
+
   ccv = NV_DATA_OMP(cc);
   cpv = NV_DATA_OMP(cp);
   idv = NV_DATA_OMP(id);
   np = webdata->np;
-  
+
   /* Loop over grid, load cc values and id values. */
   for (jy = 0; jy < MY; jy++) {
     yy = jy * webdata->dy;
@@ -584,7 +594,7 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       xyfactor = RCONST(16.0)*xx*(ONE-xx)*yy*(ONE-yy);
       xyfactor *= xyfactor;
       loc = yloc + NUM_SPECIES*jx;
-       
+
       for (is = 0; is < NUM_SPECIES; is++) {
         if (is < np) {
           ccv[loc+is] = RCONST(10.0) + (realtype)(is+1) * xyfactor;
@@ -597,10 +607,10 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       }
     }
   }
-  
+
   /* Set c' for the prey by calling the function Fweb. */
   Fweb(ZERO, cc, cp, webdata);
-  
+
   /* Set c' for predators to 0. */
   for (jy = 0; jy < MY; jy++) {
     yloc = NSMX * jy;
@@ -617,32 +627,32 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
  * Print first lines of output (problem description)
  */
 
-static void PrintHeader(sunindextype maxl, realtype rtol, realtype atol)
+static void PrintHeader(int maxl, realtype rtol, realtype atol)
 {
   printf("\nidasFoodWeb_kry_omp: Predator-prey DAE OpenMP example problem using Krylov solver for IDAS \n\n");
   printf("Number of species ns: %d", NUM_SPECIES);
   printf("     Mesh dimensions: %d x %d", MX, MY);
   printf("     System size: %d\n", NEQ);
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  rtol = %Lg   atol = %Lg\n", rtol, atol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #else
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #endif
-  printf("Linear solver: SUNLinSol_SPGMR, maxl = %ld\n",(long int) maxl);
+  printf("Linear solver: SUNLinSol_SPGMR, maxl = %d\n", maxl);
   printf("CalcIC called to correct initial predator concentrations.\n\n");
   printf("-----------------------------------------------------------\n");
   printf("  t        bottom-left  top-right");
   printf("    | nst  k      h\n");
   printf("-----------------------------------------------------------\n\n");
-  
+
 }
 
-/* 
+/*
  * PrintOutput: Print output values at output time t = tt.
  * Selected run statistics are printed.  Then values of the concentrations
- * are printed for the bottom left and top right grid points only.  
+ * are printed for the bottom left and top right grid points only.
  */
 
 static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
@@ -657,22 +667,22 @@ static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
   check_retval(&retval, "IDAGetNumSteps", 1);
   retval = IDAGetLastStep(ida_mem, &hused);
   check_retval(&retval, "IDAGetLastStep", 1);
-  
+
   c_bl = IJ_Vptr(c,0,0);
   c_tr = IJ_Vptr(c,MX-1,MY-1);
 
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
-  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n", 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4Le %12.4Le   |\n",c_bl[i],c_tr[i]);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
 #else
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
@@ -681,12 +691,12 @@ static void PrintOutput(void *ida_mem, N_Vector c, realtype t)
   printf("\n");
 }
 
-/* 
- * PrintFinalStats: Print final run data contained in iopt.              
+/*
+ * PrintFinalStats: Print final run data contained in iopt.
  */
 
 static void PrintFinalStats(void *ida_mem)
-{ 
+{
   long int nst, nre, sli, netf, nps, npevals, nrevalsLS;
   int retval;
 
@@ -716,27 +726,29 @@ static void PrintFinalStats(void *ida_mem)
 
 }
 
-/* 
- * Fweb: Rate function for the food-web problem.                        
- * This routine computes the right-hand sides of the system equations,   
- * consisting of the diffusion term and interaction term.                
- * The interaction term is computed by the function WebRates.            
+/*
+ * Fweb: Rate function for the food-web problem.
+ * This routine computes the right-hand sides of the system equations,
+ * consisting of the diffusion term and interaction term.
+ * The interaction term is computed by the function WebRates.
  */
 
-static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,  
+static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
                  UserData webdata)
-{ 
+{
   sunindextype jx, jy, is, idyu, idyl, idxu, idxl;
   realtype xx, yy, *cxy, *ratesxy, *cratexy, dcyli, dcyui, dcxli, dcxui;
-  
+
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
-  
+
+  jx = jy = is = 0;
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
     idyl = (jy!= 0  ) ? NSMX : -NSMX;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = (webdata->dx) * jx;
       idxu = (jx!= MX-1) ?  NUM_SPECIES : -NUM_SPECIES;
@@ -744,33 +756,33 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
       cxy = IJ_Vptr(cc,jx,jy);
       ratesxy = IJ_Vptr(webdata->rates,jx,jy);
       cratexy = IJ_Vptr(crate,jx,jy);
-      
+
       /* Get interaction vector at this grid point. */
       WebRates(xx, yy, cxy, ratesxy, webdata);
-      
+
       /* Loop over species, do differencing, load crate segment. */
 #pragma omp parallel for default(shared) private(is, dcyli, dcyui, dcxli, dcxui) schedule(static) num_threads(webdata->nthreads)
       for (is = 0; is < NUM_SPECIES; is++) {
-        
+
         /* Differencing in y. */
         dcyli = *(cxy+is) - *(cxy - idyl + is) ;
         dcyui = *(cxy + idyu + is) - *(cxy+is);
-        
+
         /* Differencing in x. */
         dcxli = *(cxy+is) - *(cxy - idxl + is);
         dcxui = *(cxy + idxu +is) - *(cxy+is);
-        
+
         /* Compute the crate values at (xx,yy). */
         cratexy[is] = coy[is] * (dcyui - dcyli) +
           cox[is] * (dcxui - dcxli) + ratesxy[is];
-        
+
       } /* End is loop */
     } /* End of jx loop */
   } /* End of jy loop */
-  
+
 }
 
-/* 
+/*
  * WebRates: Evaluate reaction rates at a given spatial point.
  * At a given (x,y), evaluate the array of ns reaction terms R.
  */
@@ -780,30 +792,30 @@ static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
 {
   int is;
   realtype fac;
-  
+
   for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = dotprod(NUM_SPECIES, cxy, acoef[is]);
-  
+
   fac = ONE + ALPHA*xx*yy + BETA*sin(FOURPI*xx)*sin(FOURPI*yy);
-  
-  for (is = 0; is < NUM_SPECIES; is++)  
+
+  for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = cxy[is]*( bcoef[is]*fac + ratesxy[is] );
-  
+
 }
 
 /*
- * dotprod: dot product routine for realtype arrays, for use by WebRates.    
+ * dotprod: dot product routine for realtype arrays, for use by WebRates.
  */
 
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2)
 {
   sunindextype i;
   realtype *xx1, *xx2, temp = ZERO;
-  
+
   xx1 = x1; xx2 = x2;
   for (i = 0; i < size; i++) temp += (*xx1++) * (*xx2++);
   return(temp);
-  
+
 }
 
 /*
@@ -813,7 +825,7 @@ static realtype dotprod(sunindextype size, realtype *x1, realtype *x2)
  *   opt == 1 means SUNDIALS function returns an integer value so check if
  *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, char *funcname, int opt)
@@ -822,25 +834,25 @@ static int check_retval(void *returnvalue, char *funcname, int opt)
 
   if (opt == 0 && returnvalue == NULL) {
     /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   } else if (opt == 1) {
     /* Check if retval < 0 */
     retval = (int *) returnvalue;
     if (*retval < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n", 
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
               funcname, *retval);
-      return(1); 
+      return(1);
     }
   } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
-    return(1); 
+    return(1);
   }
 
   return(0);
