@@ -2,6 +2,16 @@
  * Programmer(s): Allan Taylor, Alan Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  *
  * This example loops through the available iterative linear solvers:
  * SPGMR, SPBCG and SPTFQMR.
@@ -34,13 +44,24 @@
 #include <math.h>
 
 #include <idas/idas.h>                   /* main integrator header file       */
-#include <idas/idas_spils.h>             /* access to IDASpils interface      */
 #include <sunlinsol/sunlinsol_spgmr.h>   /* access to SPGMR SUNLinearSolver   */
 #include <sunlinsol/sunlinsol_spbcgs.h>  /* access to SPBCGS SUNLinearSolver  */
 #include <sunlinsol/sunlinsol_sptfqmr.h> /* access to SPTFQMR SUNLinearSolver */
 #include <nvector/nvector_serial.h>      /* serial N_Vector types, fct. and macros */
 #include <sundials/sundials_types.h>     /* definition of realtype */
 
+/* helpful macros */
+
+#ifndef SQRT
+#if defined(SUNDIALS_DOUBLE_PRECISION)
+#define SQRT(x) (sqrt((x)))
+#elif defined(SUNDIALS_SINGLE_PRECISION)
+#define SQRT(x) (sqrtf((x)))
+#elif defined(SUNDIALS_EXTENDED_PRECISION)
+#define SQRT(x) (sqrtl((x)))
+#endif
+#endif
+
 /* Problem Constants */
 
 #define NOUT  11
@@ -59,7 +80,7 @@
 
 /* User data type */
 
-typedef struct {  
+typedef struct {
   sunindextype mm;  /* number of grid points */
   realtype dx;
   realtype coeff;
@@ -71,22 +92,22 @@ typedef struct {
 int resHeat(realtype tres, N_Vector uu, N_Vector up,
             N_Vector resval, void *user_data);
 
-int PsetupHeat(realtype tt, 
-               N_Vector uu, N_Vector up, N_Vector rr, 
+int PsetupHeat(realtype tt,
+               N_Vector uu, N_Vector up, N_Vector rr,
                realtype c_j, void *user_data);
 
-int PsolveHeat(realtype tt, 
-               N_Vector uu, N_Vector up, N_Vector rr, 
-               N_Vector rvec, N_Vector zvec, 
+int PsolveHeat(realtype tt,
+               N_Vector uu, N_Vector up, N_Vector rr,
+               N_Vector rvec, N_Vector zvec,
                realtype c_j, realtype delta, void *user_data);
 
 /* Prototypes for private functions */
 
-static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up, 
+static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
                              N_Vector res);
 static void PrintHeader(realtype rtol, realtype atol, int linsolver);
 static void PrintOutput(void *mem, realtype t, N_Vector uu, int linsolver);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -94,38 +115,47 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
  *--------------------------------------------------------------------
  */
 
-int main(void)
+int main(int argc, char* argv[])
 {
   void *mem;
   UserData data;
   N_Vector uu, up, constraints, res;
-  int ier, iout, linsolver;
+  int retval, iout, linsolver;
   realtype rtol, atol, t0, t1, tout, tret;
   long int netf, ncfn, ncfl;
   SUNLinearSolver LS;
+  int nrmfactor;   /* LS norm conversion factor flag */
+  realtype nrmfac; /* LS norm conversion factor      */
 
-  mem = NULL;
-  data = NULL;
-  uu = up = constraints = res = NULL;
-  LS = NULL;
+  mem         = NULL;
+  data        = NULL;
+  uu          = NULL;
+  up          = NULL;
+  constraints = NULL;
+  res         = NULL;
+  LS          = NULL;
+  nrmfactor   = 0;
+
+  /* Retrieve the command-line options */
+  if (argc > 1) nrmfactor = atoi(argv[1]);
 
   /* Allocate N-vectors and the user data structure. */
 
   uu = N_VNew_Serial(NEQ);
-  if(check_flag((void *)uu, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)uu, "N_VNew_Serial", 0)) return(1);
 
   up = N_VNew_Serial(NEQ);
-  if(check_flag((void *)up, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)up, "N_VNew_Serial", 0)) return(1);
 
   res = N_VNew_Serial(NEQ);
-  if(check_flag((void *)res, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)res, "N_VNew_Serial", 0)) return(1);
 
   constraints = N_VNew_Serial(NEQ);
-  if(check_flag((void *)constraints, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)constraints, "N_VNew_Serial", 0)) return(1);
 
   data = (UserData) malloc(sizeof *data);
   data->pp = NULL;
-  if(check_flag((void *)data, "malloc", 2)) return(1);
+  if(check_retval((void *)data, "malloc", 2)) return(1);
 
   /* Assign parameters in the user data structure. */
 
@@ -133,7 +163,7 @@ int main(void)
   data->dx = ONE/(MGRID-ONE);
   data->coeff = ONE/(data->dx * data->dx);
   data->pp = N_VNew_Serial(NEQ);
-  if(check_flag((void *)data->pp, "N_VNew_Serial", 0)) return(1);
+  if(check_retval((void *)data->pp, "N_VNew_Serial", 0)) return(1);
 
   /* Initialize uu, up. */
 
@@ -148,25 +178,25 @@ int main(void)
   t0   = ZERO;
   t1   = RCONST(0.01);
   rtol = ZERO;
-  atol = RCONST(1.0e-3); 
+  atol = RCONST(1.0e-3);
 
   /* Call IDACreate and IDAMalloc to initialize solution */
 
   mem = IDACreate();
-  if(check_flag((void *)mem, "IDACreate", 0)) return(1);
+  if(check_retval((void *)mem, "IDACreate", 0)) return(1);
 
-  ier = IDASetUserData(mem, data);
-  if(check_flag(&ier, "IDASetUserData", 1)) return(1);
+  retval = IDASetUserData(mem, data);
+  if(check_retval(&retval, "IDASetUserData", 1)) return(1);
 
-  ier = IDASetConstraints(mem, constraints);
-  if(check_flag(&ier, "IDASetConstraints", 1)) return(1);
+  retval = IDASetConstraints(mem, constraints);
+  if(check_retval(&retval, "IDASetConstraints", 1)) return(1);
   N_VDestroy(constraints);
 
-  ier = IDAInit(mem, resHeat, t0, uu, up);
-  if(check_flag(&ier, "IDAInit", 1)) return(1);
+  retval = IDAInit(mem, resHeat, t0, uu, up);
+  if(check_retval(&retval, "IDAInit", 1)) return(1);
 
-  ier = IDASStolerances(mem, rtol, atol);
-  if(check_flag(&ier, "IDASStolerances", 1)) return(1);
+  retval = IDASStolerances(mem, rtol, atol);
+  if(check_retval(&retval, "IDASStolerances", 1)) return(1);
 
   /* START: Loop through SPGMR, SPBCG and SPTFQMR linear solver modules */
   for (linsolver = 0; linsolver < 3; ++linsolver) {
@@ -177,8 +207,8 @@ int main(void)
       SetInitialProfile(data, uu, up, res);
 
       /* Re-initialize IDA */
-      ier = IDAReInit(mem, t0, uu, up);
-      if (check_flag(&ier, "IDAReInit", 1)) return(1);
+      retval = IDAReInit(mem, t0, uu, up);
+      if (check_retval(&retval, "IDAReInit", 1)) return(1);
 
     }
 
@@ -195,14 +225,14 @@ int main(void)
       printf(" \n| SPGMR |\n");
       printf(" -------\n");
 
-      /* Call SUNSPGMR to specify the linear solver SPGMR with
+      /* Call SUNLinSol_SPGMR to specify the linear solver SPGMR with
          left preconditioning and the default maximum Krylov dimension */
-      LS = SUNSPGMR(uu, PREC_LEFT, 0);
-      if(check_flag((void *)LS, "SUNSPGMR", 0)) return(1);
+      LS = SUNLinSol_SPGMR(uu, PREC_LEFT, 0);
+      if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
       /* Attach the linear solver */
-      ier = IDASpilsSetLinearSolver(mem, LS);
-      if(check_flag(&ier, "IDASpilsSetLinearSolver", 1)) return 1;
+      retval = IDASetLinearSolver(mem, LS, NULL);
+      if(check_retval(&retval, "IDASetLinearSolver", 1)) return 1;
 
       break;
 
@@ -214,14 +244,14 @@ int main(void)
       printf(" \n| SPBCGS |\n");
       printf(" -------\n");
 
-      /* Call SUNSPBCGS to specify the linear solver SPBCGS with
+      /* Call SUNLinSol_SPBCGS to specify the linear solver SPBCGS with
          left preconditioning and the default maximum Krylov dimension */
-      LS = SUNSPBCGS(uu, PREC_LEFT, 0);
-      if(check_flag((void *)LS, "SUNSPBCGS", 0)) return(1);
+      LS = SUNLinSol_SPBCGS(uu, PREC_LEFT, 0);
+      if(check_retval((void *)LS, "SUNLinSol_SPBCGS", 0)) return(1);
 
       /* Attach the linear solver */
-      ier = IDASpilsSetLinearSolver(mem, LS);
-      if(check_flag(&ier, "IDASpilsSetLinearSolver", 1)) return 1;
+      retval = IDASetLinearSolver(mem, LS, NULL);
+      if(check_retval(&retval, "IDASetLinearSolver", 1)) return 1;
 
       break;
 
@@ -233,22 +263,42 @@ int main(void)
       printf(" \n| SPTFQMR |\n");
       printf(" ---------\n");
 
-      /* Call SUNSPTFQMR to specify the linear solver SPTFQMR with
+      /* Call SUNLinSol_SPTFQMR to specify the linear solver SPTFQMR with
          left preconditioning and the default maximum Krylov dimension */
-      LS = SUNSPTFQMR(uu, PREC_LEFT, 0);
-      if(check_flag((void *)LS, "SUNSPTFQMR", 0)) return(1);
+      LS = SUNLinSol_SPTFQMR(uu, PREC_LEFT, 0);
+      if(check_retval((void *)LS, "SUNLinSol_SPTFQMR", 0)) return(1);
 
       /* Attach the linear solver */
-      ier = IDASpilsSetLinearSolver(mem, LS);
-      if(check_flag(&ier, "IDASpilsSetLinearSolver", 1)) return 1;
+      retval = IDASetLinearSolver(mem, LS, NULL);
+      if(check_retval(&retval, "IDASetLinearSolver", 1)) return 1;
 
       break;
 
     }
 
     /* Specify preconditioner */
-    ier = IDASpilsSetPreconditioner(mem, PsetupHeat, PsolveHeat);
-    if(check_flag(&ier, "IDASpilsSetPreconditioner", 1)) return(1);
+    retval = IDASetPreconditioner(mem, PsetupHeat, PsolveHeat);
+    if(check_retval(&retval, "IDASetPreconditioner", 1)) return(1);
+
+        /* Set the linear solver tolerance conversion factor */
+    switch(nrmfactor) {
+
+    case(1):
+      /* use the square root of the vector length */
+      nrmfac = SQRT((realtype)NEQ);
+      break;
+    case(2):
+      /* compute with dot product */
+      nrmfac = -ONE;
+      break;
+    default:
+      /* use the default */
+      nrmfac = ZERO;
+      break;
+    }
+
+    retval = IDASetLSNormFactor(mem, nrmfac);
+    if (check_retval(&retval, "IDASetLSNormFactor", 1)) return(1);
 
     /* Print output heading. */
     PrintHeader(rtol, atol, linsolver);
@@ -262,20 +312,20 @@ int main(void)
     /* Loop over output times, call IDASolve, and print results. */
 
     for (tout = t1,iout = 1; iout <= NOUT ; iout++, tout *= TWO) {
-      ier = IDASolve(mem, tout, &tret, uu, up, IDA_NORMAL);
-      if(check_flag(&ier, "IDASolve", 1)) return(1);
+      retval = IDASolve(mem, tout, &tret, uu, up, IDA_NORMAL);
+      if(check_retval(&retval, "IDASolve", 1)) return(1);
       PrintOutput(mem, tret, uu, linsolver);
     }
 
     /* Print remaining counters. */
-    ier = IDAGetNumErrTestFails(mem, &netf);
-    check_flag(&ier, "IDAGetNumErrTestFails", 1);
+    retval = IDAGetNumErrTestFails(mem, &netf);
+    check_retval(&retval, "IDAGetNumErrTestFails", 1);
 
-    ier = IDAGetNumNonlinSolvConvFails(mem, &ncfn);
-    check_flag(&ier, "IDAGetNumNonlinSolvConvFails", 1);
+    retval = IDAGetNumNonlinSolvConvFails(mem, &ncfn);
+    check_retval(&retval, "IDAGetNumNonlinSolvConvFails", 1);
 
-    ier = IDASpilsGetNumConvFails(mem, &ncfl);
-    check_flag(&ier, "IDASpilsGetNumConvFails", 1);
+    retval = IDAGetNumLinConvFails(mem, &ncfl);
+    check_retval(&retval, "IDAGetNumLinConvFails", 1);
 
     printf("\nError test failures            = %ld\n", netf);
     printf("Nonlinear convergence failures = %ld\n", ncfn);
@@ -308,34 +358,34 @@ int main(void)
  */
 
 /*
- * resHeat: heat equation system residual function (user-supplied)      
- * This uses 5-point central differencing on the interior points, and   
- * includes algebraic equations for the boundary values.                
- * So for each interior point, the residual component has the form      
- *    res_i = u'_i - (central difference)_i                             
- * while for each boundary point, it is res_i = u_i.                     
+ * resHeat: heat equation system residual function (user-supplied)
+ * This uses 5-point central differencing on the interior points, and
+ * includes algebraic equations for the boundary values.
+ * So for each interior point, the residual component has the form
+ *    res_i = u'_i - (central difference)_i
+ * while for each boundary point, it is res_i = u_i.
  */
 
-int resHeat(realtype tt, 
-            N_Vector uu, N_Vector up, N_Vector rr, 
+int resHeat(realtype tt,
+            N_Vector uu, N_Vector up, N_Vector rr,
             void *user_data)
 {
   sunindextype i, j, offset, loc, mm;
   realtype *uu_data, *up_data, *rr_data, coeff, dif1, dif2;
   UserData data;
-  
-  uu_data = N_VGetArrayPointer(uu); 
-  up_data = N_VGetArrayPointer(up); 
+
+  uu_data = N_VGetArrayPointer(uu);
+  up_data = N_VGetArrayPointer(up);
   rr_data = N_VGetArrayPointer(rr);
 
   data = (UserData) user_data;
-  
+
   coeff = data->coeff;
   mm    = data->mm;
-  
+
   /* Initialize rr to uu, to take care of boundary equations. */
   N_VScale(ONE, uu, rr);
-  
+
   /* Loop over interior points; set res = up - (central difference). */
   for (j = 1; j < MGRID-1; j++) {
     offset = mm*j;
@@ -351,31 +401,31 @@ int resHeat(realtype tt,
 }
 
 /*
- * PsetupHeat: setup for diagonal preconditioner.   
- *                                                                 
- * The optional user-supplied functions PsetupHeat and          
- * PsolveHeat together must define the left preconditoner        
- * matrix P approximating the system Jacobian matrix               
- *                   J = dF/du + cj*dF/du'                         
- * (where the DAE system is F(t,u,u') = 0), and solve the linear   
- * systems P z = r.   This is done in this case by keeping only    
- * the diagonal elements of the J matrix above, storing them as    
- * inverses in a vector pp, when computed in PsetupHeat, for    
- * subsequent use in PsolveHeat.                                 
- *                                                                 
- * In this instance, only cj and data (user data structure, with    
- * pp etc.) are used from the PsetupdHeat argument list.         
+ * PsetupHeat: setup for diagonal preconditioner.
+ *
+ * The optional user-supplied functions PsetupHeat and
+ * PsolveHeat together must define the left preconditoner
+ * matrix P approximating the system Jacobian matrix
+ *                   J = dF/du + cj*dF/du'
+ * (where the DAE system is F(t,u,u') = 0), and solve the linear
+ * systems P z = r.   This is done in this case by keeping only
+ * the diagonal elements of the J matrix above, storing them as
+ * inverses in a vector pp, when computed in PsetupHeat, for
+ * subsequent use in PsolveHeat.
+ *
+ * In this instance, only cj and data (user data structure, with
+ * pp etc.) are used from the PsetupdHeat argument list.
  */
-  
-int PsetupHeat(realtype tt, 
-               N_Vector uu, N_Vector up, N_Vector rr, 
+
+int PsetupHeat(realtype tt,
+               N_Vector uu, N_Vector up, N_Vector rr,
                realtype c_j, void *user_data)
 {
-  
+
   sunindextype i, j, offset, loc, mm;
   realtype *ppv, pelinv;
   UserData data;
-  
+
   data = (UserData) user_data;
   ppv = N_VGetArrayPointer(data->pp);
   mm = data->mm;
@@ -383,10 +433,10 @@ int PsetupHeat(realtype tt,
   /* Initialize the entire vector to 1., then set the interior points to the
      correct value for preconditioning. */
   N_VConst(ONE,data->pp);
-  
+
   /* Compute the inverse of the preconditioner diagonal elements. */
-  pelinv = ONE/(c_j + FOUR*data->coeff); 
-  
+  pelinv = ONE/(c_j + FOUR*data->coeff);
+
   for (j = 1; j < mm-1; j++) {
     offset = mm * j;
     for (i = 1; i < mm-1; i++) {
@@ -394,20 +444,20 @@ int PsetupHeat(realtype tt,
       ppv[loc] = pelinv;
     }
   }
-  
-  return(0);  
+
+  return(0);
 }
 
 /*
- * PsolveHeat: solve preconditioner linear system.              
- * This routine multiplies the input vector rvec by the vector pp 
- * containing the inverse diagonal Jacobian elements (previously  
- * computed in PrecondHeateq), returning the result in zvec.      
+ * PsolveHeat: solve preconditioner linear system.
+ * This routine multiplies the input vector rvec by the vector pp
+ * containing the inverse diagonal Jacobian elements (previously
+ * computed in PrecondHeateq), returning the result in zvec.
  */
 
-int PsolveHeat(realtype tt, 
-               N_Vector uu, N_Vector up, N_Vector rr, 
-               N_Vector rvec, N_Vector zvec, 
+int PsolveHeat(realtype tt,
+               N_Vector uu, N_Vector up, N_Vector rr,
+               N_Vector rvec, N_Vector zvec,
                realtype c_j, realtype delta, void *user_data)
 {
   UserData data;
@@ -426,7 +476,7 @@ int PsolveHeat(realtype tt,
  * SetInitialProfile: routine to initialize u and up vectors.
  */
 
-static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up, 
+static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
                              N_Vector res)
 {
   sunindextype mm, mm1, i, j, offset, loc;
@@ -437,7 +487,7 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
   udata = N_VGetArrayPointer(uu);
   updata = N_VGetArrayPointer(up);
 
-  /* Initialize uu on all grid points. */ 
+  /* Initialize uu on all grid points. */
   mm1 = mm - 1;
   for (j = 0; j < mm; j++) {
     yfact = data->dx * j;
@@ -448,7 +498,7 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
       udata[loc] = RCONST(16.0) * xfact * (ONE - xfact) * yfact * (ONE - yfact);
     }
   }
-  
+
   /* Initialize up vector to 0. */
   N_VConst(ZERO, up);
 
@@ -466,7 +516,7 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
       if (j == 0 || j == mm1 || i == 0 || i == mm1 ) updata[loc] = ZERO;
     }
   }
-  
+
   return(0);
   }
 
@@ -482,9 +532,9 @@ static void PrintHeader(realtype rtol, realtype atol, int linsolver)
   printf(" polynomial initial conditions.\n");
   printf("                   Mesh dimensions: %d x %d", MGRID, MGRID);
   printf("       Total system size: %d\n\n", NEQ);
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  rtol = %Lg   atol = %Lg\n", rtol, atol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #else
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
@@ -515,36 +565,36 @@ static void PrintOutput(void *mem, realtype t, N_Vector uu, int linsolver)
 {
   realtype hused, umax;
   long int nst, nni, nje, nre, nreLS, nli, npe, nps;
-  int kused, ier;
-  
+  int kused, retval;
+
   umax = N_VMaxNorm(uu);
 
-  ier = IDAGetLastOrder(mem, &kused);
-  check_flag(&ier, "IDAGetLastOrder", 1);
-  ier = IDAGetNumSteps(mem, &nst);
-  check_flag(&ier, "IDAGetNumSteps", 1);
-  ier = IDAGetNumNonlinSolvIters(mem, &nni);
-  check_flag(&ier, "IDAGetNumNonlinSolvIters", 1);
-  ier = IDAGetNumResEvals(mem, &nre);
-  check_flag(&ier, "IDAGetNumResEvals", 1);
-  ier = IDAGetLastStep(mem, &hused);
-  check_flag(&ier, "IDAGetLastStep", 1);
+  retval = IDAGetLastOrder(mem, &kused);
+  check_retval(&retval, "IDAGetLastOrder", 1);
+  retval = IDAGetNumSteps(mem, &nst);
+  check_retval(&retval, "IDAGetNumSteps", 1);
+  retval = IDAGetNumNonlinSolvIters(mem, &nni);
+  check_retval(&retval, "IDAGetNumNonlinSolvIters", 1);
+  retval = IDAGetNumResEvals(mem, &nre);
+  check_retval(&retval, "IDAGetNumResEvals", 1);
+  retval = IDAGetLastStep(mem, &hused);
+  check_retval(&retval, "IDAGetLastStep", 1);
 
-  ier = IDASpilsGetNumJtimesEvals(mem, &nje);
-  check_flag(&ier, "IDASpilsGetNumJtimesEvals", 1);
-  ier = IDASpilsGetNumLinIters(mem, &nli);
-  check_flag(&ier, "IDASpilsGetNumLinIters", 1);
-  ier = IDASpilsGetNumResEvals(mem, &nreLS);
-  check_flag(&ier, "IDASpilsGetNumResEvals", 1);
-  ier = IDASpilsGetNumPrecEvals(mem, &npe);
-  check_flag(&ier, "IDASpilsGetPrecEvals", 1);
-  ier = IDASpilsGetNumPrecSolves(mem, &nps);
-  check_flag(&ier, "IDASpilsGetNumPrecSolves", 1);
+  retval = IDAGetNumJtimesEvals(mem, &nje);
+  check_retval(&retval, "IDAGetNumJtimesEvals", 1);
+  retval = IDAGetNumLinIters(mem, &nli);
+  check_retval(&retval, "IDAGetNumLinIters", 1);
+  retval = IDAGetNumLinResEvals(mem, &nreLS);
+  check_retval(&retval, "IDAGetNumLinResEvals", 1);
+  retval = IDAGetNumPrecEvals(mem, &npe);
+  check_retval(&retval, "IDAGetNumPrecEvals", 1);
+  retval = IDAGetNumPrecSolves(mem, &nps);
+  check_retval(&retval, "IDAGetNumPrecSolves", 1);
 
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf(" %5.2Lf %13.5Le  %d  %3ld  %3ld  %3ld  %4ld  %4ld  %9.2Le  %3ld %3ld\n",
          t, umax, kused, nst, nni, nje, nre, nreLS, hused, npe, nps);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf(" %5.2f %13.5e  %d  %3ld  %3ld  %3ld  %4ld  %4ld  %9.2e  %3ld %3ld\n",
          t, umax, kused, nst, nni, nje, nre, nreLS, hused, npe, nps);
 #else
@@ -557,35 +607,35 @@ static void PrintOutput(void *mem, realtype t, N_Vector uu, int linsolver)
  * Check function return value...
  *   opt == 0 means SUNDIALS function allocates memory so check if
  *            returned NULL pointer
- *   opt == 1 means SUNDIALS function returns a flag so check if
- *            flag >= 0
+ *   opt == 1 means SUNDIALS function returns an integer value so check if
+ *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n", 
+  if (opt == 0 && returnvalue == NULL) {
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   } else if (opt == 1) {
-    /* Check if flag < 0 */
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n", 
-              funcname, *errflag);
-      return(1); 
+    /* Check if retval < 0 */
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
+      return(1);
     }
-  } else if (opt == 2 && flagvalue == NULL) {
+  } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   }
