@@ -1,20 +1,20 @@
-/* ----------------------------------------------------------------- 
+/* -----------------------------------------------------------------
  * Programmer(s): Radu Serban and Cosmin Petra @ LLNL
  * -----------------------------------------------------------------
- * LLNS Copyright Start
- * Copyright (c) 2014, Lawrence Livermore National Security
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Lawrence Livermore National Laboratory in part under 
- * Contract W-7405-Eng-48 and in part under Contract DE-AC52-07NA27344.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * Adjoint sensitivity example problem.
  *
- * This simple example problem for IDAS, due to Robertson, 
- * is from chemical kinetics, and consists of the following three 
+ * This simple example problem for IDAS, due to Robertson,
+ * is from chemical kinetics, and consists of the following three
  * equations:
  *
  *      dy1/dt + p1*y1 - p2*y2*y3            = 0
@@ -35,7 +35,7 @@
  *   g(t,p,y) = y3
  *
  * The gradient dG/dp is obtained as:
- *   dG/dp = int_t0^t1 (g_p - lambda^T F_p ) dt - 
+ *   dG/dp = int_t0^t1 (g_p - lambda^T F_p ) dt -
  *           lambda^T*F_y'*y_p | _t0^t1
  *         = int_t0^t1 (lambda^T*F_p) dt
  * where lambda and are solutions of the adjoint system:
@@ -55,7 +55,6 @@
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_dense.h> /* access to dense SUNMatrix            */
 #include <sunlinsol/sunlinsol_dense.h> /* access to dense SUNLinearSolver      */
-#include <idas/idas_direct.h>          /* access to IDADls interface           */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
 #include <sundials/sundials_math.h>    /* defs. of SUNRabs, SUNRexp, etc.      */
 
@@ -101,36 +100,36 @@ typedef struct {
 
 /* Prototypes of user-supplied functions */
 
-static int res(realtype t, N_Vector yy, N_Vector yp, 
+static int res(realtype t, N_Vector yy, N_Vector yp,
                N_Vector resval, void *user_data);
-static int Jac(realtype t, realtype cj, 
-               N_Vector yy, N_Vector yp, N_Vector resvec, 
-               SUNMatrix J, void *user_data, 
+static int Jac(realtype t, realtype cj,
+               N_Vector yy, N_Vector yp, N_Vector resvec,
+               SUNMatrix J, void *user_data,
                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_data);
 static int ewt(N_Vector y, N_Vector w, void *user_data);
 
-static int resB(realtype tt, 
+static int resB(realtype tt,
                 N_Vector yy, N_Vector yp,
                 N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 void *user_dataB);
 
 static int JacB(realtype tt, realtype cjB,
                 N_Vector yy, N_Vector yp,
-                N_Vector yyB, N_Vector ypB, N_Vector rrB, 
+                N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 SUNMatrix JB, void *user_data,
                 N_Vector tmp1B, N_Vector tmp2B, N_Vector tmp3B);
 
 
-static int rhsQB(realtype tt, 
-                 N_Vector yy, N_Vector yp, 
-                 N_Vector yyB, N_Vector ypB, 
+static int rhsQB(realtype tt,
+                 N_Vector yy, N_Vector yp,
+                 N_Vector yyB, N_Vector ypB,
                  N_Vector rrQB, void *user_dataB);
 
 /* Prototypes of private functions */
 static void PrintOutput(realtype tfinal, N_Vector yB, N_Vector ypB, N_Vector qB);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -158,7 +157,7 @@ int main(int argc, char *argv[])
   realtype reltolB, abstolB, abstolQB;
   N_Vector yB, ypB, qB;
   realtype time;
-  int flag, ncheck;
+  int retval, ncheck;
 
   IDAadjCheckPointRec *ckpnt;
 
@@ -183,28 +182,28 @@ int main(int argc, char *argv[])
 
   /* User data structure */
   data = (UserData) malloc(sizeof *data);
-  if (check_flag((void *)data, "malloc", 2)) return(1);
+  if (check_retval((void *)data, "malloc", 2)) return(1);
   data->p[0] = RCONST(0.04);
   data->p[1] = RCONST(1.0e4);
   data->p[2] = RCONST(3.0e7);
 
   /* Initialize y */
   yy = N_VNew_Serial(NEQ);
-  if (check_flag((void *)yy, "N_VNew_Serial", 0)) return(1);
+  if (check_retval((void *)yy, "N_VNew_Serial", 0)) return(1);
   Ith(yy,1) = ONE;
   Ith(yy,2) = ZERO;
   Ith(yy,3) = ZERO;
 
   /* Initialize yprime */
   yp = N_VNew_Serial(NEQ);
-  if (check_flag((void *)yp, "N_VNew_Serial", 0)) return(1);
+  if (check_retval((void *)yp, "N_VNew_Serial", 0)) return(1);
   Ith(yp,1) = RCONST(-0.04);
   Ith(yp,2) = RCONST( 0.04);
   Ith(yp,3) = ZERO;
 
   /* Initialize q */
   q = N_VNew_Serial(1);
-  if (check_flag((void *)q, "N_VNew_Serial", 0)) return(1);
+  if (check_retval((void *)q, "N_VNew_Serial", 0)) return(1);
   Ith(q,1) = ZERO;
 
   /* Set the scalar realtive and absolute tolerances reltolQ and abstolQ */
@@ -215,73 +214,80 @@ int main(int argc, char *argv[])
   printf("Create and allocate IDAS memory for forward runs\n");
 
   ida_mem = IDACreate();
-  if (check_flag((void *)ida_mem, "IDACreate", 0)) return(1);
+  if (check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
-  flag = IDAInit(ida_mem, res, T0, yy, yp);
-  if (check_flag(&flag, "IDAInit", 1)) return(1);
+  retval = IDAInit(ida_mem, res, T0, yy, yp);
+  if (check_retval(&retval, "IDAInit", 1)) return(1);
 
-  flag = IDAWFtolerances(ida_mem, ewt);
-  if (check_flag(&flag, "IDAWFtolerances", 1)) return(1);
+  retval = IDAWFtolerances(ida_mem, ewt);
+  if (check_retval(&retval, "IDAWFtolerances", 1)) return(1);
 
-  flag = IDASetUserData(ida_mem, data);
-  if (check_flag(&flag, "IDASetUserData", 1)) return(1);
+  retval = IDASetUserData(ida_mem, data);
+  if (check_retval(&retval, "IDASetUserData", 1)) return(1);
 
   /* Create dense SUNMatrix for use in linear solves */
   A = SUNDenseMatrix(NEQ, NEQ);
-  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+  if(check_retval((void *)A, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LS = SUNDenseLinearSolver(yy, A);
-  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Dense(yy, A);
+  if(check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
-  flag = IDADlsSetLinearSolver(ida_mem, LS, A);
-  if(check_flag(&flag, "IDADlsSetLinearSolver", 1)) return(1);
+  retval = IDASetLinearSolver(ida_mem, LS, A);
+  if(check_retval(&retval, "IDASetLinearSolver", 1)) return(1);
 
   /* Set the user-supplied Jacobian routine */
-  flag = IDADlsSetJacFn(ida_mem, Jac);
-  if(check_flag(&flag, "IDADlsSetJacFn", 1)) return(1);
+  retval = IDASetJacFn(ida_mem, Jac);
+  if(check_retval(&retval, "IDASetJacFn", 1)) return(1);
 
-  flag = IDAQuadInit(ida_mem, rhsQ, q);
-  if (check_flag(&flag, "IDAQuadInit", 1)) return(1);
+  /* Setup quadrature integration */
+  retval = IDAQuadInit(ida_mem, rhsQ, q);
+  if (check_retval(&retval, "IDAQuadInit", 1)) return(1);
 
-  flag = IDAQuadSStolerances(ida_mem, reltolQ, abstolQ);
-  if (check_flag(&flag, "IDAQuadSStolerances", 1)) return(1);
+  retval = IDAQuadSStolerances(ida_mem, reltolQ, abstolQ);
+  if (check_retval(&retval, "IDAQuadSStolerances", 1)) return(1);
 
-  flag = IDASetQuadErrCon(ida_mem, SUNTRUE);
-  if (check_flag(&flag, "IDASetQuadErrCon", 1)) return(1);
+  retval = IDASetQuadErrCon(ida_mem, SUNTRUE);
+  if (check_retval(&retval, "IDASetQuadErrCon", 1)) return(1);
+
+  /* Call IDASetMaxNumSteps to set the maximum number of steps the
+   * solver will take in an attempt to reach the next output time
+   * during forward integration. */
+  retval = IDASetMaxNumSteps(ida_mem, 2500);
+  if (check_retval(&retval, "IDASetMaxNumSteps", 1)) return(1);
 
   /* Allocate global memory */
 
   steps = STEPS;
-  flag = IDAAdjInit(ida_mem, steps, IDA_HERMITE);
-  /*flag = IDAAdjInit(ida_mem, steps, IDA_POLYNOMIAL);*/
-  if (check_flag(&flag, "IDAAdjInit", 1)) return(1);
+  retval = IDAAdjInit(ida_mem, steps, IDA_HERMITE);
+  /*retval = IDAAdjInit(ida_mem, steps, IDA_POLYNOMIAL);*/
+  if (check_retval(&retval, "IDAAdjInit", 1)) return(1);
 
   /* Perform forward run */
   printf("Forward integration ... ");
 
   /* Integrate till TB1 and get the solution (y, y') at that time. */
-  flag = IDASolveF(ida_mem, TB1, &time, yy, yp, IDA_NORMAL, &ncheck);
-  if (check_flag(&flag, "IDASolveF", 1)) return(1);
+  retval = IDASolveF(ida_mem, TB1, &time, yy, yp, IDA_NORMAL, &ncheck);
+  if (check_retval(&retval, "IDASolveF", 1)) return(1);
 
   yyTB1 = N_VClone(yy);
   ypTB1 = N_VClone(yp);
   /* Save the states at t=TB1. */
   N_VScale(ONE, yy, yyTB1);
   N_VScale(ONE, yp, ypTB1);
-  
-  /* Continue integrating till TOUT is reached. */
-  flag = IDASolveF(ida_mem, TOUT, &time, yy, yp, IDA_NORMAL, &ncheck);
-  if (check_flag(&flag, "IDASolveF", 1)) return(1);
 
-  flag = IDAGetNumSteps(ida_mem, &nst);
-  if (check_flag(&flag, "IDAGetNumSteps", 1)) return(1);
+  /* Continue integrating till TOUT is reached. */
+  retval = IDASolveF(ida_mem, TOUT, &time, yy, yp, IDA_NORMAL, &ncheck);
+  if (check_retval(&retval, "IDASolveF", 1)) return(1);
+
+  retval = IDAGetNumSteps(ida_mem, &nst);
+  if (check_retval(&retval, "IDAGetNumSteps", 1)) return(1);
 
   printf("done ( nst = %ld )\n",nst);
 
-  flag = IDAGetQuad(ida_mem, &time, q);
-  if (check_flag(&flag, "IDAGetQuad", 1)) return(1);
+  retval = IDAGetQuad(ida_mem, &time, q);
+  if (check_retval(&retval, "IDAGetQuad", 1)) return(1);
 
   printf("--------------------------------------------------------\n");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
@@ -293,13 +299,13 @@ int main(int argc, char *argv[])
 #endif
   printf("--------------------------------------------------------\n\n");
 
-  /* Test check point linked list 
+  /* Test check point linked list
      (uncomment next block to print check point information) */
-  
-  /*  
+
+  /*
   {
     int i;
-    
+
     printf("\nList of Check Points (ncheck = %d)\n\n", ncheck);
     ckpnt = (IDAadjCheckPointRec *) malloc ( (ncheck+1)*sizeof(IDAadjCheckPointRec));
     IDAGetAdjCheckPointsInfo(ida_mem, ckpnt);
@@ -312,7 +318,7 @@ int main(int argc, char *argv[])
       printf("Step size:     %le\n",ckpnt[i].step);
       printf("\n");
     }
-    
+
   }
   */
 
@@ -321,26 +327,26 @@ int main(int argc, char *argv[])
 
   /* Allocate yB (i.e. lambda_0). */
   yB = N_VNew_Serial(NEQ);
-  if (check_flag((void *)yB, "N_VNew_Serial", 0)) return(1);
+  if (check_retval((void *)yB, "N_VNew_Serial", 0)) return(1);
 
   /* Consistently initialize yB. */
   Ith(yB,1) = ZERO;
   Ith(yB,2) = ZERO;
   Ith(yB,3) = ONE;
 
-    
+
   /* Allocate ypB (i.e. lambda'_0). */
   ypB = N_VNew_Serial(NEQ);
-  if (check_flag((void *)ypB, "N_VNew_Serial", 0)) return(1);
+  if (check_retval((void *)ypB, "N_VNew_Serial", 0)) return(1);
 
   /* Consistently initialize ypB. */
   Ith(ypB,1) = ONE;
   Ith(ypB,2) = ONE;
   Ith(ypB,3) = ZERO;
 
-  
+
   /* Set the scalar relative tolerance reltolB */
-  reltolB = RTOL;               
+  reltolB = RTOL;
 
   /* Set the scalar absolute tolerance abstolB */
   abstolB = ATOLA;
@@ -351,70 +357,71 @@ int main(int argc, char *argv[])
   /* Create and allocate IDAS memory for backward run */
   printf("Create and allocate IDAS memory for backward run\n");
 
-  flag = IDACreateB(ida_mem, &indexB);
-  if (check_flag(&flag, "IDACreateB", 1)) return(1);
+  retval = IDACreateB(ida_mem, &indexB);
+  if (check_retval(&retval, "IDACreateB", 1)) return(1);
 
-  flag = IDAInitB(ida_mem, indexB, resB, TB2, yB, ypB);
-  if (check_flag(&flag, "IDAInitB", 1)) return(1);
+  retval = IDAInitB(ida_mem, indexB, resB, TB2, yB, ypB);
+  if (check_retval(&retval, "IDAInitB", 1)) return(1);
 
-  flag = IDASStolerancesB(ida_mem, indexB, reltolB, abstolB);
-  if (check_flag(&flag, "IDASStolerancesB", 1)) return(1);
+  retval = IDASStolerancesB(ida_mem, indexB, reltolB, abstolB);
+  if (check_retval(&retval, "IDASStolerancesB", 1)) return(1);
 
-  flag = IDASetUserDataB(ida_mem, indexB, data);
-  if (check_flag(&flag, "IDASetUserDataB", 1)) return(1);
+  retval = IDASetUserDataB(ida_mem, indexB, data);
+  if (check_retval(&retval, "IDASetUserDataB", 1)) return(1);
 
-  flag = IDASetMaxNumStepsB(ida_mem, indexB, 1000);
+  retval = IDASetMaxNumStepsB(ida_mem, indexB, 1000);
+  if (check_retval(&retval, "IDASetMaxNumStepsB", 1)) return(1);
 
   /* Create dense SUNMatrix for use in linear solves */
   AB = SUNDenseMatrix(NEQ, NEQ);
-  if(check_flag((void *)AB, "SUNDenseMatrix", 0)) return(1);
+  if(check_retval((void *)AB, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LSB = SUNDenseLinearSolver(yB, AB);
-  if(check_flag((void *)LSB, "SUNDenseLinearSolver", 0)) return(1);
+  LSB = SUNLinSol_Dense(yB, AB);
+  if(check_retval((void *)LSB, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
-  flag = IDADlsSetLinearSolverB(ida_mem, indexB, LSB, AB);
-  if(check_flag(&flag, "IDADlsSetLinearSolverB", 1)) return(1);
+  retval = IDASetLinearSolverB(ida_mem, indexB, LSB, AB);
+  if(check_retval(&retval, "IDASetLinearSolverB", 1)) return(1);
 
   /* Set the user-supplied Jacobian routine */
-  flag = IDADlsSetJacFnB(ida_mem, indexB, JacB);
-  if(check_flag(&flag, "IDADlsSetJacFnB", 1)) return(1);
+  retval = IDASetJacFnB(ida_mem, indexB, JacB);
+  if(check_retval(&retval, "IDASetJacFnB", 1)) return(1);
 
   /* Quadrature for backward problem. */
- 
+
   /* Initialize qB */
   qB = N_VNew_Serial(NP);
-  if (check_flag((void *)qB, "N_VNew", 0)) return(1);
+  if (check_retval((void *)qB, "N_VNew", 0)) return(1);
   Ith(qB,1) = ZERO;
   Ith(qB,2) = ZERO;
   Ith(qB,3) = ZERO;
 
-  flag = IDAQuadInitB(ida_mem, indexB, rhsQB, qB);
-  if (check_flag(&flag, "IDAQuadInitB", 1)) return(1);
+  retval = IDAQuadInitB(ida_mem, indexB, rhsQB, qB);
+  if (check_retval(&retval, "IDAQuadInitB", 1)) return(1);
 
-  flag = IDAQuadSStolerancesB(ida_mem, indexB, reltolB, abstolQB);
-  if (check_flag(&flag, "IDAQuadSStolerancesB", 1)) return(1);
+  retval = IDAQuadSStolerancesB(ida_mem, indexB, reltolB, abstolQB);
+  if (check_retval(&retval, "IDAQuadSStolerancesB", 1)) return(1);
 
   /* Include quadratures in error control. */
-  flag = IDASetQuadErrConB(ida_mem, indexB, SUNTRUE);
-  if (check_flag(&flag, "IDASetQuadErrConB", 1)) return(1);
+  retval = IDASetQuadErrConB(ida_mem, indexB, SUNTRUE);
+  if (check_retval(&retval, "IDASetQuadErrConB", 1)) return(1);
 
 
   /* Backward Integration */
   printf("Backward integration ... ");
 
-  flag = IDASolveB(ida_mem, T0, IDA_NORMAL);
-  if (check_flag(&flag, "IDASolveB", 1)) return(1);
+  retval = IDASolveB(ida_mem, T0, IDA_NORMAL);
+  if (check_retval(&retval, "IDASolveB", 1)) return(1);
 
   IDAGetNumSteps(IDAGetAdjIDABmem(ida_mem, indexB), &nstB);
   printf("done ( nst = %ld )\n", nstB);
 
-  flag = IDAGetB(ida_mem, indexB, &time, yB, ypB);
-  if (check_flag(&flag, "IDAGetB", 1)) return(1);
+  retval = IDAGetB(ida_mem, indexB, &time, yB, ypB);
+  if (check_retval(&retval, "IDAGetB", 1)) return(1);
 
-  flag = IDAGetQuadB(ida_mem, indexB, &time, qB);
-  if (check_flag(&flag, "IDAGetB", 1)) return(1);
+  retval = IDAGetQuadB(ida_mem, indexB, &time, qB);
+  if (check_retval(&retval, "IDAGetB", 1)) return(1);
 
   PrintOutput(TB2, yB, ypB, qB);
 
@@ -437,14 +444,14 @@ int main(int argc, char *argv[])
   Ith(qB,2) = ZERO;
   Ith(qB,3) = ZERO;
 
-  flag = IDAReInitB(ida_mem, indexB, TB1, yB, ypB);
-  if (check_flag(&flag, "IDAReInitB", 1)) return(1);
+  retval = IDAReInitB(ida_mem, indexB, TB1, yB, ypB);
+  if (check_retval(&retval, "IDAReInitB", 1)) return(1);
 
   /* Also reinitialize quadratures. */
-  flag = IDAQuadInitB(ida_mem, indexB, rhsQB, qB);
-  if (check_flag(&flag, "IDAQuadInitB", 1)) return(1);
+  retval = IDAQuadReInitB(ida_mem, indexB, qB);
+  if (check_retval(&retval, "IDAQuadReInitB", 1)) return(1);
 
-  /* Use IDACalcICB to compute consistent initial conditions 
+  /* Use IDACalcICB to compute consistent initial conditions
      for this backward problem. */
 
   id = N_VNew_Serial(NEQ);
@@ -453,29 +460,29 @@ int main(int argc, char *argv[])
   Ith(id,3) = 0.0;
 
   /* Specify which variables are differential (1) and which algebraic (0).*/
-  flag = IDASetIdB(ida_mem, indexB, id);
-  if (check_flag(&flag, "IDASetId", 1)) return(1);
+  retval = IDASetIdB(ida_mem, indexB, id);
+  if (check_retval(&retval, "IDASetId", 1)) return(1);
 
-  flag = IDACalcICB(ida_mem, indexB, T1B, yyTB1, ypTB1);
-  if (check_flag(&flag, "IDACalcICB", 1)) return(1);
+  retval = IDACalcICB(ida_mem, indexB, T1B, yyTB1, ypTB1);
+  if (check_retval(&retval, "IDACalcICB", 1)) return(1);
 
   /* Get the consistent IC found by IDAS. */
-  flag = IDAGetConsistentICB(ida_mem, indexB, yB, ypB);
-  if (check_flag(&flag, "IDAGetConsistentICB", 1)) return(1);
+  retval = IDAGetConsistentICB(ida_mem, indexB, yB, ypB);
+  if (check_retval(&retval, "IDAGetConsistentICB", 1)) return(1);
 
   printf("Backward integration ... ");
 
-  flag = IDASolveB(ida_mem, T0, IDA_NORMAL);
-  if (check_flag(&flag, "IDASolveB", 1)) return(1);
+  retval = IDASolveB(ida_mem, T0, IDA_NORMAL);
+  if (check_retval(&retval, "IDASolveB", 1)) return(1);
 
   IDAGetNumSteps(IDAGetAdjIDABmem(ida_mem, indexB), &nstB);
   printf("done ( nst = %ld )\n", nstB);
 
-  flag = IDAGetB(ida_mem, indexB, &time, yB, ypB);
-  if (check_flag(&flag, "IDAGetB", 1)) return(1);
+  retval = IDAGetB(ida_mem, indexB, &time, yB, ypB);
+  if (check_retval(&retval, "IDAGetB", 1)) return(1);
 
-  flag = IDAGetQuadB(ida_mem, indexB, &time, qB);
-  if (check_flag(&flag, "IDAGetQuadB", 1)) return(1);
+  retval = IDAGetQuadB(ida_mem, indexB, &time, qB);
+  if (check_retval(&retval, "IDAGetQuadB", 1)) return(1);
 
   PrintOutput(TB1, yB, ypB, qB);
 
@@ -512,7 +519,7 @@ int main(int argc, char *argv[])
  */
 
 /*
- * f routine. Compute f(t,y). 
+ * f routine. Compute f(t,y).
 */
 
 static int res(realtype t, N_Vector yy, N_Vector yp, N_Vector resval, void *user_data)
@@ -521,7 +528,7 @@ static int res(realtype t, N_Vector yy, N_Vector yp, N_Vector resval, void *user
   UserData data;
   realtype p1, p2, p3;
 
-  y1  = Ith(yy,1); y2  = Ith(yy,2); y3  = Ith(yy,3); 
+  y1  = Ith(yy,1); y2  = Ith(yy,2); y3  = Ith(yy,3);
   yp1 = Ith(yp,1); yp2 = Ith(yp,2);
   rval = N_VGetArrayPointer(resval);
 
@@ -536,32 +543,32 @@ static int res(realtype t, N_Vector yy, N_Vector yp, N_Vector resval, void *user
   return(0);
 }
 
-/* 
- * Jacobian routine. Compute J(t,y). 
+/*
+ * Jacobian routine. Compute J(t,y).
 */
 
 static int Jac(realtype t, realtype cj,
-               N_Vector yy, N_Vector yp, N_Vector resvec, 
-               SUNMatrix J, void *user_data, 
+               N_Vector yy, N_Vector yp, N_Vector resvec,
+               SUNMatrix J, void *user_data,
                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   realtype y2, y3;
   UserData data;
   realtype p1, p2, p3;
- 
+
   y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   data = (UserData) user_data;
   p1 = data->p[0]; p2 = data->p[1]; p3 = data->p[2];
 
   IJth(J,1,1) = p1+cj;
   IJth(J,2,1) = -p1;
-  IJth(J,3,1) = ONE;     
+  IJth(J,3,1) = ONE;
 
   IJth(J,1,2) = -p2*y3;
-  IJth(J,2,2) = p2*y3+2*p3*y2+cj; 
+  IJth(J,2,2) = p2*y3+2*p3*y2+cj;
   IJth(J,3,2) = ONE;
-                     
+
   IJth(J,1,3) = -p2*y2;
   IJth(J,2,3) = p2*y2;
   IJth(J,3,3) = ONE;
@@ -569,13 +576,13 @@ static int Jac(realtype t, realtype cj,
   return(0);
 }
 
-/* 
- * rhsQ routine. Compute fQ(t,y). 
+/*
+ * rhsQ routine. Compute fQ(t,y).
 */
 
 static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_data)
 {
-  Ith(qdot,1) = Ith(yy,3);  
+  Ith(qdot,1) = Ith(yy,3);
   return(0);
 }
 
@@ -603,12 +610,12 @@ static int ewt(N_Vector y, N_Vector w, void *user_data)
   return(0);
 }
 
- 
-/* 
+
+/*
  * resB routine.
 */
 
-static int resB(realtype tt, 
+static int resB(realtype tt,
                  N_Vector yy, N_Vector yp,
                  N_Vector yyB, N_Vector ypB, N_Vector rrB,
                  void *user_dataB)
@@ -619,7 +626,7 @@ static int resB(realtype tt,
   realtype l1, l2, l3;
   realtype lp1, lp2;
   realtype l21;
-  
+
   data = (UserData) user_dataB;
 
   /* The p vector */
@@ -627,7 +634,7 @@ static int resB(realtype tt,
 
   /* The y  vector */
   y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   /* The lambda vector */
   l1 = Ith(yyB,1); l2 = Ith(yyB,2); l3 = Ith(yyB,3);
 
@@ -648,27 +655,27 @@ static int resB(realtype tt,
 /*Jacobian for backward problem. */
 static int JacB(realtype tt, realtype cj,
                 N_Vector yy, N_Vector yp,
-                N_Vector yyB, N_Vector ypB, N_Vector rrB, 
+                N_Vector yyB, N_Vector ypB, N_Vector rrB,
                 SUNMatrix JB, void *user_data,
                 N_Vector tmp1B, N_Vector tmp2B, N_Vector tmp3B)
 {
   realtype y2, y3;
   UserData data;
   realtype p1, p2, p3;
- 
+
   y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   data = (UserData) user_data;
   p1 = data->p[0]; p2 = data->p[1]; p3 = data->p[2];
 
   IJth(JB,1,1) = -p1+cj;
   IJth(JB,1,2) = p1;
-  IJth(JB,1,3) = -ONE;     
+  IJth(JB,1,3) = -ONE;
 
   IJth(JB,2,1) = p2*y3;
-  IJth(JB,2,2) = -(p2*y3+RCONST(2.0)*p3*y2)+cj; 
+  IJth(JB,2,2) = -(p2*y3+RCONST(2.0)*p3*y2)+cj;
   IJth(JB,2,3) = -ONE;
-                     
+
   IJth(JB,3,1) = p2*y2;
   IJth(JB,3,2) = -p2*y2;
   IJth(JB,3,3) = -ONE;
@@ -677,9 +684,9 @@ static int JacB(realtype tt, realtype cj,
   return(0);
 }
 
-static int rhsQB(realtype tt, 
-                 N_Vector yy, N_Vector yp, 
-                 N_Vector yyB, N_Vector ypB, 
+static int rhsQB(realtype tt,
+                 N_Vector yy, N_Vector yp,
+                 N_Vector yyB, N_Vector ypB,
                  N_Vector rrQB, void *user_dataB)
 {
   realtype y1, y2, y3;
@@ -688,10 +695,10 @@ static int rhsQB(realtype tt,
 
   /* The y vector */
   y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   /* The lambda vector */
   l1 = Ith(yyB,1); l2 = Ith(yyB,2);
-  
+
   /* Temporary variables */
   l21 = l2-l1;
 
@@ -718,56 +725,56 @@ static void PrintOutput(realtype tfinal, N_Vector yB, N_Vector ypB, N_Vector qB)
   printf("--------------------------------------------------------\n");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("tB0:        %12.4Le\n",tfinal);
-  printf("dG/dp:      %12.4Le %12.4Le %12.4Le\n", 
+  printf("dG/dp:      %12.4Le %12.4Le %12.4Le\n",
          -Ith(qB,1), -Ith(qB,2), -Ith(qB,3));
-  printf("lambda(t0): %12.4Le %12.4Le %12.4Le\n", 
+  printf("lambda(t0): %12.4Le %12.4Le %12.4Le\n",
          Ith(yB,1), Ith(yB,2), Ith(yB,3));
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("tB0:        %12.4e\n",tfinal);
-  printf("dG/dp:      %12.4e %12.4e %12.4e\n", 
+  printf("dG/dp:      %12.4e %12.4e %12.4e\n",
          -Ith(qB,1), -Ith(qB,2), -Ith(qB,3));
-  printf("lambda(t0): %12.4e %12.4e %12.4e\n", 
+  printf("lambda(t0): %12.4e %12.4e %12.4e\n",
          Ith(yB,1), Ith(yB,2), Ith(yB,3));
 #else
   printf("tB0:        %12.4e\n",tfinal);
-  printf("dG/dp:      %12.4e %12.4e %12.4e\n", 
+  printf("dG/dp:      %12.4e %12.4e %12.4e\n",
          -Ith(qB,1), -Ith(qB,2), -Ith(qB,3));
-  printf("lambda(t0): %12.4e %12.4e %12.4e\n", 
+  printf("lambda(t0): %12.4e %12.4e %12.4e\n",
          Ith(yB,1), Ith(yB,2), Ith(yB,3));
 #endif
   printf("--------------------------------------------------------\n\n");
 }
 
-/* 
+/*
  * Check function return value.
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns an integer value so check if
+ *             retval < 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && returnvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1); }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *retval);
       return(1); }}
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && returnvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1); }
