@@ -2,6 +2,16 @@
  * Programmer(s): Allan Taylor, Alan Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * Example (serial):
  *
  * This example solves a nonlinear system that arises from a system
@@ -79,7 +89,6 @@
 
 #include <kinsol/kinsol.h>             /* access to KINSOL func., consts.      */
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
-#include <kinsol/kinsol_spils.h>       /* access to KINSpils interface         */
 #include <sunlinsol/sunlinsol_spgmr.h> /* access to SPGMR SUNLinearSolver      */
 #include <sundials/sundials_dense.h>   /* use generic dense solver in precond. */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
@@ -88,9 +97,9 @@
 /* Problem Constants */
 
 #define NUM_SPECIES     6  /* must equal 2*(number of prey or predators)
-                              number of prey = number of predators       */ 
+                              number of prey = number of predators       */
 
-#define PI       RCONST(3.1415926535898)   /* pi */ 
+#define PI       RCONST(3.1415926535898)   /* pi */
 
 #define MX          8              /* MX = number of x mesh points */
 #define MY          8              /* MY = number of y mesh points */
@@ -118,12 +127,12 @@
 
 /* IJ_Vptr is defined in order to translate from the underlying 3D structure
    of the dependent variable vector to the 1D storage scheme for an N-vector.
-   IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
+   IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
    indices is = 0, jx = i, jy = j.    */
 
 #define IJ_Vptr(vv,i,j)   (&NV_Ith_S(vv, i*NUM_SPECIES + j*NSMX))
 
-/* Type : UserData 
+/* Type : UserData
    contains preconditioner blocks, pivot arrays, and problem constants */
 
 typedef struct {
@@ -145,8 +154,8 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
                        N_Vector fval, N_Vector fscale,
                        void *user_data);
 
-static int PrecSolveBD(N_Vector cc, N_Vector cscale, 
-                       N_Vector fval, N_Vector fscale, 
+static int PrecSolveBD(N_Vector cc, N_Vector cscale,
+                       N_Vector fval, N_Vector fscale,
                        N_Vector vv, void *user_data);
 
 /* Private Helper Functions */
@@ -155,11 +164,11 @@ static UserData AllocUserData(void);
 static void InitUserData(UserData data);
 static void FreeUserData(UserData data);
 static void SetInitialProfiles(N_Vector cc, N_Vector sc);
-static void PrintHeader(int globalstrategy, int maxl, int maxlrst, 
+static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
                         realtype fnormtol, realtype scsteptol);
 static void PrintOutput(N_Vector cc);
 static void PrintFinalStats(void *kmem);
-static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                     void *user_data);
 static realtype DotProd(sunindextype size, realtype *x1, realtype *x2);
 static int check_flag(void *flagvalue, const char *funcname, int opt);
@@ -185,7 +194,7 @@ int main(void)
   LS = NULL;
   data = NULL;
 
-  /* Allocate memory, and set problem data, initial values, tolerances */ 
+  /* Allocate memory, and set problem data, initial values, tolerances */
   globalstrategy = KIN_NONE;
 
   data = AllocUserData();
@@ -228,29 +237,27 @@ int main(void)
 
   /* We no longer need the constraints vector since KINSetConstraints
      creates a private copy for KINSOL to use. */
-  N_VDestroy_Serial(constraints);
+  N_VDestroy(constraints);
 
 
-  /* Create SUNSPGMR object with right preconditioning and the 
+  /* Create SUNLinSol_SPGMR object with right preconditioning and the
      maximum Krylov dimension maxl */
-  maxl = 15; 
-  LS = SUNSPGMR(cc, PREC_RIGHT, maxl);
-  if(check_flag((void *)LS, "SUNSPGMR", 0)) return(1);
+  maxl = 15;
+  LS = SUNLinSol_SPGMR(cc, PREC_RIGHT, maxl);
+  if(check_flag((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
   /* Attach the linear solver to KINSOL */
-  flag = KINSpilsSetLinearSolver(kmem, LS);
-  if (check_flag(&flag, "KINSpilsSetLinearSolver", 1)) return 1;
+  flag = KINSetLinearSolver(kmem, LS, NULL);
+  if (check_flag(&flag, "KINSetLinearSolver", 1)) return 1;
 
   /* Set the maximum number of restarts */
   maxlrst = 2;
-  flag = SUNSPGMRSetMaxRestarts(LS, maxlrst);
-  if (check_flag(&flag, "SUNSPGMRSpilsSetMaxRestarts", 1)) return(1);
+  flag = SUNLinSol_SPGMRSetMaxRestarts(LS, maxlrst);
+  if (check_flag(&flag, "SUNLinSol_SPGMRSetMaxRestarts", 1)) return(1);
 
   /* Specify the preconditioner setup and solve routines */
-  flag = KINSpilsSetPreconditioner(kmem,
-				   PrecSetupBD,
-				   PrecSolveBD);
-  if (check_flag(&flag, "KINSpilsSetPreconditioner", 1)) return(1);
+  flag = KINSetPreconditioner(kmem, PrecSetupBD, PrecSolveBD);
+  if (check_flag(&flag, "KINSetPreconditioner", 1)) return(1);
 
   /* Print out the problem size, solution parameters, initial guess. */
   PrintHeader(globalstrategy, maxl, maxlrst, fnormtol, scsteptol);
@@ -266,11 +273,11 @@ int main(void)
   printf("\n\nComputed equilibrium species concentrations:\n");
   PrintOutput(cc);
 
-  /* Print final statistics and free memory */  
+  /* Print final statistics and free memory */
   PrintFinalStats(kmem);
 
-  N_VDestroy_Serial(cc);
-  N_VDestroy_Serial(sc);
+  N_VDestroy(cc);
+  N_VDestroy(sc);
   KINFree(&kmem);
   SUNLinSolFree(LS);
   FreeUserData(data);
@@ -291,8 +298,8 @@ int main(void)
  *--------------------------------------------------------------------
  */
 
-/* 
- * System function for predator-prey system 
+/*
+ * System function for predator-prey system
  */
 
 static int func(N_Vector cc, N_Vector fval, void *user_data)
@@ -300,20 +307,20 @@ static int func(N_Vector cc, N_Vector fval, void *user_data)
   realtype xx, yy, delx, dely, *cxy, *rxy, *fxy, dcyli, dcyui, dcxli, dcxri;
   sunindextype jx, jy, is, idyu, idyl, idxr, idxl;
   UserData data;
-  
+
   data = (UserData)user_data;
   delx = data->dx;
   dely = data->dy;
-  
+
   /* Loop over all mesh points, evaluating rate array at each point*/
   for (jy = 0; jy < MY; jy++) {
-    
+
     yy = dely*jy;
 
     /* Set lower/upper index shifts, special at boundaries. */
     idyl = (jy != 0   ) ? NSMX : -NSMX;
     idyu = (jy != MY-1) ? NSMX : -NSMX;
-    
+
     for (jx = 0; jx < MX; jx++) {
 
       xx = delx*jx;
@@ -328,32 +335,32 @@ static int func(N_Vector cc, N_Vector fval, void *user_data)
 
       /* Get species interaction rate array at (xx,yy) */
       WebRate(xx, yy, cxy, rxy, user_data);
-      
+
       for(is = 0; is < NUM_SPECIES; is++) {
-        
+
         /* Differencing in x direction */
         dcyli = *(cxy+is) - *(cxy - idyl + is) ;
         dcyui = *(cxy + idyu + is) - *(cxy+is);
-        
+
         /* Differencing in y direction */
         dcxli = *(cxy+is) - *(cxy - idxl + is);
         dcxri = *(cxy + idxr +is) - *(cxy+is);
-        
+
         /* Compute the total rate value at (xx,yy) */
         fxy[is] = (coy)[is] * (dcyui - dcyli) +
           (cox)[is] * (dcxri - dcxli) + rxy[is];
-        
+
       } /* end of is loop */
-      
+
     } /* end of jx loop */
-    
+
   } /* end of jy loop */
 
   return(0);
 }
 
 /*
- * Preconditioner setup routine. Generate and preprocess P. 
+ * Preconditioner setup routine. Generate and preprocess P.
  */
 
 static int PrecSetupBD(N_Vector cc, N_Vector cscale,
@@ -364,21 +371,21 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
   realtype *cxy, *scxy, **Pxy, *ratesxy, *Pxycol, perturb_rates[NUM_SPECIES];
   sunindextype i, j, jx, jy, ret;
   UserData data;
-  
+
   data = (UserData) user_data;
   delx = data->dx;
   dely = data->dy;
-  
+
   uround = data->uround;
   sqruround = data->sqruround;
   fac = N_VWL2Norm(fval, fscale);
   r0 = THOUSAND * uround * fac * NEQ;
   if(r0 == ZERO) r0 = ONE;
-  
+
   /* Loop over spatial points; get size NUM_SPECIES Jacobian block at each */
   for (jy = 0; jy < MY; jy++) {
     yy = jy*dely;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = jx*delx;
       Pxy = (data->P)[jx][jy];
@@ -388,53 +395,53 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
 
       /* Compute difference quotients of interaction rate fn. */
       for (j = 0; j < NUM_SPECIES; j++) {
-        
+
         csave = cxy[j];  /* Save the j,jx,jy element of cc */
         r = SUNMAX(sqruround*SUNRabs(csave), r0/scxy[j]);
         cxy[j] += r; /* Perturb the j,jx,jy element of cc */
         fac = ONE/r;
-        
+
         WebRate(xx, yy, cxy, perturb_rates, data);
-        
+
         /* Restore j,jx,jy element of cc */
         cxy[j] = csave;
-        
+
         /* Load the j-th column of difference quotients */
         Pxycol = Pxy[j];
         for (i = 0; i < NUM_SPECIES; i++)
           Pxycol[i] = (perturb_rates[i] - ratesxy[i]) * fac;
-        
+
       } /* end of j loop */
-      
+
       /* Do LU decomposition of size NUM_SPECIES preconditioner block */
       ret = denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (data->pivot)[jx][jy]);
       if (ret != 0) return(1);
-      
+
     } /* end of jx loop */
-    
+
   } /* end of jy loop */
-  
+
   return(0);
 }
 
 /*
- * Preconditioner solve routine 
+ * Preconditioner solve routine
  */
 
-static int PrecSolveBD(N_Vector cc, N_Vector cscale, 
-                       N_Vector fval, N_Vector fscale, 
+static int PrecSolveBD(N_Vector cc, N_Vector cscale,
+                       N_Vector fval, N_Vector fscale,
                        N_Vector vv, void *user_data)
 {
   realtype **Pxy, *vxy;
   sunindextype *piv, jx, jy;
   UserData data;
-  
+
   data = (UserData)user_data;
-  
+
   for (jx=0; jx<MX; jx++) {
-    
+
     for (jy=0; jy<MY; jy++) {
-      
+
       /* For each (jx,jy), solve a linear system of size NUM_SPECIES.
          vxy is the address of the corresponding portion of the vector vv;
          Pxy is the address of the corresponding block of the matrix P;
@@ -443,45 +450,45 @@ static int PrecSolveBD(N_Vector cc, N_Vector cscale,
       Pxy = (data->P)[jx][jy];
       piv = (data->pivot)[jx][jy];
       denseGETRS(Pxy, NUM_SPECIES, piv, vxy);
-      
+
     } /* end of jy loop */
-    
+
   } /* end of jx loop */
-  
+
   return(0);
 }
 
 /*
- * Interaction rate function routine 
+ * Interaction rate function routine
  */
 
-static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                     void *user_data)
 {
   sunindextype i;
   realtype fac;
   UserData data;
-  
+
   data = (UserData)user_data;
-  
+
   for (i = 0; i<NUM_SPECIES; i++)
     ratesxy[i] = DotProd(NUM_SPECIES, cxy, acoef[i]);
-  
+
   fac = ONE + ALPHA * xx * yy;
-  
+
   for (i = 0; i < NUM_SPECIES; i++)
     ratesxy[i] = cxy[i] * ( bcoef[i] * fac + ratesxy[i] );
 }
 
 /*
- * Dot product routine for realtype arrays 
+ * Dot product routine for realtype arrays
  */
 
 static realtype DotProd(sunindextype size, realtype *x1, realtype *x2)
 {
   sunindextype i;
   realtype *xx1, *xx2, temp = ZERO;
-  
+
   xx1 = x1; xx2 = x2;
   for (i = 0; i < size; i++) temp += (*xx1++) * (*xx2++);
 
@@ -495,7 +502,7 @@ static realtype DotProd(sunindextype size, realtype *x1, realtype *x2)
  */
 
 /*
- * Allocate memory for data structure of type UserData 
+ * Allocate memory for data structure of type UserData
  */
 
 static UserData AllocUserData(void)
@@ -504,7 +511,7 @@ static UserData AllocUserData(void)
   UserData data;
 
   data = (UserData) malloc(sizeof *data);
-  
+
   for (jx=0; jx < MX; jx++) {
     for (jy=0; jy < MY; jy++) {
       (data->P)[jx][jy] = newDenseMat(NUM_SPECIES, NUM_SPECIES);
@@ -516,12 +523,12 @@ static UserData AllocUserData(void)
   bcoef = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
   cox   = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
   coy   = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
-  
+
   return(data);
 }
 
 /*
- * Load problem constants in data 
+ * Load problem constants in data
  */
 
 static void InitUserData(UserData data)
@@ -575,7 +582,7 @@ static void InitUserData(UserData data)
 }
 
 /*
- * Free data memory 
+ * Free data memory
  */
 
 static void FreeUserData(UserData data)
@@ -593,12 +600,12 @@ static void FreeUserData(UserData data)
   free(bcoef);
   free(cox);
   free(coy);
-  N_VDestroy_Serial(data->rates);
+  N_VDestroy(data->rates);
   free(data);
 }
 
 /*
- * Set initial conditions in cc 
+ * Set initial conditions in cc
  */
 
 static void SetInitialProfiles(N_Vector cc, N_Vector sc)
@@ -606,7 +613,7 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   int i, jx, jy;
   realtype *cloc, *sloc;
   realtype  ctemp[NUM_SPECIES], stemp[NUM_SPECIES];
-  
+
   /* Initialize arrays ctemp and stemp used in the loading process */
   for (i = 0; i < NUM_SPECIES/2; i++) {
     ctemp[i] = PREYIN;
@@ -630,11 +637,11 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   }
 }
 
-/* 
+/*
  * Print first lines of output (problem description)
  */
 
-static void PrintHeader(int globalstrategy, int maxl, int maxlrst, 
+static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
                         realtype fnormtol, realtype scsteptol)
 {
   printf("\nPredator-prey test problem --  KINSol (serial version)\n\n");
@@ -647,7 +654,7 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
          maxl, maxlrst);
   printf("Preconditioning uses interaction-only block-diagonal matrix\n");
   printf("Positivity constraints imposed on all components \n");
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  fnormtol = %Lg   scsteptol = %Lg\n",
          fnormtol, scsteptol);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
@@ -659,30 +666,30 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
 #endif
 
   printf("\nInitial profile of concentration\n");
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
-  printf("At all mesh points:  %Lg %Lg %Lg   %Lg %Lg %Lg\n", 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("At all mesh points:  %Lg %Lg %Lg   %Lg %Lg %Lg\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
-  printf("At all mesh points:  %g %g %g   %g %g %g\n", 
+  printf("At all mesh points:  %g %g %g   %g %g %g\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #else
-  printf("At all mesh points:  %g %g %g   %g %g %g\n", 
+  printf("At all mesh points:  %g %g %g   %g %g %g\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #endif
 }
 
 /*
- * Print sampled values of current cc 
+ * Print sampled values of current cc
  */
 
 static void PrintOutput(N_Vector cc)
 {
   int is, jx, jy;
   realtype *ct;
-  
+
   jy = 0; jx = 0;
   ct = IJ_Vptr(cc,jx,jy);
   printf("\nAt bottom left:");
@@ -698,7 +705,7 @@ static void PrintOutput(N_Vector cc)
     printf(" %g",ct[is]);
 #endif
   }
-  
+
   jy = MY-1; jx = MX-1;
   ct = IJ_Vptr(cc,jx,jy);
   printf("\n\nAt top right:");
@@ -718,7 +725,7 @@ static void PrintOutput(N_Vector cc)
 }
 
 /*
- * Print final statistics contained in iopt 
+ * Print final statistics contained in iopt
  */
 
 static void PrintFinalStats(void *kmem)
@@ -730,16 +737,16 @@ static void PrintFinalStats(void *kmem)
   check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
   flag = KINGetNumFuncEvals(kmem, &nfe);
   check_flag(&flag, "KINGetNumFuncEvals", 1);
-  flag = KINSpilsGetNumLinIters(kmem, &nli);
-  check_flag(&flag, "KINSpilsGetNumLinIters", 1);
-  flag = KINSpilsGetNumPrecEvals(kmem, &npe);
-  check_flag(&flag, "KINSpilsGetNumPrecEvals", 1);
-  flag = KINSpilsGetNumPrecSolves(kmem, &nps);
-  check_flag(&flag, "KINSpilsGetNumPrecSolves", 1);
-  flag = KINSpilsGetNumConvFails(kmem, &ncfl);
-  check_flag(&flag, "KINSpilsGetNumConvFails", 1);
-  flag = KINSpilsGetNumFuncEvals(kmem, &nfeSG);
-  check_flag(&flag, "KINSpilsGetNumFuncEvals", 1);
+  flag = KINGetNumLinIters(kmem, &nli);
+  check_flag(&flag, "KINGetNumLinIters", 1);
+  flag = KINGetNumPrecEvals(kmem, &npe);
+  check_flag(&flag, "KINGetNumPrecEvals", 1);
+  flag = KINGetNumPrecSolves(kmem, &nps);
+  check_flag(&flag, "KINGetNumPrecSolves", 1);
+  flag = KINGetNumLinConvFails(kmem, &ncfl);
+  check_flag(&flag, "KINGetNumLinConvFails", 1);
+  flag = KINGetNumLinFuncEvals(kmem, &nfeSG);
+  check_flag(&flag, "KINGetNumLinFuncEvals", 1);
 
   printf("Final Statistics.. \n");
   printf("nni    = %5ld    nli   = %5ld\n", nni, nli);
@@ -755,7 +762,7 @@ static void PrintFinalStats(void *kmem)
  *    opt == 1 means SUNDIALS function returns a flag so check if
  *             flag >= 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
 static int check_flag(void *flagvalue, const char *funcname, int opt)
@@ -764,7 +771,7 @@ static int check_flag(void *flagvalue, const char *funcname, int opt)
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
+    fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
