@@ -1,9 +1,9 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -13,7 +13,7 @@
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the testing routine to check the SUNLinSol LapackDense
- * module implementation. 
+ * module implementation.
  * -----------------------------------------------------------------
  */
 
@@ -30,7 +30,7 @@
 /* ----------------------------------------------------------------------
  * SUNLinSol_LapackDense Testing Routine
  * --------------------------------------------------------------------*/
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
   int             fails = 0;          /* counter for test failures  */
   sunindextype    cols, rows;         /* matrix columns, rows       */
@@ -47,10 +47,10 @@ int main(int argc, char *argv[])
     return(-1);
   }
 
-  cols = atol(argv[1]); 
+  cols = (sunindextype) atol(argv[1]);
   if (cols <= 0) {
     printf("ERROR: number of matrix columns must be a positive integer \n");
-    return(-1); 
+    return(-1);
   }
 
   rows = cols;
@@ -73,7 +73,7 @@ int main(int argc, char *argv[])
   for (j=0; j<cols; j++) {
     colj = SUNDenseMatrix_Column(A, j);
     for (k=0; k<rows; k++)
-      colj[k] = (realtype) rand() / (realtype) RAND_MAX / cols;    
+      colj[k] = (realtype) rand() / (realtype) RAND_MAX / cols;
   }
 
   /* Create anti-identity matrix */
@@ -82,14 +82,14 @@ int main(int argc, char *argv[])
     colj = SUNDenseMatrix_Column(I,j);
     colj[k] = 1;
     j = j-1;
-  }    
-  
+  }
+
   /* Add anti-identity to ensure the solver needs to do row-swapping */
   for (k=0; k<rows; k++){
     for(j=0; j<cols; j++){
       colj = SUNDenseMatrix_Column(A,j);
       colIj = SUNDenseMatrix_Column(I,j);
-      colj[k]  = colj[k] + colIj[k]; 
+      colj[k]  = colj[k] + colIj[k];
    }
   }
 
@@ -97,7 +97,7 @@ int main(int argc, char *argv[])
   xdata = N_VGetArrayPointer(x);
   for (j=0; j<cols; j++) {
     xdata[j] = (realtype) rand() / (realtype) RAND_MAX;
-  } 
+  }
 
   /* copy A and x into B and y to print in case of solver failure */
   SUNMatCopy(A, B);
@@ -112,13 +112,14 @@ int main(int argc, char *argv[])
 
   /* Create dense linear solver */
   LS = SUNLinSol_LapackDense(x, A);
-  
+
   /* Run Tests */
   fails += Test_SUNLinSolInitialize(LS, 0);
   fails += Test_SUNLinSolSetup(LS, A, 0);
-  fails += Test_SUNLinSolSolve(LS, A, x, b, 10*UNIT_ROUNDOFF, 0);
- 
+  fails += Test_SUNLinSolSolve(LS, A, x, b, 100*UNIT_ROUNDOFF, SUNTRUE, 0);
+
   fails += Test_SUNLinSolGetType(LS, SUNLINEARSOLVER_DIRECT, 0);
+  fails += Test_SUNLinSolGetID(LS, SUNLINEARSOLVER_LAPACKDENSE, 0);
   fails += Test_SUNLinSolLastFlag(LS, 0);
   fails += Test_SUNLinSolSpace(LS, 0);
 
@@ -141,8 +142,10 @@ int main(int argc, char *argv[])
   SUNLinSolFree(LS);
   SUNMatDestroy(A);
   SUNMatDestroy(B);
+  SUNMatDestroy(I);
   N_VDestroy(x);
   N_VDestroy(y);
+  N_VDestroy(b);
 
   return(fails);
 }
@@ -155,14 +158,14 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
   int failure = 0;
   sunindextype i, local_length;
   realtype *Xdata, *Ydata, maxerr;
-  
+
   Xdata = N_VGetArrayPointer(X);
   Ydata = N_VGetArrayPointer(Y);
   local_length = N_VGetLength_Serial(X);
-  
+
   /* check vector data */
   for(i=0; i < local_length; i++)
-    failure += FNEQ(Xdata[i], Ydata[i], tol);
+    failure += SUNRCompareTol(Xdata[i], Ydata[i], tol);
 
   if (failure > ZERO) {
     maxerr = ZERO;
@@ -175,3 +178,7 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
   else
     return(0);
 }
+
+void sync_device()
+{
+}
