@@ -1,23 +1,19 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the testing routine to check the SUNLinSol SPGMR module 
- * implementation. 
+ * This is the testing routine to check the SUNLinSol SPGMR module
+ * implementation.
  * -----------------------------------------------------------------
  */
 
@@ -51,7 +47,7 @@ typedef struct {
   sunindextype Nloc;  /* local problem size */
   N_Vector d;         /* matrix diagonal */
   N_Vector s1;        /* scaling vectors supplied to SPGMR */
-  N_Vector s2;        
+  N_Vector s2;
   MPI_Comm comm;      /* communicator object */
   int myid;           /* MPI process ID */
   int nprocs;         /* total number of MPI processes */
@@ -73,7 +69,7 @@ static realtype urand();
 sunindextype local_problem_size;
 
 /* ----------------------------------------------------------------------
- * SUNSPGMR Linear Solver Testing Routine
+ * SUNLinSol_SPGMR Linear Solver Testing Routine
  *
  * We run multiple tests to exercise this solver:
  * 1. simple tridiagonal system (no preconditioning)
@@ -83,27 +79,26 @@ sunindextype local_problem_size;
  * 5. tridiagonal system w/ scale vector s2 (no preconditioning)
  * 6. tridiagonal system w/ scale vector s2 (Jacobi preconditioning)
  *
- * Note: We construct a tridiagonal matrix Ahat, a random solution xhat, 
- *       and a corresponding rhs vector bhat = Ahat*xhat, such that each 
- *       of these is unit-less.  To test row/column scaling, we use the 
- *       matrix A = S1-inverse Ahat S2, rhs vector b = S1-inverse bhat, 
- *       and solution vector x = (S2-inverse) xhat; hence the linear 
- *       system has rows scaled by S1-inverse and columns scaled by S2, 
- *       where S1 and S2 are the diagonal matrices with entries from the 
- *       vectors s1 and s2, the 'scaling' vectors supplied to SPGMR 
- *       having strictly positive entries.  When this is combined with 
- *       preconditioning, assume that Phat is the desired preconditioner 
+ * Note: We construct a tridiagonal matrix Ahat, a random solution xhat,
+ *       and a corresponding rhs vector bhat = Ahat*xhat, such that each
+ *       of these is unit-less.  To test row/column scaling, we use the
+ *       matrix A = S1-inverse Ahat S2, rhs vector b = S1-inverse bhat,
+ *       and solution vector x = (S2-inverse) xhat; hence the linear
+ *       system has rows scaled by S1-inverse and columns scaled by S2,
+ *       where S1 and S2 are the diagonal matrices with entries from the
+ *       vectors s1 and s2, the 'scaling' vectors supplied to SPGMR
+ *       having strictly positive entries.  When this is combined with
+ *       preconditioning, assume that Phat is the desired preconditioner
  *       for Ahat, then our preconditioning matrix P \approx A should be
  *         left prec:  P-inverse \approx S1-inverse Ahat-inverse S1
  *         right prec:  P-inverse \approx S2-inverse Ahat-inverse S2.
- *       Here we use a diagonal preconditioner D, so the S*-inverse 
+ *       Here we use a diagonal preconditioner D, so the S*-inverse
  *       and S* in the product cancel one another.
  * --------------------------------------------------------------------*/
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
   int             fails=0;          /* counter for test failures */
   int             passfail=0;       /* overall pass/fail flag    */
-  int             mpierr;           /* mpi error flag            */
   SUNLinearSolver LS;               /* linear solver object      */
   N_Vector        xhat, x, b;       /* test vectors              */
   UserData        ProbData;         /* problem data structure    */
@@ -132,31 +127,31 @@ int main(int argc, char *argv[])
     printf("  timing output flag should be 0 or 1 \n");
     return 1;
   }
-  ProbData.Nloc = atol(argv[1]);
+  ProbData.Nloc = (sunindextype) atol(argv[1]);
   local_problem_size = ProbData.Nloc;
   if (ProbData.Nloc <= 0) {
     printf("ERROR: local problem size must be a positive integer\n");
-    return 1; 
+    return 1;
   }
   gstype = atoi(argv[2]);
   if ((gstype < 1) || (gstype > 2)) {
     printf("ERROR: Gram-Schmidt orthogonalization type must be either 1 or 2\n");
-    return 1; 
+    return 1;
   }
   pretype = atoi(argv[3]);
   if ((pretype < 1) || (pretype > 2)) {
     printf("ERROR: Preconditioning type must be either 1 or 2\n");
-    return 1; 
+    return 1;
   }
   maxl = atoi(argv[4]);
   if (maxl <= 0) {
     printf("ERROR: Maximum Krylov subspace dimension must be a positive integer\n");
-    return 1; 
+    return 1;
   }
   tol = atof(argv[5]);
   if (tol <= ZERO) {
     printf("ERROR: Solver tolerance must be a positive real number\n");
-    return 1; 
+    return 1;
   }
   print_timing = atoi(argv[6]);
   SetTiming(print_timing);
@@ -169,10 +164,10 @@ int main(int argc, char *argv[])
     printf("  Gram-Schmidt orthogonalization type = %i\n", gstype);
     printf("  Preconditioning type = %i\n", pretype);
     printf("  Maximum Krylov subspace dimension = %i\n", maxl);
-    printf("  Solver Tolerance = %"GSYM"\n", tol);
+    printf("  Solver Tolerance = %g\n", tol);
     printf("  timing output flag = %i\n\n", print_timing);
   }
-  
+
   /* Create vectors */
   x = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
                       ProbData.nprocs * ProbData.Nloc);
@@ -195,31 +190,33 @@ int main(int argc, char *argv[])
 
   /* Fill xhat vector with uniform random data in [1,2] */
   vecdata = N_VGetArrayPointer(xhat);
-  for (i=0; i<ProbData.Nloc; i++) 
+  for (i=0; i<ProbData.Nloc; i++)
     vecdata[i] = ONE + urand();
 
   /* Fill Jacobi vector with matrix diagonal */
   N_VConst(FIVE, ProbData.d);
-  
+
   /* Create SPGMR linear solver */
-  LS = SUNSPGMR(x, pretype, maxl);
-  fails += Test_SUNLinSolGetType(LS, SUNLINEARSOLVER_ITERATIVE, 
+  LS = SUNLinSol_SPGMR(x, pretype, maxl);
+  fails += Test_SUNLinSolGetType(LS, SUNLINEARSOLVER_ITERATIVE,
                                  ProbData.myid);
+  fails += Test_SUNLinSolGetID(LS, SUNLINEARSOLVER_SPGMR, ProbData.myid);
   fails += Test_SUNLinSolSetATimes(LS, &ProbData, ATimes, ProbData.myid);
   fails += Test_SUNLinSolSetPreconditioner(LS, &ProbData, PSetup,
                                            PSolve, ProbData.myid);
   fails += Test_SUNLinSolSetScalingVectors(LS, ProbData.s1, ProbData.s2,
                                            ProbData.myid);
+  fails += Test_SUNLinSolSetZeroGuess(LS, ProbData.myid);
   fails += Test_SUNLinSolInitialize(LS, ProbData.myid);
   fails += Test_SUNLinSolSpace(LS, ProbData.myid);
-  fails += SUNSPGMRSetGSType(LS, gstype);  
+  fails += SUNLinSol_SPGMRSetGSType(LS, gstype);
   if (fails) {
-    printf("FAIL: SUNSPGMR module failed %i initialization tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module failed %i initialization tests\n\n", fails);
     return 1;
   } else if (ProbData.myid == 0)
-    printf("SUCCESS: SUNSPGMR module passed all initialization tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module passed all initialization tests\n\n");
+
 
-  
   /*** Test 1: simple Poisson-like solve (no preconditioning) ***/
 
   /* set scaling vectors */
@@ -234,23 +231,24 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, PREC_NONE);  
+  fails += SUNLinSol_SPGMRSetPrecType(LS, PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
   fails += Test_SUNLinSolResid(LS, ProbData.myid);
-  
+
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 1, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 1, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 1, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 1, passed all tests\n\n");
   }
 
-  
+
   /*** Test 2: simple Poisson-like solve (Jacobi preconditioning) ***/
 
   /* set scaling vectors */
@@ -265,9 +263,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, pretype);  
+  fails += SUNLinSol_SPGMRSetPrecType(LS, pretype);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -275,13 +274,13 @@ int main(int argc, char *argv[])
 
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 2, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 2, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 2, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 2, passed all tests\n\n");
   }
 
-  
+
   /*** Test 3: Poisson-like solve w/ scaled rows (no preconditioning) ***/
 
   /* set scaling vectors */
@@ -298,9 +297,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, PREC_NONE);  
+  fails += SUNLinSol_SPGMRSetPrecType(LS, PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -308,13 +308,13 @@ int main(int argc, char *argv[])
 
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 3, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 3, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 3, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 3, passed all tests\n\n");
   }
 
-  
+
   /*** Test 4: Poisson-like solve w/ scaled rows (Jacobi preconditioning) ***/
 
   /* set scaling vectors */
@@ -331,9 +331,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, pretype);  
+  fails += SUNLinSol_SPGMRSetPrecType(LS, pretype);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -341,13 +342,13 @@ int main(int argc, char *argv[])
 
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 4, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 4, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 4, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 4, passed all tests\n\n");
   }
 
-  
+
   /*** Test 5: Poisson-like solve w/ scaled columns (no preconditioning) ***/
 
   /* set scaling vectors */
@@ -364,9 +365,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, PREC_NONE);
+  fails += SUNLinSol_SPGMRSetPrecType(LS, PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -374,13 +376,13 @@ int main(int argc, char *argv[])
 
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 5, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 5, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 5, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 5, passed all tests\n\n");
   }
 
-  
+
   /*** Test 6: Poisson-like solve w/ scaled columns (Jacobi preconditioning) ***/
 
   /* set scaling vector, Jacobi solver vector */
@@ -397,9 +399,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNSPGMRSetPrecType(LS, pretype);  
+  fails += SUNLinSol_SPGMRSetPrecType(LS, pretype);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -407,15 +410,15 @@ int main(int argc, char *argv[])
 
   /* Print result */
   if (fails) {
-    printf("FAIL: SUNSPGMR module, problem 6, failed %i tests\n\n", fails);
+    printf("FAIL: SUNLinSol_SPGMR module, problem 6, failed %i tests\n\n", fails);
     passfail += 1;
   } else if (ProbData.myid == 0) {
-    printf("SUCCESS: SUNSPGMR module, problem 6, passed all tests\n\n");
+    printf("SUCCESS: SUNLinSol_SPGMR module, problem 6, passed all tests\n\n");
   }
 
   /* check if any other process failed */
-  mpierr = MPI_Allreduce(&passfail, &fails, 1, MPI_INT, MPI_MAX, ProbData.comm);
-  
+  (void) MPI_Allreduce(&passfail, &fails, 1, MPI_INT, MPI_MAX, ProbData.comm);
+
   /* Free solver and vectors */
   SUNLinSolFree(LS);
   N_VDestroy(x);
@@ -444,7 +447,7 @@ int ATimes(void* Data, N_Vector v_vec, N_Vector z_vec)
   UserData *ProbData;
   MPI_Request SendReqL, SendReqR, RecvReqL, RecvReqR;
   MPI_Status stat;
-  
+
   /* access user data structure and vector data */
   ProbData = (UserData *) Data;
   v = N_VGetArrayPointer(v_vec);
@@ -456,40 +459,31 @@ int ATimes(void* Data, N_Vector v_vec, N_Vector z_vec)
   s2 = N_VGetArrayPointer(ProbData->s2);
   if (check_flag(s2, "N_VGetArrayPointer", 0)) return 1;
   Nloc = ProbData->Nloc;
-  
-  /* MPI equivalent of realtype type */
-  #if defined(SUNDIALS_SINGLE_PRECISION)
-  #define REALTYPE_MPI_TYPE MPI_FLOAT
-  #elif defined(SUNDIALS_DOUBLE_PRECISION)
-  #define REALTYPE_MPI_TYPE MPI_DOUBLE
-  #elif defined(SUNDIALS_EXTENDED_PRECISION)
-  #define REALTYPE_MPI_TYPE MPI_LONG_DOUBLE
-  #endif
-  
+
   /* send/recv boundary data with neighbors */
   vL = vR = ZERO;
   vsL = v[0]*s2[0];
   vsR = v[Nloc-1]*s2[Nloc-1];
   if (ProbData->myid > 0) {                   /* left neighbor exists */
-    ierr = MPI_Irecv(&vL, 1, REALTYPE_MPI_TYPE, ProbData->myid-1,
+    ierr = MPI_Irecv(&vL, 1, MPI_SUNREALTYPE, ProbData->myid-1,
                      MPI_ANY_TAG, ProbData->comm, &RecvReqL);
     if (ierr != MPI_SUCCESS) return 1;
-    ierr = MPI_Isend(&vsL, 1, REALTYPE_MPI_TYPE, ProbData->myid-1,
+    ierr = MPI_Isend(&vsL, 1, MPI_SUNREALTYPE, ProbData->myid-1,
                      0, ProbData->comm, &SendReqL);
     if (ierr != MPI_SUCCESS) return 1;
   }
   if (ProbData->myid < ProbData->nprocs-1) {  /* right neighbor exists */
-    ierr = MPI_Irecv(&vR, 1, REALTYPE_MPI_TYPE, ProbData->myid+1,
+    ierr = MPI_Irecv(&vR, 1, MPI_SUNREALTYPE, ProbData->myid+1,
                      MPI_ANY_TAG, ProbData->comm, &RecvReqR);
     if (ierr != MPI_SUCCESS) return 1;
-    ierr = MPI_Isend(&vsR, 1, REALTYPE_MPI_TYPE, ProbData->myid+1,
+    ierr = MPI_Isend(&vsR, 1, MPI_SUNREALTYPE, ProbData->myid+1,
                      1, ProbData->comm, &SendReqR);
     if (ierr != MPI_SUCCESS) return 1;
   }
-    
-  
+
+
   /* iterate through interior of local domain, performing product */
-  for (i=1; i<Nloc-1; i++) 
+  for (i=1; i<Nloc-1; i++)
     z[i] = (-v[i-1]*s2[i-1] + FIVE*v[i]*s2[i] - v[i+1]*s2[i+1])/s1[i];
 
   /* wait on neighbor data to arrive */
@@ -501,18 +495,18 @@ int ATimes(void* Data, N_Vector v_vec, N_Vector z_vec)
     ierr = MPI_Wait(&RecvReqR, &stat);
     if (ierr != MPI_SUCCESS) return 1;
   }
-  
+
   /* perform product at subdomain boundaries (note: vL/vR are zero at boundary)*/
   z[0] = (-vL + FIVE*v[0]*s2[0] - v[1]*s2[1])/s1[0];
   z[Nloc-1] = (-v[Nloc-2]*s2[Nloc-2] + FIVE*v[Nloc-1]*s2[Nloc-1] - vR)/s1[Nloc-1];
-  
+
   /* return with success */
   return 0;
 }
-  
+
 /* preconditioner setup -- nothing to do here since everything is already stored */
 int PSetup(void* Data) { return 0; }
-  
+
 /* preconditioner solve */
 int PSolve(void* Data, N_Vector r_vec, N_Vector z_vec, realtype tol, int lr)
 {
@@ -520,7 +514,7 @@ int PSolve(void* Data, N_Vector r_vec, N_Vector z_vec, realtype tol, int lr)
   realtype *r, *z, *d;
   sunindextype i;
   UserData *ProbData;
-  
+
   /* access user data structure and vector data */
   ProbData = (UserData *) Data;
   r = N_VGetArrayPointer(r_vec);
@@ -529,9 +523,9 @@ int PSolve(void* Data, N_Vector r_vec, N_Vector z_vec, realtype tol, int lr)
   if (check_flag(z, "N_VGetArrayPointer", 0)) return 1;
   d = N_VGetArrayPointer(ProbData->d);
   if (check_flag(d, "N_VGetArrayPointer", 0)) return 1;
-  
+
   /* iterate through domain, performing Jacobi solve */
-  for (i=0; i<ProbData->Nloc; i++) 
+  for (i=0; i<ProbData->Nloc; i++)
     z[i] = r[i] / d[i];
 
   /* return with success */
@@ -577,24 +571,26 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
   int failure = 0;
   sunindextype i;
   realtype *Xdata, *Ydata, maxerr;
-  
+
   Xdata = N_VGetArrayPointer(X);
   Ydata = N_VGetArrayPointer(Y);
-  
+
   /* check vector data */
   for(i=0; i<local_problem_size; i++)
-    failure += FNEQ(Xdata[i], Ydata[i], FIVE*tol*SUNRabs(Xdata[i]));
+    failure += SUNRCompareTol(Xdata[i], Ydata[i], tol);
 
   if (failure > ZERO) {
     maxerr = ZERO;
     for(i=0; i < local_problem_size; i++)
       maxerr = SUNMAX(SUNRabs(Xdata[i]-Ydata[i])/SUNRabs(Xdata[i]), maxerr);
-    printf("check err failure: maxerr = %g (tol = %g)\n",
-	   maxerr, FIVE*tol);
+    printf("check err failure: maxerr = %"GSYM" (tol = %"GSYM")\n",
+	   maxerr, tol);
     return(1);
   }
   else
     return(0);
 }
 
-
+void sync_device()
+{
+}
