@@ -1,24 +1,20 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  *                David Gardner @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the testing routine to check the SUNMatrix Dense module 
- * implementation. 
+ * This is the testing routine to check the SUNMatrix Dense module
+ * implementation.
  * -----------------------------------------------------------------
  */
 
@@ -31,11 +27,20 @@
 #include <sundials/sundials_math.h>
 #include "test_sunmatrix.h"
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define GSYM "Lg"
+#define ESYM "Le"
+#define FSYM "Lf"
+#else
+#define GSYM "g"
+#define ESYM "e"
+#define FSYM "f"
+#endif
 
 /* ----------------------------------------------------------------------
  * Main SUNMatrix Testing Routine
  * --------------------------------------------------------------------*/
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
   int          fails = 0;        /* counter for test failures  */
   sunindextype matrows, matcols; /* vector length              */
@@ -51,22 +56,22 @@ int main(int argc, char *argv[])
     printf("ERROR: THREE (3) Input required: matrix rows, matrix cols, print timing \n");
     return(-1);
   }
-  
-  matrows = atol(argv[1]); 
+
+  matrows = (sunindextype) atol(argv[1]);
   if (matrows <= 0) {
     printf("ERROR: number of rows must be a positive integer \n");
-    return(-1); 
+    return(-1);
   }
-  
-  matcols = atol(argv[2]); 
+
+  matcols = (sunindextype) atol(argv[2]);
   if (matcols <= 0) {
     printf("ERROR: number of cols must be a positive integer \n");
-    return(-1); 
+    return(-1);
   }
 
   print_timing = atoi(argv[3]);
   SetTiming(print_timing);
-  
+
   square = (matrows == matcols) ? 1 : 0;
   printf("\nDense matrix test: size %ld by %ld\n\n",
          (long int) matrows, (long int) matcols);
@@ -76,14 +81,15 @@ int main(int argc, char *argv[])
   y = NULL;
   A = NULL;
   I = NULL;
-  
+
   /* Create vectors and matrices */
   x = N_VNew_Serial(matcols);
   y = N_VNew_Serial(matrows);
   A = SUNDenseMatrix(matrows, matcols);
+  I = NULL;
   if (square)
     I = SUNDenseMatrix(matrows, matcols);
-  
+
   /* Fill matrices and vectors */
   Adata = SUNDenseMatrix_Data(A);
   for(j=0; j < matcols; j++) {
@@ -110,15 +116,16 @@ int main(int argc, char *argv[])
     n = m + matcols - 1;
     ydata[i] = HALF*(n+1-m)*(n+m);
   }
-    
+
   /* SUNMatrix Tests */
   fails += Test_SUNMatGetID(A, SUNMATRIX_DENSE, 0);
   fails += Test_SUNMatClone(A, 0);
   fails += Test_SUNMatCopy(A, 0);
   fails += Test_SUNMatZero(A, 0);
-  fails += Test_SUNMatScaleAdd(A, I, 0);
-  if (square) 
+  if (square) {
+    fails += Test_SUNMatScaleAdd(A, I, 0);
     fails += Test_SUNMatScaleAddI(A, I, 0);
+  }
   fails += Test_SUNMatMatvec(A, x, y, 0);
   fails += Test_SUNMatSpace(A, 0);
 
@@ -140,8 +147,8 @@ int main(int argc, char *argv[])
   }
 
   /* Free vectors and matrices */
-  N_VDestroy_Serial(x);
-  N_VDestroy_Serial(y);
+  N_VDestroy(x);
+  N_VDestroy(y);
   SUNMatDestroy(A);
   if (square)
     SUNMatDestroy(I);
@@ -158,7 +165,7 @@ int check_matrix(SUNMatrix A, SUNMatrix B, realtype tol)
   realtype *Adata, *Bdata;
   sunindextype Aldata, Bldata;
   sunindextype i;
-  
+
   /* get data pointers */
   Adata = SUNDenseMatrix_Data(A);
   Bdata = SUNDenseMatrix_Data(B);
@@ -171,10 +178,10 @@ int check_matrix(SUNMatrix A, SUNMatrix B, realtype tol)
     printf(">>> ERROR: check_matrix: Different data array lengths \n");
     return(1);
   }
-  
+
   /* compare data */
   for(i=0; i < Aldata; i++){
-    failure += FNEQ(Adata[i], Bdata[i], tol);
+    failure += SUNRCompareTol(Adata[i], Bdata[i], tol);
   }
 
   if (failure > ZERO)
@@ -189,24 +196,24 @@ int check_matrix_entry(SUNMatrix A, realtype val, realtype tol)
   realtype *Adata;
   sunindextype Aldata;
   sunindextype i;
-  
+
   /* get data pointer */
   Adata = SUNDenseMatrix_Data(A);
 
   /* compare data */
   Aldata = SUNDenseMatrix_LData(A);
   for(i=0; i < Aldata; i++){
-    failure += FNEQ(Adata[i], val, tol);
+    failure += SUNRCompareTol(Adata[i], val, tol);
   }
 
   if (failure > ZERO) {
     printf("Check_matrix_entry failures:\n");
     for(i=0; i < Aldata; i++)
-      if (FNEQ(Adata[i], val, tol) != 0)
-        printf("  Adata[%ld] = %g != %g (err = %g)\n", (long int) i,
+      if (SUNRCompareTol(Adata[i], val, tol) != 0)
+        printf("  Adata[%ld] = %"GSYM" != %"GSYM" (err = %"GSYM")\n", (long int) i,
                Adata[i], val, SUNRabs(Adata[i]-val));
   }
-  
+
   if (failure > ZERO)
     return(1);
   else
@@ -225,8 +232,8 @@ int check_vector(N_Vector x, N_Vector y, realtype tol)
   ydata = N_VGetArrayPointer(y);
 
   /* check data lengths */
-  xldata = N_VGetLength_Serial(x);
-  yldata = N_VGetLength_Serial(y);
+  xldata = N_VGetLength(x);
+  yldata = N_VGetLength(y);
 
   if (xldata != yldata) {
     printf(">>> ERROR: check_vector: Different data array lengths \n");
@@ -235,16 +242,16 @@ int check_vector(N_Vector x, N_Vector y, realtype tol)
 
   /* check vector data */
   for(i=0; i < xldata; i++)
-    failure += FNEQ(xdata[i], ydata[i], tol);
+    failure += SUNRCompareTol(xdata[i], ydata[i], tol);
 
   if (failure > ZERO) {
     printf("Check_vector failures:\n");
     for(i=0; i < xldata; i++)
-      if (FNEQ(xdata[i], ydata[i], tol) != 0)
-        printf("  xdata[%ld] = %g != %g (err = %g)\n", (long int) i,
+      if (SUNRCompareTol(xdata[i], ydata[i], tol) != 0)
+        printf("  xdata[%ld] = %"GSYM" != %"GSYM" (err = %"GSYM")\n", (long int) i,
                xdata[i], ydata[i], SUNRabs(xdata[i]-ydata[i]));
   }
-  
+
   if (failure > ZERO)
     return(1);
   else
@@ -267,3 +274,9 @@ booleantype is_square(SUNMatrix A)
   else
     return SUNFALSE;
 }
+
+void sync_device(SUNMatrix A)
+{
+  /* not running on GPU, just return */
+  return;
+}
