@@ -1,24 +1,20 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  *                David Gardner @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the testing routine to check the SUNMatrix Sparse module 
- * implementation. 
+ * This is the testing routine to check the SUNMatrix Sparse module
+ * implementation.
  * -----------------------------------------------------------------
  */
 
@@ -36,13 +32,13 @@
 int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
                          N_Vector y, N_Vector z);
 int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y);
-
-
+int Test_SUNSparseMatrixToCSC(SUNMatrix A);
+int Test_SUNSparseMatrixToCSR(SUNMatrix A);
 
 /* ----------------------------------------------------------------------
  * Main SUNMatrix Testing Routine
  * --------------------------------------------------------------------*/
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
   int          fails=0;                    /* counter for test failures  */
   sunindextype matrows, matcols;           /* matrix dims                */
@@ -51,39 +47,39 @@ int main(int argc, char *argv[])
   realtype*    vecdata;                    /* pointers to vector data    */
   SUNMatrix    A, B, C, D, I;              /* test matrices              */
   realtype*    matdata;                    /* pointer to matrix data     */
-  sunindextype i, j, k, kstart, kend, N, uband, lband, suband;
+  sunindextype i, j, k, kstart, kend, N, uband, lband;
   sunindextype *colptrs, *rowindices;
   sunindextype *rowptrs, *colindices;
-  int          print_timing, square; 
+  int          print_timing, square;
 
   /* check input and set vector length */
   if (argc < 5){
     printf("ERROR: FOUR (4) Input required: matrix rows, matrix cols, matrix type (0/1), print timing \n");
     return(-1);
   }
-  
-  matrows = atol(argv[1]); 
+
+  matrows = (sunindextype) atol(argv[1]);
   if (matrows < 1) {
     printf("ERROR: number of rows must be a positive integer\n");
-    return(-1); 
+    return(-1);
   }
-  
-  matcols = atol(argv[2]); 
+
+  matcols = (sunindextype) atol(argv[2]);
   if (matcols < 1) {
     printf("ERROR: number of cols must be a positive integer\n");
-    return(-1); 
+    return(-1);
   }
-  
-  k = atol(argv[3]); 
+
+  k = (sunindextype) atol(argv[3]);
   if ((k != 0) && (k != 1)) {
     printf("ERROR: matrix type must be 0 or 1\n");
-    return(-1); 
+    return(-1);
   }
   mattype = (k == 0) ? CSC_MAT : CSR_MAT;
-  
+
   print_timing = atoi(argv[4]);
   SetTiming(print_timing);
-  
+
   square = (matrows == matcols) ? 1 : 0;
   printf("\nSparse matrix test: size %ld by %ld, type = %i\n\n",
          (long int) matrows, (long int) matcols, mattype);
@@ -97,10 +93,10 @@ int main(int argc, char *argv[])
   C = NULL;
   D = NULL;
   I = NULL;
-  
+
   /* check creating sparse matrix from dense matrix */
   B = SUNDenseMatrix(5,6);
-  
+
   matdata = SUNDenseMatrix_Data(B);
   matdata[2]  = RCONST(1.0);    /* [ 0 2 0 0 7 0 ] */
   matdata[5]  = RCONST(2.0);    /* [ 0 0 4 0 8 0 ] */
@@ -147,7 +143,7 @@ int main(int argc, char *argv[])
     SUNMatDestroy(C);
 
   } else {
-  
+
     /* Check CSC */
     D = SUNSparseMatrix(5, 6, 9, CSC_MAT);
     colptrs = SUNSparseMatrix_IndexPointers(D);
@@ -169,7 +165,7 @@ int main(int argc, char *argv[])
     colptrs[5] = 8;
     matdata[8] = RCONST(9.0);   rowindices[8] = 4;
     colptrs[6] = 9;
-  
+
     A = SUNSparseFromDenseMatrix(B, 1e-15, CSC_MAT);
     fails += check_matrix(A, D, 1e-15);
 
@@ -184,13 +180,12 @@ int main(int argc, char *argv[])
   }
   SUNMatDestroy(B);
 
-  
+
   /* check creating sparse matrix from banded matrix */
   N = 7;
   uband = 1;
   lband = 2;                                   /* B(i,j) = j + (j-i) */
-  suband = 3;
-  B = SUNBandMatrix(N, uband, lband, suband);  /* B = [  0  2  0  0  0  0  0 ] */
+  B = SUNBandMatrix(N, uband, lband);          /* B = [  0  2  0  0  0  0  0 ] */
   for (j=0; j<N; j++) {                        /*     [ -1  1  3  0  0  0  0 ] */
     matdata = SUNBandMatrix_Column(B, j);      /*     [ -2  0  2  4  0  0  0 ] */
     kstart = (j<uband) ? -j : -uband;          /*     [  0 -1  1  3  5  0  0 ] */
@@ -235,7 +230,7 @@ int main(int argc, char *argv[])
     matdata[19] = RCONST(4.0);   colindices[19] = 5;
     matdata[20] = RCONST(6.0);   colindices[20] = 6;
     rowptrs[ 7] = 21;
-  
+
     A = SUNSparseFromBandMatrix(B, ZERO, CSR_MAT);
     fails += check_matrix(A, C, 1e-15);
 
@@ -248,7 +243,7 @@ int main(int argc, char *argv[])
     SUNMatDestroy(C);
 
   } else {
-    
+
     /* Check CSC */
     D = SUNSparseMatrix(7, 7, 21, CSC_MAT);
     colptrs = SUNSparseMatrix_IndexPointers(D);
@@ -295,11 +290,12 @@ int main(int argc, char *argv[])
     SUNMatDestroy(A);
     SUNMatDestroy(D);
   }
-  
+
   SUNMatDestroy(B);
 
 
   /* Create/fill I matrix */
+  I = NULL;
   if (square) {
     I = SUNSparseMatrix(matrows, matcols, matcols, mattype);
     matdata    = SUNSparseMatrix_Data(I);
@@ -312,7 +308,7 @@ int main(int argc, char *argv[])
     }
     rowptrs[matrows] = matrows;
   }
-  
+
   /* Create/fill random dense matrices, create sparse from them */
   C = SUNDenseMatrix(matrows, matcols);
   D = SUNDenseMatrix(matrows, matcols);
@@ -330,13 +326,13 @@ int main(int argc, char *argv[])
   }
   A = SUNSparseFromDenseMatrix(C, ZERO, mattype);
   B = SUNSparseFromDenseMatrix(D, ZERO, mattype);
-  
+
   /* Create vectors and fill */
   x = N_VNew_Serial(matcols);
   y = N_VNew_Serial(matrows);
   z = N_VNew_Serial(matrows);
   vecdata = N_VGetArrayPointer(x);
-  for(i=0; i<matcols; i++) 
+  for(i=0; i<matcols; i++)
     vecdata[i] = (realtype) rand() / (realtype) RAND_MAX;
   if (SUNMatMatvec(C, x, y) != 0) {
     printf("FAIL: SUNMatrix module Dense matvec failure \n \n");
@@ -370,6 +366,11 @@ int main(int argc, char *argv[])
   }
   fails += Test_SUNMatMatvec(A, x, y, 0);
   fails += Test_SUNMatSpace(A, 0);
+  if (mattype == CSR_MAT) {
+    fails += Test_SUNSparseMatrixToCSC(A);
+  } else {
+    fails += Test_SUNSparseMatrixToCSR(A);
+  }
 
   /* Print result */
   if (fails) {
@@ -408,7 +409,7 @@ int main(int argc, char *argv[])
 
 /* ----------------------------------------------------------------------
  * Extra ScaleAdd tests for sparse matrices:
- *    A and B should have different sparsity patterns, and neither should 
+ *    A and B should have different sparsity patterns, and neither should
  *      contain sufficient storage to for their sum
  *    y should already equal A*x
  *    z should already equal B*x
@@ -419,7 +420,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
   int       failure;
   SUNMatrix C, D, E;
   N_Vector  u, v;
-  realtype  tol=1e-14;
+  realtype  tol=100*UNIT_ROUNDOFF;
 
   /* create clones for test */
   C = SUNMatClone(A);
@@ -429,7 +430,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
   /* test 1: add A to B (output must be enlarged) */
   failure = SUNMatCopy(A, C);            /* C = A */
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
     SUNMatDestroy(C);  N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
@@ -445,7 +446,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
            failure);
     SUNMatDestroy(C);  N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
-  N_VLinearSum(ONE,y,ONE,z,v);           /* v = y+z */ 
+  N_VLinearSum(ONE,y,ONE,z,v);           /* v = y+z */
   failure = check_vector(u, v, tol);     /* u ?= v */
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAdd2 check 1 \n");
@@ -469,33 +470,33 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
   }
   else {
     printf("    PASSED test -- SUNMatScaleAdd2 check 1 \n");
-  }    
+  }
 
   /* test 2: add A to a matrix with sufficient but misplaced storage */
   D = SUNMatClone(A);
   failure = SUNSparseMatrix_Reallocate(D, SM_NNZ_S(A)+SM_NNZ_S(B));
   failure = SUNMatCopy(A, D);            /* D = A */
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
-    SUNMatDestroy(C);  SUNMatDestroy(D);  
+    SUNMatDestroy(C);  SUNMatDestroy(D);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
   failure = SUNMatScaleAdd(ONE, D, B);   /* D = A+B */
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAdd returned %d \n",
            failure);
-    SUNMatDestroy(C);  SUNMatDestroy(D);  
+    SUNMatDestroy(C);  SUNMatDestroy(D);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
   failure = SUNMatMatvec(D, x, u);       /* u = Cx = Ax+Bx */
   if (failure) {
     printf(">>> FAILED test -- SUNMatMatvec returned %d \n",
            failure);
-    SUNMatDestroy(C);  SUNMatDestroy(D);  
+    SUNMatDestroy(C);  SUNMatDestroy(D);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
-  N_VLinearSum(ONE,y,ONE,z,v);           /* v = y+z */ 
+  N_VLinearSum(ONE,y,ONE,z,v);           /* v = y+z */
   failure = check_vector(u, v, tol);     /* u ?= v */
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAdd2 check 2 \n");
@@ -515,19 +516,19 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
     N_VPrint_Serial(u);
     printf("\nv =\n");
     N_VPrint_Serial(v);
-    SUNMatDestroy(C);  SUNMatDestroy(D);  
+    SUNMatDestroy(C);  SUNMatDestroy(D);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
   else {
     printf("    PASSED test -- SUNMatScaleAdd2 check 2 \n");
-  }    
+  }
 
 
   /* test 3: add A to a matrix with the appropriate structure already in place */
   E = SUNMatClone(C);
   failure = SUNMatCopy(C, E);                /* E = A + B */
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
     SUNMatDestroy(C);  SUNMatDestroy(D);  SUNMatDestroy(E);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
@@ -546,7 +547,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
     SUNMatDestroy(C);  SUNMatDestroy(D);  SUNMatDestroy(E);
     N_VDestroy(u);  N_VDestroy(v);  return(1);
   }
-  N_VLinearSum(NEG_ONE,y,ZERO,z,v);          /* v = -y */ 
+  N_VLinearSum(NEG_ONE,y,ZERO,z,v);          /* v = -y */
   failure = check_vector(u, v, tol);         /* v ?= u */
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAdd2 check 3 \n");
@@ -572,7 +573,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
   else {
     printf("    PASSED test -- SUNMatScaleAdd2 check 3 \n");
   }
-  
+
   SUNMatDestroy(C);
   SUNMatDestroy(D);
   SUNMatDestroy(E);
@@ -583,7 +584,7 @@ int Test_SUNMatScaleAdd2(SUNMatrix A, SUNMatrix B, N_Vector x,
 
 /* ----------------------------------------------------------------------
  * Extra ScaleAddI tests for sparse matrices:
- *    A should not contain values on the diagonal, nor should it contain 
+ *    A should not contain values on the diagonal, nor should it contain
  *      sufficient storage to add those in
  *    y should already equal A*x
  * --------------------------------------------------------------------*/
@@ -592,7 +593,7 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   int       failure;
   SUNMatrix B, C, D;
   N_Vector  w, z;
-  realtype  tol=1e-14;
+  realtype  tol=200*UNIT_ROUNDOFF;
 
   /* create clones for test */
   B = SUNMatClone(A);
@@ -602,7 +603,7 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   /* test 1: add I to a matrix with insufficient storage */
   failure = SUNMatCopy(A, B);
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
     SUNMatDestroy(B);  N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
@@ -618,7 +619,7 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
            failure);
     SUNMatDestroy(B);  N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
-  N_VLinearSum(ONE,x,NEG_ONE,y,w);
+  N_VLinearSum(ONE,x,NEG_ONE,y,w);  /* B x = (I - A) x = x - Ax = x - y = w */
   failure = check_vector(z, w, tol);
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAddI2 check 1 \n");
@@ -634,7 +635,7 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   }
   else {
     printf("    PASSED test -- SUNMatScaleAddI2 check 1 \n");
-  }    
+  }
 
   /* test 2: add I to a matrix with sufficient but misplaced
      storage */
@@ -642,23 +643,23 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   failure = SUNSparseMatrix_Reallocate(C, SM_NNZ_S(A)+SM_ROWS_S(A));
   failure = SUNMatCopy(A, C);
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
-    SUNMatDestroy(B);  SUNMatDestroy(C);  
+    SUNMatDestroy(B);  SUNMatDestroy(C);
     N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
   failure = SUNMatScaleAddI(NEG_ONE, C);   /* C = I-A */
   if (failure) {
     printf(">>> FAILED test -- SUNMatScaleAddI returned %d \n",
            failure);
-    SUNMatDestroy(B);  SUNMatDestroy(C);  
+    SUNMatDestroy(B);  SUNMatDestroy(C);
     N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
   failure = SUNMatMatvec(C, x, z);
   if (failure) {
     printf(">>> FAILED test -- SUNMatMatvec returned %d \n",
            failure);
-    SUNMatDestroy(B);  SUNMatDestroy(C);  
+    SUNMatDestroy(B);  SUNMatDestroy(C);
     N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
   N_VLinearSum(ONE,x,NEG_ONE,y,w);
@@ -673,19 +674,19 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
     N_VPrint_Serial(z);
     printf("\nw =\n");
     N_VPrint_Serial(w);
-    SUNMatDestroy(B);  SUNMatDestroy(C);  
+    SUNMatDestroy(B);  SUNMatDestroy(C);
     N_VDestroy(z);  N_VDestroy(w);  return(1);
   }
   else {
     printf("    PASSED test -- SUNMatScaleAddI2 check 2 \n");
-  }    
+  }
 
 
   /* test 3: add I to a matrix with appropriate structure already in place */
   D = SUNMatClone(C);
   failure = SUNMatCopy(C, D);
   if (failure) {
-    printf(">>> FAILED test -- SUNMatCopy returned %d \n", 
+    printf(">>> FAILED test -- SUNMatCopy returned %d \n",
            failure);
     SUNMatDestroy(B);  SUNMatDestroy(C);  SUNMatDestroy(D);
     N_VDestroy(z);  N_VDestroy(w);  return(1);
@@ -721,7 +722,7 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   else {
     printf("    PASSED test -- SUNMatScaleAddI2 check 3 \n");
   }
-  
+
   SUNMatDestroy(B);
   SUNMatDestroy(C);
   SUNMatDestroy(D);
@@ -730,6 +731,79 @@ int Test_SUNMatScaleAddI2(SUNMatrix A, N_Vector x, N_Vector y)
   return(0);
 }
 
+int Test_SUNSparseMatrixToCSR(SUNMatrix A)
+{
+  int       failure;
+  SUNMatrix csc, csr;
+  realtype  tol=200*UNIT_ROUNDOFF;
+
+  failure = SUNSparseMatrix_ToCSR(A, &csr);
+
+  if (failure) {
+    printf(">>> FAILED test -- SUNSparseMatrix_ToCSR returned nonzero\n");
+    SUNMatDestroy(csr);
+    return(1);
+  }
+
+  /* check entries */
+  if (SUNSparseMatrix_ToCSC(csr, &csc)) {
+    printf(">>> FAILED test -- SUNSparseMatrix_ToCSC returned nonzero\n");
+    SUNMatDestroy(csr);
+    return(1);
+  }
+
+  if (check_matrix(A, csc, tol)) {
+    printf(">>> FAILED test --  Test_SUNSparseMatrixToCSR check_matrix failed\n");
+    SUNMatDestroy(csr);
+    SUNMatDestroy(csc);
+    return(1);
+  }
+
+  printf("    PASSED test -- SUNSparseMatrixToCSR\n");
+
+  SUNMatDestroy(csr);
+  SUNMatDestroy(csc);
+
+  return(0);
+}
+
+int Test_SUNSparseMatrixToCSC(SUNMatrix A)
+{
+  int       failure;
+  SUNMatrix csc=NULL, csr=NULL;
+  realtype  tol=200*UNIT_ROUNDOFF;
+
+  failure = SUNSparseMatrix_ToCSC(A, &csc);
+
+  if (failure) {
+    printf(">>> FAILED test -- SUNSparseMatrix_ToCSC returned nonzero\n");
+    SUNMatDestroy(csr);
+    return(1);
+  }
+
+  /* check entries */
+  if (SUNSparseMatrix_ToCSR(csc, &csr)) {
+    printf(">>> FAILED test -- SUNSparseMatrix_ToCSR returned nonzero\n");
+    SUNMatDestroy(csc);
+    return(1);
+  }
+
+  if (check_matrix(A, csr, tol)) {
+    printf(">>> FAILED test --  Test_SUNSparseMatrixToCSC check_martrix failed\n");
+    SUNMatDestroy(csr);
+    SUNMatDestroy(csc);
+    return(1);
+  }
+
+  printf("    PASSED test -- SUNSparseMatrixToCSC\n");
+
+  SUNMatDestroy(csr);
+  SUNMatDestroy(csc);
+
+  return(0);
+}
+
+
 /* ----------------------------------------------------------------------
  * Check matrix
  * --------------------------------------------------------------------*/
@@ -740,14 +814,14 @@ int check_matrix(SUNMatrix A, SUNMatrix B, realtype tol)
   sunindextype *Aindexptrs, *Bindexptrs;
   sunindextype *Aindexvals, *Bindexvals;
   sunindextype i, ANP, BNP, Annz, Bnnz;
-  
+
   /* get matrix pointers */
   Adata = SUNSparseMatrix_Data(A);
   Aindexptrs = SUNSparseMatrix_IndexPointers(A);
   Aindexvals = SUNSparseMatrix_IndexValues(A);
   ANP = SUNSparseMatrix_NP(A);
   Annz = Aindexptrs[ANP];
-  
+
   Bdata = SUNSparseMatrix_Data(B);
   Bindexptrs = SUNSparseMatrix_IndexPointers(B);
   Bindexvals = SUNSparseMatrix_IndexValues(B);
@@ -783,22 +857,22 @@ int check_matrix(SUNMatrix A, SUNMatrix B, realtype tol)
   }
 
   /* compare sparsity patterns */
-  for (i=0; i<ANP; i++) 
+  for (i=0; i<ANP; i++)
     failure += (Aindexptrs[i] != Bindexptrs[i]);
   if (failure > ZERO) {
     printf(">>> ERROR: check_matrix: Different indexptrs \n");
     return(1);
   }
-  for (i=0; i<Annz; i++) 
+  for (i=0; i<Annz; i++)
     failure += (Aindexvals[i] != Bindexvals[i]);
   if (failure > ZERO) {
     printf(">>> ERROR: check_matrix: Different indexvals \n");
     return(1);
   }
-  
+
   /* compare matrix values */
   for(i=0; i<Annz; i++)
-    failure += FNEQ(Adata[i], Bdata[i], tol);
+    failure += SUNRCompareTol(Adata[i], Bdata[i], tol);
   if (failure > ZERO) {
     printf(">>> ERROR: check_matrix: Different entries \n");
     return(1);
@@ -813,7 +887,7 @@ int check_matrix_entry(SUNMatrix A, realtype val, realtype tol)
   realtype *Adata;
   sunindextype *indexptrs;
   sunindextype i, NP;
-  
+
   /* get data pointer */
   Adata = SUNSparseMatrix_Data(A);
 
@@ -821,7 +895,7 @@ int check_matrix_entry(SUNMatrix A, realtype val, realtype tol)
   indexptrs = SUNSparseMatrix_IndexPointers(A);
   NP = SUNSparseMatrix_NP(A);
   for(i=0; i < indexptrs[NP]; i++){
-    failure += FNEQ(Adata[i], val, tol);
+    failure += SUNRCompareTol(Adata[i], val, tol);
   }
 
   if (failure > ZERO)
@@ -852,7 +926,7 @@ int check_vector(N_Vector x, N_Vector y, realtype tol)
 
   /* check vector data */
   for(i=0; i < xldata; i++){
-    failure += FNEQ(xdata[i], ydata[i], tol);
+    failure += SUNRCompareTol(xdata[i], ydata[i], tol);
   }
 
   if (failure > ZERO)
@@ -877,3 +951,9 @@ booleantype is_square(SUNMatrix A)
   else
     return SUNFALSE;
 }
+
+void sync_device(SUNMatrix A)
+{
+  /* not running on GPU, just return */
+  return;
+}
