@@ -5,7 +5,7 @@
  *                and Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -34,11 +34,11 @@
 #define TWO          RCONST(2.0)
 
 /* Prototypes of CVBandPrecSetup and CVBandPrecSolve */
-static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                           booleantype jok, booleantype *jcurPtr, 
+static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                           booleantype jok, booleantype *jcurPtr,
                            realtype gamma, void *bp_data);
-static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                           N_Vector r, N_Vector z, 
+static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                           N_Vector r, N_Vector z,
                            realtype gamma, realtype delta,
                            int lr, void *bp_data);
 
@@ -46,16 +46,16 @@ static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
 static int CVBandPrecFree(CVodeMem cv_mem);
 
 /* Prototype for difference quotient Jacobian calculation routine */
-static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y, 
+static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
                         N_Vector fy, N_Vector ftemp, N_Vector ytemp);
 
 
 /*-----------------------------------------------------------------
   Initialization, Free, and Get Functions
-  NOTE: The band linear solver assumes a serial/OpenMP/Pthreads 
-        implementation of the NVECTOR package. Therefore, 
-        CVBandPrecInit will first test for a compatible N_Vector 
-        internal representation by checking that the function 
+  NOTE: The band linear solver assumes a serial/OpenMP/Pthreads
+        implementation of the NVECTOR package. Therefore,
+        CVBandPrecInit will first test for a compatible N_Vector
+        internal representation by checking that the function
         N_VGetArrayPointer exists.
   -----------------------------------------------------------------*/
 int CVBandPrecInit(void *cvode_mem, sunindextype N,
@@ -149,7 +149,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBANDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBANDPRE",
                     "CVBandPrecInit", MSGBP_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -161,7 +161,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     SUNMatDestroy(pdata->savedJ);
     N_VDestroy(pdata->tmp1);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBANDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBANDPRE",
                     "CVBandPrecInit", MSGBP_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -175,11 +175,11 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     N_VDestroy(pdata->tmp1);
     N_VDestroy(pdata->tmp2);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVBANDPRE", 
+    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVBANDPRE",
                     "CVBandPrecInit", MSGBP_SUNLS_FAIL);
     return(CVLS_SUNLS_FAIL);
   }
-  
+
   /* make sure P_data is free from any previous allocations */
   if (cvls_mem->pfree)
     cvls_mem->pfree(cv_mem);
@@ -198,7 +198,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
 }
 
 
-int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP, 
+int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP,
                             long int *leniwBP)
 {
   CVodeMem cv_mem;
@@ -207,7 +207,7 @@ int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP,
   sunindextype lrw1, liw1;
   long int lrw, liw;
   int flag;
-  
+
   if (cvode_mem == NULL) {
     cvProcessError(NULL, CVLS_MEM_NULL, "CVBANDPRE",
                    "CVBandPrecGetWorkSpace", MSGBP_MEM_NULL);
@@ -226,7 +226,7 @@ int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP,
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVBANDPRE",
                    "CVBandPrecGetWorkSpace", MSGBP_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
   /* sum space requirements for all objects in pdata */
@@ -284,7 +284,7 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVBANDPRE",
                    "CVBandPrecGetNumRhsEvals", MSGBP_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
   *nfevalsBP = pdata->nfeBP;
@@ -300,16 +300,16 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
   difference quotient Jacobian to create a preconditioner.
   CVBandPrecSetup calculates a new J, if necessary, then
   calculates P = I - gamma*J, and does an LU factorization of P.
- 
+
   The parameters of CVBandPrecSetup are as follows:
- 
+
   t       is the current value of the independent variable.
- 
+
   y       is the current value of the dependent variable vector,
           namely the predicted value of y(t).
- 
+
   fy      is the vector f(t,y).
- 
+
   jok     is an input flag indicating whether Jacobian-related
           data needs to be recomputed, as follows:
             jok == SUNFALSE means recompute Jacobian-related data
@@ -319,29 +319,28 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
                    (with the current value of gamma).
           A CVBandPrecSetup call with jok == SUNTRUE should only
           occur after a call with jok == SUNFALSE.
- 
+
   *jcurPtr is a pointer to an output integer flag which is
            set by CVBandPrecSetup as follows:
              *jcurPtr = SUNTRUE if Jacobian data was recomputed.
              *jcurPtr = SUNFALSE if Jacobian data was not recomputed,
                         but saved data was reused.
- 
+
   gamma   is the scalar appearing in the Newton matrix.
- 
+
   bp_data is a pointer to preconditoner data (set by CVBandPrecInit)
- 
+
   The value to be returned by the CVBandPrecSetup function is
     0  if successful, or
     1  if the band factorization failed.
   -----------------------------------------------------------------*/
-static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                           booleantype jok, booleantype *jcurPtr, 
+static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                           booleantype jok, booleantype *jcurPtr,
                            realtype gamma, void *bp_data)
 {
   CVBandPrecData pdata;
   CVodeMem cv_mem;
   int retval;
-  sunindextype ier;
 
   /* Assume matrix and lpivots have already been allocated. */
   pdata = (CVBandPrecData) bp_data;
@@ -353,7 +352,7 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNFALSE;
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "CVBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -367,7 +366,7 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNTRUE;
     retval = SUNMatZero(pdata->savedJ);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "CVBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -375,10 +374,10 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
       return(1);
     }
 
-    retval = CVBandPDQJac(pdata, t, y, fy, 
+    retval = CVBandPDQJac(pdata, t, y, fy,
                            pdata->tmp1, pdata->tmp2);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "CVBandPrecSetup", MSGBP_RHSFUNC_FAILED);
       return(-1);
     }
@@ -388,7 +387,7 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
 
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "CVBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -397,18 +396,18 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     }
 
   }
-  
+
   /* Scale and add identity to get savedP = I - gamma*J. */
   retval = SUNMatScaleAddI(-gamma, pdata->savedP);
   if (retval) {
-    cvProcessError(cv_mem, -1, "CVBANDPRE", 
+    cvProcessError(cv_mem, -1, "CVBANDPRE",
                    "CVBandPrecSetup", MSGBP_SUNMAT_FAIL);
     return(-1);
   }
 
   /* Do LU factorization of matrix and return error flag */
-  ier = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
-  return(ier);
+  retval = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
+  return(retval);
 }
 
 
@@ -417,20 +416,20 @@ static int CVBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
   -----------------------------------------------------------------
   CVBandPrecSolve solves a linear system P z = r, where P is the
   matrix computed by CVBandPrecond.
- 
+
   The parameters of CVBandPrecSolve used here are as follows:
- 
+
   r is the right-hand side vector of the linear system.
- 
+
   bp_data is a pointer to preconditoner data (set by CVBandPrecInit)
- 
+
   z is the output vector computed by CVBandPrecSolve.
- 
+
   The value returned by the CVBandPrecSolve function is always 0,
   indicating success.
-  -----------------------------------------------------------------*/ 
-static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                           N_Vector r, N_Vector z, realtype gamma, 
+  -----------------------------------------------------------------*/
+static int CVBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                           N_Vector r, N_Vector z, realtype gamma,
                            realtype delta, int lr, void *bp_data)
 {
   CVBandPrecData pdata;
@@ -452,7 +451,7 @@ static int CVBandPrecFree(CVodeMem cv_mem)
 
   if (cv_mem->cv_lmem == NULL) return(0);
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
-  
+
   if (cvls_mem->P_data == NULL) return(0);
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
@@ -472,15 +471,15 @@ static int CVBandPrecFree(CVodeMem cv_mem)
 /*-----------------------------------------------------------------
   CVBandPDQJac
   -----------------------------------------------------------------
-  This routine generates a banded difference quotient approximation 
-  to the Jacobian of f(t,y). It assumes that a band SUNMatrix is 
-  stored column-wise, and that elements within each column are 
+  This routine generates a banded difference quotient approximation
+  to the Jacobian of f(t,y). It assumes that a band SUNMatrix is
+  stored column-wise, and that elements within each column are
   contiguous. This makes it possible to get the address of a column
-  of J via the accessor function SUNBandMatrix_Column() and to 
-  write a simple for loop to set each of the elements of a column 
+  of J via the accessor function SUNBandMatrix_Column() and to
+  write a simple for loop to set each of the elements of a column
   in succession.
   -----------------------------------------------------------------*/
-static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y, 
+static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
                         N_Vector fy, N_Vector ftemp, N_Vector ytemp)
 {
   CVodeMem cv_mem;
@@ -490,6 +489,9 @@ static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   realtype *y_data, *ytemp_data, *cns_data;
   int retval;
 
+  /* initialize cns_data to avoid compiler warning */
+  cns_data = NULL;
+
   cv_mem = (CVodeMem) pdata->cvode_mem;
 
   /* Obtain pointers to the data for various vectors */
@@ -498,8 +500,8 @@ static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   ftemp_data = N_VGetArrayPointer(ftemp);
   y_data     = N_VGetArrayPointer(y);
   ytemp_data = N_VGetArrayPointer(ytemp);
-  if (cv_mem->cv_constraints != NULL)
-    cns_data  = N_VGetArrayPointer(cv_mem->cv_constraints);
+  if (cv_mem->cv_constraintsSet)
+    cns_data = N_VGetArrayPointer(cv_mem->cv_constraints);
 
   /* Load ytemp with y = predicted y vector. */
   N_VScale(ONE, y, ytemp);
@@ -513,16 +515,16 @@ static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   /* Set bandwidth and number of column groups for band differencing. */
   width = pdata->ml + pdata->mu + 1;
   ngroups = SUNMIN(width, pdata->N);
-  
+
   for (group = 1; group <= ngroups; group++) {
-    
+
     /* Increment all y_j in group. */
     for(j = group-1; j < pdata->N; j += width) {
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
       yj = y_data[j];
 
       /* Adjust sign(inc) again if yj has an inequality constraint. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -544,7 +546,7 @@ static int CVBandPDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
 
       /* Adjust sign(inc) as before. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
