@@ -2,26 +2,22 @@
  * Programmer(s): Carol Woodward @ LLNL
  *                Daniel R. Reynolds @ SMU
  *-----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  *-----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include "fida.h"
 #include "ida_impl.h"
-#include <ida/ida_direct.h>
+#include <ida/ida_ls.h>
 #include <sunmatrix/sunmatrix_sparse.h>
 
 /*=============================================================*/
@@ -56,7 +52,7 @@ void FIDA_SPARSESETJAC(int *ier)
                   "Sparse Fortran users must configure SUNDIALS with 64-bit integers.");
   *ier = 1;
 #else  
-  *ier = IDADlsSetJacFn(IDA_idamem, FIDASparseJac);
+  *ier = IDASetJacFn(IDA_idamem, FIDASparseJac);
 #endif
 }
 
