@@ -1,25 +1,21 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *        Alan C. Hindmarsh and Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This file contains implementations of routines for a
  * band-block-diagonal preconditioner, i.e. a block-diagonal
- * matrix with banded blocks, for use with IDA, the IDASPILS 
+ * matrix with banded blocks, for use with IDA, the IDASPILS
  * linear solver interface.
  *
  * NOTE: With only one processor in use, a banded matrix results
@@ -32,7 +28,7 @@
 #include <stdlib.h>
 
 #include "ida_impl.h"
-#include "ida_spils_impl.h"
+#include "ida_ls_impl.h"
 #include "ida_bbdpre_impl.h"
 #include <sundials/sundials_math.h>
 #include <nvector/nvector_serial.h>
@@ -54,54 +50,54 @@ static int IDABBDPrecFree(IDAMem ida_mem);
 
 /* Prototype for difference quotient Jacobian calculation routine */
 static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
-                     N_Vector yy, N_Vector yp, N_Vector gref, 
+                     N_Vector yy, N_Vector yp, N_Vector gref,
                      N_Vector ytemp, N_Vector yptemp, N_Vector gtemp);
 
 /*---------------------------------------------------------------
   User-Callable Functions: initialization, reinit and free
   ---------------------------------------------------------------*/
-int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal, 
-                   sunindextype mudq, sunindextype mldq, 
-                   sunindextype mukeep, sunindextype mlkeep, 
-                   realtype dq_rel_yy, 
+int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
+                   sunindextype mudq, sunindextype mldq,
+                   sunindextype mukeep, sunindextype mlkeep,
+                   realtype dq_rel_yy,
                    IDABBDLocalFn Gres, IDABBDCommFn Gcomm)
 {
   IDAMem IDA_mem;
-  IDASpilsMem idaspils_mem;
+  IDALsMem idals_mem;
   IBBDPrecData pdata;
   sunindextype muk, mlk, storage_mu, lrw1, liw1;
   long int lrw, liw;
   int flag;
 
   if (ida_mem == NULL) {
-    IDAProcessError(NULL, IDASPILS_MEM_NULL, "IDABBDPRE",
+    IDAProcessError(NULL, IDALS_MEM_NULL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_NULL);
-    return(IDASPILS_MEM_NULL);
+    return(IDALS_MEM_NULL);
   }
   IDA_mem = (IDAMem) ida_mem;
 
-  /* Test if the SPILS linear solver interface has been created */
+  /* Test if the LS linear solver interface has been created */
   if (IDA_mem->ida_lmem == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_LMEM_NULL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_LMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_LMEM_NULL);
-    return(IDASPILS_LMEM_NULL);
+    return(IDALS_LMEM_NULL);
   }
-  idaspils_mem = (IDASpilsMem) IDA_mem->ida_lmem;
+  idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
   /* Test compatibility of NVECTOR package with the BBD preconditioner */
   if(IDA_mem->ida_tempv1->ops->nvgetarraypointer == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_ILL_INPUT, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_BAD_NVECTOR);
-    return(IDASPILS_ILL_INPUT);
+    return(IDALS_ILL_INPUT);
   }
 
   /* Allocate data memory. */
   pdata = NULL;
   pdata = (IBBDPrecData) malloc(sizeof *pdata);
   if (pdata == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
 
   /* Set pointers to glocal and gcomm; load half-bandwidths. */
@@ -120,12 +116,12 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
 
   /* Allocate memory for preconditioner matrix. */
   pdata->PP = NULL;
-  pdata->PP = SUNBandMatrix(Nlocal, muk, mlk, storage_mu);
-  if (pdata->PP == NULL) { 
+  pdata->PP = SUNBandMatrixStorage(Nlocal, muk, mlk, storage_mu);
+  if (pdata->PP == NULL) {
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL); 
+    return(IDALS_MEM_FAIL);
   }
 
   /* Allocate memory for temporary N_Vectors */
@@ -134,9 +130,9 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
   if (pdata->zlocal == NULL) {
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE", 
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
   pdata->rlocal = NULL;
   pdata->rlocal = N_VNewEmpty_Serial(Nlocal);
@@ -144,35 +140,35 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
     N_VDestroy(pdata->zlocal);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE", 
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
   pdata->tempv1 = NULL;
-  pdata->tempv1 = N_VClone(IDA_mem->ida_tempv1); 
+  pdata->tempv1 = N_VClone(IDA_mem->ida_tempv1);
   if (pdata->tempv1 == NULL){
     N_VDestroy(pdata->rlocal);
     N_VDestroy(pdata->zlocal);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
   pdata->tempv2 = NULL;
-  pdata->tempv2 = N_VClone(IDA_mem->ida_tempv1); 
+  pdata->tempv2 = N_VClone(IDA_mem->ida_tempv1);
   if (pdata->tempv2 == NULL){
     N_VDestroy(pdata->rlocal);
     N_VDestroy(pdata->zlocal);
     N_VDestroy(pdata->tempv1);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
   pdata->tempv3 = NULL;
-  pdata->tempv3 = N_VClone(IDA_mem->ida_tempv1); 
+  pdata->tempv3 = N_VClone(IDA_mem->ida_tempv1);
   if (pdata->tempv3 == NULL){
     N_VDestroy(pdata->rlocal);
     N_VDestroy(pdata->zlocal);
@@ -180,12 +176,12 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
     N_VDestroy(pdata->tempv2);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
   pdata->tempv4 = NULL;
-  pdata->tempv4 = N_VClone(IDA_mem->ida_tempv1); 
+  pdata->tempv4 = N_VClone(IDA_mem->ida_tempv1);
   if (pdata->tempv4 == NULL){
     N_VDestroy(pdata->rlocal);
     N_VDestroy(pdata->zlocal);
@@ -194,14 +190,14 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
     N_VDestroy(pdata->tempv3);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
 
   /* Allocate memory for banded linear solver */
   pdata->LS = NULL;
-  pdata->LS = SUNBandLinearSolver(pdata->rlocal, pdata->PP);
+  pdata->LS = SUNLinSol_Band(pdata->rlocal, pdata->PP);
   if (pdata->LS == NULL) {
     N_VDestroy(pdata->zlocal);
     N_VDestroy(pdata->rlocal);
@@ -211,9 +207,9 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
     N_VDestroy(pdata->tempv4);
     SUNMatDestroy(pdata->PP);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_MEM_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_MEM_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_MEM_FAIL);
-    return(IDASPILS_MEM_FAIL);
+    return(IDALS_MEM_FAIL);
   }
 
   /* initialize band linear solver object */
@@ -228,18 +224,18 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->PP);
     SUNLinSolFree(pdata->LS);
     free(pdata); pdata = NULL;
-    IDAProcessError(IDA_mem, IDASPILS_SUNLS_FAIL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_SUNLS_FAIL, "IDABBDPRE",
                     "IDABBDPrecInit", MSGBBD_SUNLS_FAIL);
-    return(IDASPILS_SUNLS_FAIL);
+    return(IDALS_SUNLS_FAIL);
   }
- 
+
   /* Set rel_yy based on input value dq_rel_yy (0 implies default). */
   pdata->rel_yy = (dq_rel_yy > ZERO) ?
-    dq_rel_yy : SUNRsqrt(IDA_mem->ida_uround); 
+    dq_rel_yy : SUNRsqrt(IDA_mem->ida_uround);
 
   /* Store Nlocal to be used in IDABBDPrecSetup */
   pdata->n_local = Nlocal;
-  
+
   /* Set work space sizes and initialize nge. */
   pdata->rpwsize = 0;
   pdata->ipwsize = 0;
@@ -266,19 +262,19 @@ int IDABBDPrecInit(void *ida_mem, sunindextype Nlocal,
   pdata->nge = 0;
 
   /* make sure pdata is free from any previous allocations */
-  if (idaspils_mem->pfree) 
-    idaspils_mem->pfree(IDA_mem);
+  if (idals_mem->pfree)
+    idals_mem->pfree(IDA_mem);
 
-  /* Point to the new pdata field in the SPILS memory */
-  idaspils_mem->pdata = pdata;
+  /* Point to the new pdata field in the LS memory */
+  idals_mem->pdata = pdata;
 
   /* Attach the pfree function */
-  idaspils_mem->pfree = IDABBDPrecFree;
+  idals_mem->pfree = IDABBDPrecFree;
 
   /* Attach preconditioner solve and setup functions */
-  flag = IDASpilsSetPreconditioner(ida_mem,
-                                   IDABBDPrecSetup,
-                                   IDABBDPrecSolve);
+  flag = IDASetPreconditioner(ida_mem,
+                              IDABBDPrecSetup,
+                              IDABBDPrecSolve);
 
   return(flag);
 }
@@ -289,32 +285,32 @@ int IDABBDPrecReInit(void *ida_mem, sunindextype mudq,
                      sunindextype mldq, realtype dq_rel_yy)
 {
   IDAMem IDA_mem;
-  IDASpilsMem idaspils_mem;
+  IDALsMem idals_mem;
   IBBDPrecData pdata;
   sunindextype Nlocal;
 
   if (ida_mem == NULL) {
-    IDAProcessError(NULL, IDASPILS_MEM_NULL, "IDABBDPRE",
+    IDAProcessError(NULL, IDALS_MEM_NULL, "IDABBDPRE",
                     "IDABBDPrecReInit", MSGBBD_MEM_NULL);
-    return(IDASPILS_MEM_NULL);
+    return(IDALS_MEM_NULL);
   }
   IDA_mem = (IDAMem) ida_mem;
 
-  /* Test if the SPILS linear solver interface has been created */
+  /* Test if the LS linear solver interface has been created */
   if (IDA_mem->ida_lmem == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_LMEM_NULL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_LMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecReInit", MSGBBD_LMEM_NULL);
-    return(IDASPILS_LMEM_NULL);
+    return(IDALS_LMEM_NULL);
   }
-  idaspils_mem = (IDASpilsMem) IDA_mem->ida_lmem;
+  idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
   /* Test if the preconditioner data is non-NULL */
-  if (idaspils_mem->pdata == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_PMEM_NULL, "IDABBDPRE",
+  if (idals_mem->pdata == NULL) {
+    IDAProcessError(IDA_mem, IDALS_PMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecReInit", MSGBBD_PMEM_NULL);
-    return(IDASPILS_PMEM_NULL);
-  } 
-  pdata = (IBBDPrecData) idaspils_mem->pdata;
+    return(IDALS_PMEM_NULL);
+  }
+  pdata = (IBBDPrecData) idals_mem->pdata;
 
   /* Load half-bandwidths. */
   Nlocal = pdata->n_local;
@@ -323,12 +319,12 @@ int IDABBDPrecReInit(void *ida_mem, sunindextype mudq,
 
   /* Set rel_yy based on input value dq_rel_yy (0 implies default). */
   pdata->rel_yy = (dq_rel_yy > ZERO) ?
-    dq_rel_yy : SUNRsqrt(IDA_mem->ida_uround); 
+    dq_rel_yy : SUNRsqrt(IDA_mem->ida_uround);
 
   /* Re-initialize nge */
   pdata->nge = 0;
 
-  return(IDASPILS_SUCCESS);
+  return(IDALS_SUCCESS);
 }
 
 
@@ -338,34 +334,34 @@ int IDABBDPrecGetWorkSpace(void *ida_mem,
                            long int *leniwBBDP)
 {
   IDAMem IDA_mem;
-  IDASpilsMem idaspils_mem;
+  IDALsMem idals_mem;
   IBBDPrecData pdata;
 
   if (ida_mem == NULL) {
-    IDAProcessError(NULL, IDASPILS_MEM_NULL, "IDABBDPRE",
+    IDAProcessError(NULL, IDALS_MEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetWorkSpace", MSGBBD_MEM_NULL);
-    return(IDASPILS_MEM_NULL);
+    return(IDALS_MEM_NULL);
   }
   IDA_mem = (IDAMem) ida_mem;
 
   if (IDA_mem->ida_lmem == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_LMEM_NULL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_LMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetWorkSpace", MSGBBD_LMEM_NULL);
-    return(IDASPILS_LMEM_NULL);
+    return(IDALS_LMEM_NULL);
   }
-  idaspils_mem = (IDASpilsMem) IDA_mem->ida_lmem;
+  idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
-  if (idaspils_mem->pdata == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_PMEM_NULL, "IDABBDPRE",
+  if (idals_mem->pdata == NULL) {
+    IDAProcessError(IDA_mem, IDALS_PMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetWorkSpace", MSGBBD_PMEM_NULL);
-    return(IDASPILS_PMEM_NULL);
-  } 
-  pdata = (IBBDPrecData) idaspils_mem->pdata;
+    return(IDALS_PMEM_NULL);
+  }
+  pdata = (IBBDPrecData) idals_mem->pdata;
 
   *lenrwBBDP = pdata->rpwsize;
   *leniwBBDP = pdata->ipwsize;
 
-  return(IDASPILS_SUCCESS);
+  return(IDALS_SUCCESS);
 }
 
 
@@ -374,33 +370,33 @@ int IDABBDPrecGetNumGfnEvals(void *ida_mem,
                              long int *ngevalsBBDP)
 {
   IDAMem IDA_mem;
-  IDASpilsMem idaspils_mem;
+  IDALsMem idals_mem;
   IBBDPrecData pdata;
 
   if (ida_mem == NULL) {
-    IDAProcessError(NULL, IDASPILS_MEM_NULL, "IDABBDPRE",
+    IDAProcessError(NULL, IDALS_MEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetNumGfnEvals", MSGBBD_MEM_NULL);
-    return(IDASPILS_MEM_NULL);
+    return(IDALS_MEM_NULL);
   }
   IDA_mem = (IDAMem) ida_mem;
 
   if (IDA_mem->ida_lmem == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_LMEM_NULL, "IDABBDPRE",
+    IDAProcessError(IDA_mem, IDALS_LMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetNumGfnEvals", MSGBBD_LMEM_NULL);
-    return(IDASPILS_LMEM_NULL);
+    return(IDALS_LMEM_NULL);
   }
-  idaspils_mem = (IDASpilsMem) IDA_mem->ida_lmem;
+  idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
-  if (idaspils_mem->pdata == NULL) {
-    IDAProcessError(IDA_mem, IDASPILS_PMEM_NULL, "IDABBDPRE",
+  if (idals_mem->pdata == NULL) {
+    IDAProcessError(IDA_mem, IDALS_PMEM_NULL, "IDABBDPRE",
                     "IDABBDPrecGetNumGfnEvals", MSGBBD_PMEM_NULL);
-    return(IDASPILS_PMEM_NULL);
-  } 
-  pdata = (IBBDPrecData) idaspils_mem->pdata;
+    return(IDALS_PMEM_NULL);
+  }
+  pdata = (IBBDPrecData) idals_mem->pdata;
 
   *ngevalsBBDP = pdata->nge;
 
-  return(IDASPILS_SUCCESS);
+  return(IDALS_SUCCESS);
 }
 
 
@@ -413,23 +409,23 @@ int IDABBDPrecGetNumGfnEvals(void *ida_mem,
   scheme via calls to the user-supplied routines glocal, gcomm.
   After generating the block in the band matrix PP, this routine
   does an LU factorization in place in PP.
- 
+
   The IDABBDPrecSetup parameters used here are as follows:
- 
+
   tt is the current value of the independent variable t.
- 
+
   yy is the current value of the dependent variable vector,
      namely the predicted value of y(t).
- 
+
   yp is the current value of the derivative vector y',
      namely the predicted value of y'(t).
- 
+
   c_j is the scalar in the system Jacobian, proportional to 1/hh.
- 
+
   bbd_data is the pointer to BBD memory set by IDABBDInit
- 
+
   The argument rr is not used.
- 
+
   Return value:
   The value returned by this IDABBDPrecSetup function is a int
   flag indicating whether it was successful. This value is
@@ -440,7 +436,6 @@ int IDABBDPrecGetNumGfnEvals(void *ida_mem,
 static int IDABBDPrecSetup(realtype tt, N_Vector yy, N_Vector yp,
                            N_Vector rr, realtype c_j, void *bbd_data)
 {
-  sunindextype ier;
   IBBDPrecData pdata;
   IDAMem IDA_mem;
   int retval;
@@ -460,11 +455,11 @@ static int IDABBDPrecSetup(realtype tt, N_Vector yy, N_Vector yp,
   }
   if (retval > 0) {
     return(1);
-  } 
- 
+  }
+
   /* Do LU factorization of matrix and return error flag */
-  ier = SUNLinSolSetup_Band(pdata->LS, pdata->PP);
-  return(ier);
+  retval = SUNLinSolSetup_Band(pdata->LS, pdata->PP);
+  return(retval);
 }
 
 
@@ -474,18 +469,18 @@ static int IDABBDPrecSetup(realtype tt, N_Vector yy, N_Vector yp,
   The function IDABBDPrecSolve computes a solution to the linear
   system P z = r, where P is the left preconditioner defined by
   the routine IDABBDPrecSetup.
- 
+
   The IDABBDPrecSolve parameters used here are as follows:
- 
+
   rvec is the input right-hand side vector r.
- 
+
   zvec is the computed solution vector z.
- 
+
   bbd_data is the pointer to BBD data set by IDABBDInit.
- 
+
   The arguments tt, yy, yp, rr, c_j and delta are NOT used.
- 
-  IDABBDPrecSolve returns the value returned from the linear 
+
+  IDABBDPrecSolve returns the value returned from the linear
   solver object.
   ---------------------------------------------------------------*/
 static int IDABBDPrecSolve(realtype tt, N_Vector yy, N_Vector yp,
@@ -494,15 +489,15 @@ static int IDABBDPrecSolve(realtype tt, N_Vector yy, N_Vector yp,
 {
   IBBDPrecData pdata;
   int retval;
-  
+
   pdata = (IBBDPrecData) bbd_data;
 
   /* Attach local data arrays for rvec and zvec to rlocal and zlocal */
   N_VSetArrayPointer(N_VGetArrayPointer(rvec), pdata->rlocal);
   N_VSetArrayPointer(N_VGetArrayPointer(zvec), pdata->zlocal);
-  
+
   /* Call banded solver object to do the work */
-  retval = SUNLinSolSolve(pdata->LS, pdata->PP, pdata->zlocal, 
+  retval = SUNLinSolSolve(pdata->LS, pdata->PP, pdata->zlocal,
                           pdata->rlocal, ZERO);
 
   /* Detach local data arrays from rlocal and zlocal */
@@ -516,14 +511,14 @@ static int IDABBDPrecSolve(realtype tt, N_Vector yy, N_Vector yp,
 /*-------------------------------------------------------------*/
 static int IDABBDPrecFree(IDAMem IDA_mem)
 {
-  IDASpilsMem idaspils_mem;
+  IDALsMem idals_mem;
   IBBDPrecData pdata;
-  
+
   if (IDA_mem->ida_lmem == NULL) return(0);
-  idaspils_mem = (IDASpilsMem) IDA_mem->ida_lmem;
-  
-  if (idaspils_mem->pdata == NULL) return(0);
-  pdata = (IBBDPrecData) idaspils_mem->pdata;
+  idals_mem = (IDALsMem) IDA_mem->ida_lmem;
+
+  if (idals_mem->pdata == NULL) return(0);
+  pdata = (IBBDPrecData) idals_mem->pdata;
 
   SUNLinSolFree(pdata->LS);
   N_VDestroy(pdata->rlocal);
@@ -548,19 +543,19 @@ static int IDABBDPrecFree(IDAMem IDA_mem)
   to the local block of the Jacobian of G(t,y,y'). It assumes that
   a band matrix of type SUNMatrix is stored column-wise, and that
   elements within each column are contiguous.
- 
+
   All matrix elements are generated as difference quotients, by way
   of calls to the user routine glocal. By virtue of the band
   structure, the number of these calls is bandwidth + 1, where
   bandwidth = mldq + mudq + 1. But the band matrix kept has
   bandwidth = mlkeep + mukeep + 1. This routine also assumes that
   the local elements of a vector are stored contiguously.
- 
+
   Return values are: 0 (success), > 0 (recoverable error),
   or < 0 (nonrecoverable error).
   ----------------------------------------------------------------*/
 static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
-                     N_Vector yy, N_Vector yp, N_Vector gref, 
+                     N_Vector yy, N_Vector yp, N_Vector gref,
                      N_Vector ytemp, N_Vector yptemp, N_Vector gtemp)
 {
   IDAMem IDA_mem;
@@ -582,7 +577,7 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
   ypdata    = N_VGetArrayPointer(yp);
   gtempdata = N_VGetArrayPointer(gtemp);
   ewtdata   = N_VGetArrayPointer(IDA_mem->ida_ewt);
-  if (IDA_mem->ida_constraints != NULL) 
+  if (IDA_mem->ida_constraintsSet)
     cnsdata = N_VGetArrayPointer(IDA_mem->ida_constraints);
   ytempdata = N_VGetArrayPointer(ytemp);
   yptempdata= N_VGetArrayPointer(yptemp);
@@ -594,7 +589,7 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
     if (retval != 0) return(retval);
   }
 
-  retval = pdata->glocal(pdata->n_local, tt, yy, yp, gref, IDA_mem->ida_user_data); 
+  retval = pdata->glocal(pdata->n_local, tt, yy, yp, gref, IDA_mem->ida_user_data);
   pdata->nge++;
   if (retval != 0) return(retval);
 
@@ -604,13 +599,13 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
 
   /* Loop over groups. */
   for(group = 1; group <= ngroups; group++) {
-    
+
     /* Loop over the components in this group. */
     for(j = group-1; j < pdata->n_local; j += width) {
       yj = ydata[j];
       ypj = ypdata[j];
       ewtj = ewtdata[j];
-      
+
       /* Set increment inc to yj based on rel_yy*abs(yj), with
          adjustments using ypj and ewtj if this is small, and a further
          adjustment to give it the same sign as hh*ypj. */
@@ -618,9 +613,9 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
         SUNMAX(SUNRabs(yj), SUNMAX( SUNRabs(IDA_mem->ida_hh*ypj), ONE/ewtj));
       if (IDA_mem->ida_hh*ypj < ZERO)  inc = -inc;
       inc = (yj + inc) - yj;
-      
+
       /* Adjust sign(inc) again if yj has an inequality constraint. */
-      if (IDA_mem->ida_constraints != NULL) {
+      if (IDA_mem->ida_constraintsSet) {
         conj = cnsdata[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj <  ZERO) inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -629,12 +624,12 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
       /* Increment yj and ypj. */
       ytempdata[j] += inc;
       yptempdata[j] += cj*inc;
-      
+
     }
 
     /* Evaluate G with incremented y and yp arguments. */
     retval = pdata->glocal(pdata->n_local, tt, ytemp, yptemp,
-                           gtemp, IDA_mem->ida_user_data); 
+                           gtemp, IDA_mem->ida_user_data);
     pdata->nge++;
     if (retval != 0) return(retval);
 
@@ -649,7 +644,7 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
         SUNMAX(SUNRabs(yj), SUNMAX( SUNRabs(IDA_mem->ida_hh*ypj), ONE/ewtj));
       if (IDA_mem->ida_hh*ypj < ZERO)  inc = -inc;
       inc = (yj + inc) - yj;
-      if (IDA_mem->ida_constraints != NULL) {
+      if (IDA_mem->ida_constraintsSet) {
         conj = cnsdata[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj <  ZERO) inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -665,7 +660,6 @@ static int IBBDDQJac(IBBDPrecData pdata, realtype tt, realtype cj,
           inc_inv * (gtempdata[i] - grefdata[i]);
     }
   }
-  
+
   return(0);
 }
-
