@@ -1,13 +1,9 @@
-/*
- * -----------------------------------------------------------------
- * $Revision$
- * $Date$
- * ----------------------------------------------------------------- 
+/* -----------------------------------------------------------------
  * Programmer(s): Alan Hindmarsh, Radu Serban and
  *                Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -16,24 +12,23 @@
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the implementation file for the optional inputs and     
- * outputs for the IDA solver.                                    
- * -----------------------------------------------------------------
- */
+ * This is the implementation file for the optional inputs and
+ * outputs for the IDA solver.
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "ida_impl.h"
-
-#include <sundials/sundials_math.h>
+#include "sundials/sundials_types.h"
+#include "sundials/sundials_math.h"
 
 #define ZERO    RCONST(0.0)
 #define HALF    RCONST(0.5)
 #define ONE     RCONST(1.0)
 #define TWOPT5  RCONST(2.5)
 
-/* 
+/*
  * =================================================================
  * IDA optional input functions
  * =================================================================
@@ -117,7 +112,7 @@ int IDASetMaxOrd(void *ida_mem, int maxord)
   if (maxord > maxord_alloc) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxOrd", MSG_BAD_MAXORD);
     return(IDA_ILL_INPUT);
-  }  
+  }
 
   IDA_mem->ida_maxord = SUNMIN(maxord,MAXORD_DEFAULT);
 
@@ -346,7 +341,7 @@ int IDASetId(void *ida_mem, N_Vector id)
       IDA_mem->ida_lrw -= IDA_mem->ida_lrw1;
       IDA_mem->ida_liw -= IDA_mem->ida_liw1;
     }
-    IDA_mem->ida_idMallocDone = SUNFALSE;    
+    IDA_mem->ida_idMallocDone = SUNFALSE;
     return(IDA_SUCCESS);
   }
 
@@ -403,9 +398,9 @@ int IDASetConstraints(void *ida_mem, N_Vector constraints)
   /*  Check the constraints vector */
 
   temptest = N_VMaxNorm(constraints);
-  if((temptest > TWOPT5) || (temptest < HALF)){ 
+  if((temptest > TWOPT5) || (temptest < HALF)){
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetConstraints", MSG_BAD_CONSTR);
-    return(IDA_ILL_INPUT); 
+    return(IDA_ILL_INPUT);
   }
 
   if ( !(IDA_mem->ida_constraintsMallocDone) ) {
@@ -424,7 +419,7 @@ int IDASetConstraints(void *ida_mem, N_Vector constraints)
   return(IDA_SUCCESS);
 }
 
-/* 
+/*
  * IDASetRootDirection
  *
  * Specifies the direction of zero-crossings to be monitored.
@@ -446,7 +441,7 @@ int IDASetRootDirection(void *ida_mem, int *rootdir)
   nrt = IDA_mem->ida_nrtfn;
   if (nrt==0) {
     IDAProcessError(NULL, IDA_ILL_INPUT, "IDA", "IDASetRootDirection", MSG_NO_ROOT);
-    return(IDA_ILL_INPUT);    
+    return(IDA_ILL_INPUT);
   }
 
   for(i=0; i<nrt; i++) IDA_mem->ida_rootdir[i] = rootdir[i];
@@ -473,12 +468,12 @@ int IDASetNoInactiveRootWarn(void *ida_mem)
   IDA_mem = (IDAMem) ida_mem;
 
   IDA_mem->ida_mxgnull = 0;
-  
+
   return(IDA_SUCCESS);
 }
 
 
-/* 
+/*
  * =================================================================
  * IDA IC optional input functions
  * =================================================================
@@ -544,7 +539,7 @@ int IDASetMaxNumJacsIC(void *ida_mem, int maxnj)
    if (maxnj <= 0) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxNumJacsIC", MSG_BAD_MAXNJ);
     return(IDA_ILL_INPUT);
-  } 
+  }
 
   IDA_mem->ida_maxnj = maxnj;
 
@@ -638,7 +633,7 @@ int IDASetStepToleranceIC(void *ida_mem, realtype steptol)
   return(IDA_SUCCESS);
 }
 
-/* 
+/*
  * =================================================================
  * IDA optional input functions
  * =================================================================
@@ -737,13 +732,13 @@ int IDAGetNumBacktrackOps(void *ida_mem, long int *nbacktracks)
 int IDAGetConsistentIC(void *ida_mem, N_Vector yy0, N_Vector yp0)
 {
   IDAMem IDA_mem;
-  
+
   if (ida_mem == NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetConsistentIC", MSG_NO_MEM);
     return (IDA_MEM_NULL);
   }
 
-  IDA_mem = (IDAMem) ida_mem; 
+  IDA_mem = (IDAMem) ida_mem;
 
   if (IDA_mem->ida_kused != 0) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDAGetConsistentIC", MSG_TOO_LATE);
@@ -794,6 +789,60 @@ int IDAGetCurrentOrder(void *ida_mem, int *kcur)
 
 /*-----------------------------------------------------------------*/
 
+int IDAGetCurrentCj(void *ida_mem, realtype *cj)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentCjRatio", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *cj = IDA_mem->ida_cj;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetCurrentY(void *ida_mem, N_Vector *ycur)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentY", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *ycur = IDA_mem->ida_yy;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetCurrentYp(void *ida_mem, N_Vector *ypcur)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentYp", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *ypcur = IDA_mem->ida_yp;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDAGetActualInitStep(void *ida_mem, realtype *hinused)
 {
   IDAMem IDA_mem;
@@ -887,13 +936,13 @@ int IDAGetTolScaleFactor(void *ida_mem, realtype *tolsfact)
 int IDAGetErrWeights(void *ida_mem, N_Vector eweight)
 {
   IDAMem IDA_mem;
-  
+
   if (ida_mem == NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetErrWeights", MSG_NO_MEM);
     return (IDA_MEM_NULL);
   }
 
-  IDA_mem = (IDAMem) ida_mem; 
+  IDA_mem = (IDAMem) ida_mem;
 
   N_VScale(ONE, IDA_mem->ida_ewt, eweight);
 
@@ -938,9 +987,9 @@ int IDAGetWorkSpace(void *ida_mem, long int *lenrw, long int *leniw)
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals, 
+int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals,
                           long int *nlinsetups, long int *netfails,
-                          int *klast, int *kcur, realtype *hinused, realtype *hlast, 
+                          int *klast, int *kcur, realtype *hinused, realtype *hlast,
                           realtype *hcur, realtype *tcur)
 {
   IDAMem IDA_mem;
@@ -960,7 +1009,7 @@ int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals,
   *kcur       = IDA_mem->ida_kk;
   *hinused    = IDA_mem->ida_h0u;
   *hlast      = IDA_mem->ida_hused;
-  *hcur       = IDA_mem->ida_hh;  
+  *hcur       = IDA_mem->ida_hh;
   *tcur       = IDA_mem->ida_tn;
 
   return(IDA_SUCCESS);
@@ -1010,8 +1059,6 @@ int IDAGetRootInfo(void *ida_mem, int *rootsfound)
 int IDAGetNumNonlinSolvIters(void *ida_mem, long int *nniters)
 {
   IDAMem IDA_mem;
-  long int nls_iters;
-  int retval;
 
   if (ida_mem==NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
@@ -1021,23 +1068,8 @@ int IDAGetNumNonlinSolvIters(void *ida_mem, long int *nniters)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  /* get number of iterations for IC calc */
   *nniters = IDA_mem->ida_nni;
 
-  /* check that the NLS is non-NULL */
-  if (IDA_mem->NLS == NULL) {
-    IDAProcessError(NULL, IDA_MEM_FAIL, "IDA",
-                    "IDAGetNumNonlinSolvIters", MSG_MEM_FAIL);
-    return(IDA_MEM_FAIL);
-  }
-
-  /* get number of iterations from the NLS */
-  retval = SUNNonlinSolGetNumIters(IDA_mem->NLS, &nls_iters);
-  if (retval != IDA_SUCCESS) return(retval);
-
-  /* update the number of nonlinear iterations */  
-  *nniters += nls_iters;
-
   return(IDA_SUCCESS);
 }
 
@@ -1064,8 +1096,6 @@ int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
 int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nncfails)
 {
   IDAMem IDA_mem;
-  long int nls_iters;
-  int retval;
 
   if (ida_mem==NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
@@ -1078,20 +1108,6 @@ int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nncfails)
   *nniters  = IDA_mem->ida_nni;
   *nncfails = IDA_mem->ida_ncfn;
 
-  /* check that the NLS is non-NULL */
-  if (IDA_mem->NLS == NULL) {
-    IDAProcessError(NULL, IDA_MEM_FAIL, "IDA",
-                    "IDAGetNonlinSolvStats", MSG_MEM_FAIL);
-    return(IDA_MEM_FAIL);
-  }
-
-  /* get number of iterations from the NLS */
-  retval = SUNNonlinSolGetNumIters(IDA_mem->NLS, &nls_iters);
-  if (retval != IDA_SUCCESS) return(retval);
-  
-  /* update the number of nonlinear iterations */
-  *nniters += nls_iters;
-
   return(IDA_SUCCESS);
 }
 
@@ -1173,11 +1189,15 @@ char *IDAGetReturnFlagName(long int flag)
   case IDA_LINESEARCH_FAIL:
     sprintf(name,"IDA_LINESEARCH_FAIL");
     break;
-
+  case IDA_NLS_SETUP_FAIL:
+    sprintf(name,"IDA_NLS_SETUP_FAIL");
+    break;
+  case IDA_NLS_FAIL:
+    sprintf(name,"IDA_NLS_FAIL");
+    break;
   default:
     sprintf(name,"NONE");
   }
 
   return(name);
 }
-
