@@ -3,7 +3,7 @@
  *                Alan C. Hindmarsh and Radu Serban @ LLNL
  *-----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -22,6 +22,7 @@
 #include "ida_impl.h"
 #include "ida_ls_impl.h"
 #include <sundials/sundials_math.h>
+#include <sundials/sundials_linearsolver.h>
 #include <sunmatrix/sunmatrix_band.h>
 #include <sunmatrix/sunmatrix_dense.h>
 #include <sunmatrix/sunmatrix_sparse.h>
@@ -45,9 +46,11 @@
   ---------------------------------------------------------------*/
 int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
 {
-  IDAMem   IDA_mem;
-  IDALsMem idals_mem;
-  int      retval, LSType;
+  IDAMem      IDA_mem;
+  IDALsMem    idals_mem;
+  int         retval, LSType;
+  booleantype iterative;    /* is the solver iterative?    */
+  booleantype matrixbased;  /* is a matrix structure used? */
 
   /* Return immediately if any input is NULL */
   if (ida_mem == NULL) {
@@ -64,50 +67,72 @@ int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
   IDA_mem = (IDAMem) ida_mem;
 
   /* Test if solver is compatible with LS interface */
-  if ( (LS->ops->gettype == NULL) ||
-       (LS->ops->initialize == NULL) ||
-       (LS->ops->setup == NULL) ||
-       (LS->ops->solve == NULL) ) {
+  if ( (LS->ops->gettype == NULL) || (LS->ops->solve == NULL) ) {
     IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS",
                    "IDASetLinearSolver",
                    "LS object is missing a required operation");
     return(IDALS_ILL_INPUT);
   }
 
+  /* Retrieve the LS type */
+  LSType = SUNLinSolGetType(LS);
+
+  /* Set flags based on LS type */
+  iterative   = (LSType != SUNLINEARSOLVER_DIRECT);
+  matrixbased = ((LSType != SUNLINEARSOLVER_ITERATIVE) &&
+                 (LSType != SUNLINEARSOLVER_MATRIX_EMBEDDED));
+
   /* Test if vector is compatible with LS interface */
-  if ( (IDA_mem->ida_tempv1->ops->nvdotprod == NULL) ||
-       (IDA_mem->ida_tempv1->ops->nvconst == NULL) ) {
+  if (IDA_mem->ida_tempv1->ops->nvconst == NULL ||
+      IDA_mem->ida_tempv1->ops->nvwrmsnorm == NULL) {
     IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS",
                     "IDASetLinearSolver", MSG_LS_BAD_NVECTOR);
     return(IDALS_ILL_INPUT);
   }
 
-  /* Retrieve the LS type */
-  LSType = SUNLinSolGetType(LS);
+  /* Ensure that A is NULL when LS is matrix-embedded */
+  if ((LSType == SUNLINEARSOLVER_MATRIX_EMBEDDED) && (A != NULL)) {
+    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
+                    "Incompatible inputs: matrix-embedded LS requires NULL matrix");
+    return(IDALS_ILL_INPUT);
+  }
 
   /* Check for compatible LS type, matrix and "atimes" support */
-  if ( ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-         (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) &&
-       ( (LS->ops->resid == NULL) ||
-         (LS->ops->numiters == NULL) ) ) {
-    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
-                   "Iterative LS object requires 'resid' and 'numiters' routines");
-    return(IDALS_ILL_INPUT);
-  }
-  if ((LSType == SUNLINEARSOLVER_ITERATIVE) && (LS->ops->setatimes == NULL)) {
-    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
-                    "Incompatible inputs: iterative LS must support ATimes routine");
-    return(IDALS_ILL_INPUT);
-  }
-  if ((LSType == SUNLINEARSOLVER_DIRECT) && (A == NULL)) {
+  if (iterative) {
+
+    if (IDA_mem->ida_tempv1->ops->nvgetlength == NULL) {
+      IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS",
+                      "IDASetLinearSolver", MSG_LS_BAD_NVECTOR);
+      return(IDALS_ILL_INPUT);
+    }
+
+    if (LSType != SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+      if (LS->ops->resid == NULL || LS->ops->numiters == NULL) {
+        IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
+                        "Iterative LS object requires 'resid' and 'numiters' routines");
+        return(IDALS_ILL_INPUT);
+      }
+    }
+
+    if (!matrixbased && (LSType != SUNLINEARSOLVER_MATRIX_EMBEDDED) &&
+        (LS->ops->setatimes == NULL)) {
+      IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
+                      "Incompatible inputs: iterative LS must support ATimes routine");
+      return(IDALS_ILL_INPUT);
+    }
+
+    if (matrixbased && (A == NULL)) {
+      IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
+                      "Incompatible inputs: matrix-iterative LS requires non-NULL matrix");
+      return(IDALS_ILL_INPUT);
+    }
+
+  } else if (A == NULL) {
+
     IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
                     "Incompatible inputs: direct LS requires non-NULL matrix");
     return(IDALS_ILL_INPUT);
-  }
-  if ((LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) && (A == NULL)) {
-    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetLinearSolver",
-                    "Incompatible inputs: matrix-iterative LS requires non-NULL matrix");
-    return(IDALS_ILL_INPUT);
+
   }
 
   /* free any existing system solver attached to IDA */
@@ -120,9 +145,7 @@ int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
   IDA_mem->ida_lfree  = idaLsFree;
 
   /* Set ida_lperf if using an iterative SUNLinearSolver object */
-  IDA_mem->ida_lperf = ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-                         (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) ?
-    idaLsPerf : NULL;
+  IDA_mem->ida_lperf = (iterative) ? idaLsPerf : NULL;
 
   /* Allocate memory for IDALsMemRec */
   idals_mem = NULL;
@@ -137,6 +160,10 @@ int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
   /* set SUNLinearSolver pointer */
   idals_mem->LS = LS;
 
+  /* Linear solver type information */
+  idals_mem->iterative   = iterative;
+  idals_mem->matrixbased = matrixbased;
+
   /* Set defaults for Jacobian-related fields */
   idals_mem->J = A;
   if (A != NULL) {
@@ -151,6 +178,7 @@ int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
   idals_mem->jtimesDQ = SUNTRUE;
   idals_mem->jtsetup  = NULL;
   idals_mem->jtimes   = idaLsDQJtimes;
+  idals_mem->jt_res   = IDA_mem->ida_res;
   idals_mem->jt_data  = IDA_mem;
 
   /* Set defaults for preconditioner-related fields */
@@ -219,10 +247,15 @@ int IDASetLinearSolver(void *ida_mem, SUNLinearSolver LS, SUNMatrix A)
     return(IDALS_MEM_FAIL);
   }
 
-  /* Compute sqrtN from a dot product */
-  N_VConst(ONE, idals_mem->ytemp);
-  idals_mem->sqrtN = SUNRsqrt( N_VDotProd(idals_mem->ytemp,
-                                          idals_mem->ytemp) );
+  /* For iterative LS, compute sqrtN */
+  if (iterative)
+    idals_mem->nrmfac = SUNRsqrt( N_VGetLength(idals_mem->ytemp) );
+
+  /* For matrix-based LS, enable solution scaling */
+  if (matrixbased)
+    idals_mem->scalesol = SUNTRUE;
+  else
+    idals_mem->scalesol = SUNFALSE;
 
   /* Attach linear solver memory to integrator memory */
   IDA_mem->ida_lmem = idals_mem;
@@ -295,6 +328,58 @@ int IDASetEpsLin(void *ida_mem, realtype eplifac)
 }
 
 
+/* IDASetWRMSNormFactor sets or computes the factor to use when converting from
+   the integrator tolerance to the linear solver tolerance (WRMS to L2 norm). */
+int IDASetLSNormFactor(void *ida_mem, realtype nrmfac)
+{
+  IDAMem   IDA_mem;
+  IDALsMem idals_mem;
+  int      retval;
+
+  /* access IDALsMem structure */
+  retval = idaLs_AccessLMem(ida_mem, "IDASetLSNormFactor",
+                            &IDA_mem, &idals_mem);
+  if (retval != IDALS_SUCCESS) return(retval);
+
+  if (nrmfac > ZERO) {
+    /* user-provided factor */
+    idals_mem->nrmfac = nrmfac;
+  } else if (nrmfac < ZERO) {
+    /* compute factor for WRMS norm with dot product */
+    N_VConst(ONE, idals_mem->ytemp);
+    idals_mem->nrmfac = SUNRsqrt(N_VDotProd(idals_mem->ytemp, idals_mem->ytemp));
+  } else {
+    /* compute default factor for WRMS norm from vector legnth */
+    idals_mem->nrmfac = SUNRsqrt(N_VGetLength(idals_mem->ytemp));
+  }
+
+  return(IDALS_SUCCESS);
+}
+
+
+/* IDASetLinearSolutionScaling enables or disables scaling the linear solver
+   solution to account for changes in cj. */
+int IDASetLinearSolutionScaling(void *ida_mem, booleantype onoff)
+{
+  IDAMem   IDA_mem;
+  IDALsMem idals_mem;
+  int      retval;
+
+  /* access IDALsMem structure */
+  retval = idaLs_AccessLMem(ida_mem, "IDASetLinearSolutionScaling",
+                            &IDA_mem, &idals_mem);
+  if (retval != IDALS_SUCCESS) return(retval);
+
+  /* check for valid solver type */
+  if (!(idals_mem->matrixbased)) return(IDALS_ILL_INPUT);
+
+  /* set solution scaling flag */
+  idals_mem->scalesol = onoff;
+
+  return(IDALS_SUCCESS);
+}
+
+
 /* IDASetIncrementFactor specifies increment factor for DQ approximations to Jv */
 int IDASetIncrementFactor(void *ida_mem, realtype dqincfac)
 {
@@ -397,6 +482,7 @@ int IDASetJacTimes(void *ida_mem, IDALsJacTimesSetupFn jtsetup,
     idals_mem->jtimesDQ = SUNTRUE;
     idals_mem->jtsetup  = NULL;
     idals_mem->jtimes   = idaLsDQJtimes;
+    idals_mem->jt_res   = IDA_mem->ida_res;
     idals_mem->jt_data  = IDA_mem;
   }
 
@@ -404,6 +490,37 @@ int IDASetJacTimes(void *ida_mem, IDALsJacTimesSetupFn jtsetup,
 }
 
 
+/* IDASetJacTimesResFn specifies an alternative user-supplied DAE residual
+   function to use in the internal finite difference Jacobian-vector
+   product */
+int IDASetJacTimesResFn(void *ida_mem, IDAResFn jtimesResFn)
+{
+  IDAMem   IDA_mem;
+  IDALsMem idals_mem;
+  int      retval;
+
+  /* access IDALsMem structure */
+  retval = idaLs_AccessLMem(ida_mem, "IDASetJacTimesResFn",
+                            &IDA_mem, &idals_mem);
+  if (retval != IDALS_SUCCESS) return(retval);
+
+  /* check if using internal finite difference approximation */
+  if (!(idals_mem->jtimesDQ)) {
+    IDAProcessError(IDA_mem, IDALS_ILL_INPUT, "IDALS", "IDASetJacTimesResFn",
+                    "Internal finite-difference Jacobian-vector product is disabled.");
+    return(IDALS_ILL_INPUT);
+  }
+
+  /* store function pointers for Res function (NULL implies use DAE Res) */
+  if (jtimesResFn != NULL)
+    idals_mem->jt_res = jtimesResFn;
+  else
+    idals_mem->jt_res = IDA_mem->ida_res;
+
+  return(IDALS_SUCCESS);
+}
+
+
 /* IDAGetLinWorkSpace returns the length of workspace allocated
    for the IDALS linear solver interface */
 int IDAGetLinWorkSpace(void *ida_mem, long int *lenrwLS,
@@ -769,7 +886,6 @@ int idaLsDQJac(realtype t, realtype c_j, N_Vector y, N_Vector yp,
 
   /* Verify that N_Vector supports required operations */
   if (IDA_mem->ida_tempv1->ops->nvcloneempty == NULL ||
-      IDA_mem->ida_tempv1->ops->nvwrmsnorm == NULL ||
       IDA_mem->ida_tempv1->ops->nvlinearsum == NULL ||
       IDA_mem->ida_tempv1->ops->nvdestroy == NULL ||
       IDA_mem->ida_tempv1->ops->nvscale == NULL ||
@@ -823,7 +939,7 @@ int idaLsDenseDQJac(realtype tt, realtype c_j, N_Vector yy,
   idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
   /* access matrix dimension */
-  N = SUNDenseMatrix_Rows(Jac);
+  N = SUNDenseMatrix_Columns(Jac);
 
   /* Rename work vectors for readibility */
   rtemp = tmp1;
@@ -835,7 +951,7 @@ int idaLsDenseDQJac(realtype tt, realtype c_j, N_Vector yy,
   ewt_data = N_VGetArrayPointer(IDA_mem->ida_ewt);
   y_data   = N_VGetArrayPointer(yy);
   yp_data  = N_VGetArrayPointer(yp);
-  if(IDA_mem->ida_constraints!=NULL)
+  if(IDA_mem->ida_constraintsSet)
     cns_data = N_VGetArrayPointer(IDA_mem->ida_constraints);
 
   srur = SUNRsqrt(IDA_mem->ida_uround);
@@ -860,7 +976,7 @@ int idaLsDenseDQJac(realtype tt, realtype c_j, N_Vector yy,
     inc = (yj + inc) - yj;
 
     /* Adjust sign(inc) again if y_j has an inequality constraint. */
-    if (IDA_mem->ida_constraints != NULL) {
+    if (IDA_mem->ida_constraintsSet) {
       conj = cns_data[j];
       if (SUNRabs(conj) == ONE)      {if((yj+inc)*conj <  ZERO) inc = -inc;}
       else if (SUNRabs(conj) == TWO) {if((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -939,7 +1055,7 @@ int idaLsBandDQJac(realtype tt, realtype c_j, N_Vector yy,
   rtemp_data  = N_VGetArrayPointer(rtemp);
   ytemp_data  = N_VGetArrayPointer(ytemp);
   yptemp_data = N_VGetArrayPointer(yptemp);
-  if (IDA_mem->ida_constraints != NULL)
+  if (IDA_mem->ida_constraintsSet)
     cns_data = N_VGetArrayPointer(IDA_mem->ida_constraints);
 
   /* Initialize ytemp and yptemp. */
@@ -969,7 +1085,7 @@ int idaLsBandDQJac(realtype tt, realtype c_j, N_Vector yy,
         inc = (yj + inc) - yj;
 
         /* Adjust sign(inc) again if yj has an inequality constraint. */
-        if (IDA_mem->ida_constraints != NULL) {
+        if (IDA_mem->ida_constraintsSet) {
           conj = cns_data[j];
           if (SUNRabs(conj) == ONE)      {if((yj+inc)*conj <  ZERO) inc = -inc;}
           else if (SUNRabs(conj) == TWO) {if((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -999,7 +1115,7 @@ int idaLsBandDQJac(realtype tt, realtype c_j, N_Vector yy,
                     ONE/ewtj );
       if (IDA_mem->ida_hh*ypj < ZERO)  inc = -inc;
       inc = (yj + inc) - yj;
-      if (IDA_mem->ida_constraints != NULL) {
+      if (IDA_mem->ida_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if((yj+inc)*conj <  ZERO) inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -1040,14 +1156,18 @@ int idaLsDQJtimes(realtype tt, N_Vector yy, N_Vector yp, N_Vector rr,
   N_Vector y_tmp, yp_tmp;
   realtype sig, siginv;
   int      iter, retval;
+  SUNLinearSolver_ID LSID;
 
   /* access IDALsMem structure */
   retval = idaLs_AccessLMem(ida_mem, "idaLsDQJtimes",
                             &IDA_mem, &idals_mem);
   if (retval != IDALS_SUCCESS)  return(retval);
 
-  sig = idals_mem->sqrtN * idals_mem->dqincfac;  /* GMRES */
-  /*sig = idals_mem->dqincfac / N_VWrmsNorm(v, IDA_mem->ida_ewt);*/  /* BiCGStab/TFQMR */
+  LSID = SUNLinSolGetID(idals_mem->LS);
+  if (LSID == SUNLINEARSOLVER_SPGMR || LSID == SUNLINEARSOLVER_SPFGMR)
+    sig = idals_mem->nrmfac * idals_mem->dqincfac;
+  else
+    sig = idals_mem->dqincfac / N_VWrmsNorm(v, IDA_mem->ida_ewt);
 
   /* Rename work1 and work2 for readibility */
   y_tmp  = work1;
@@ -1060,7 +1180,7 @@ int idaLsDQJtimes(realtype tt, N_Vector yy, N_Vector yp, N_Vector rr,
     N_VLinearSum(c_j*sig, v, ONE, yp, yp_tmp);
 
     /* Call res for Jv = F(t, y_tmp, yp_tmp), and return if it failed. */
-    retval = IDA_mem->ida_res(tt, y_tmp, yp_tmp, Jv, IDA_mem->ida_user_data);
+    retval = idals_mem->jt_res(tt, y_tmp, yp_tmp, Jv, IDA_mem->ida_user_data);
     idals_mem->nreDQ++;
     if (retval == 0) break;
     if (retval < 0)  return(-1);
@@ -1156,6 +1276,12 @@ int idaLsInitialize(IDAMem IDA_mem)
   if ( (idals_mem->J == NULL) && (idals_mem->pset == NULL) )
     IDA_mem->ida_lsetup = NULL;
 
+  /* When using a matrix-embedded linear solver disable lsetup call */
+  if (SUNLinSolGetType(idals_mem->LS) == SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+    IDA_mem->ida_lsetup = NULL;
+    idals_mem->scalesol = SUNFALSE;
+  }
+
   /* Call LS initialize routine */
   idals_mem->last_flag = SUNLinSolInitialize(idals_mem->LS);
   return(idals_mem->last_flag);
@@ -1183,6 +1309,12 @@ int idaLsSetup(IDAMem IDA_mem, N_Vector y, N_Vector yp, N_Vector r,
   }
   idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
+  /* Immediately return when using matrix-embedded linear solver */
+  if (SUNLinSolGetType(idals_mem->LS) == SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+    idals_mem->last_flag = IDALS_SUCCESS;
+    return(idals_mem->last_flag);
+  }
+
   /* Set IDALs N_Vector pointers to inputs */
   idals_mem->ycur  = y;
   idals_mem->ypcur = yp;
@@ -1194,13 +1326,15 @@ int idaLsSetup(IDAMem IDA_mem, N_Vector y, N_Vector yp, N_Vector r,
     /* Increment nje counter. */
     idals_mem->nje++;
 
-    /* Zero out J; call Jacobian routine jac; return if it failed. */
-    retval = SUNMatZero(idals_mem->J);
-    if (retval != 0) {
-      IDAProcessError(IDA_mem, IDALS_SUNMAT_FAIL, "IDALS",
-                      "idaLsSetup", MSG_LS_MATZERO_FAILED);
-      idals_mem->last_flag = IDALS_SUNMAT_FAIL;
-      return(idals_mem->last_flag);
+    /* Clear the linear system matrix if necessary */
+    if (SUNLinSolGetType(idals_mem->LS) == SUNLINEARSOLVER_DIRECT) {
+      retval = SUNMatZero(idals_mem->J);
+      if (retval != 0) {
+        IDAProcessError(IDA_mem, IDALS_SUNMAT_FAIL, "IDALS",
+                        "idaLsSetup", MSG_LS_MATZERO_FAILED);
+        idals_mem->last_flag = IDALS_SUNMAT_FAIL;
+        return(idals_mem->last_flag);
+      }
     }
 
     /* Call Jacobian routine */
@@ -1241,7 +1375,7 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
 {
   IDALsMem idals_mem;
   int      nli_inc, retval;
-  realtype tol, w_mean, LSType;
+  realtype tol, w_mean;
 
   /* access IDALsMem structure */
   if (IDA_mem->ida_lmem == NULL) {
@@ -1251,17 +1385,13 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
   }
   idals_mem = (IDALsMem) IDA_mem->ida_lmem;
 
-  /* Retrieve the LS type */
-  LSType = SUNLinSolGetType(idals_mem->LS);
-
   /* If the linear solver is iterative: set convergence test constant tol,
      in terms of the Newton convergence test constant epsNewt and safety
-     factors. The factor sqrt(Neq) assures that the convergence test is
+     factors. The factor nrmlfac assures that the convergence test is
      applied to the WRMS norm of the residual vector, rather than the
      weighted L2 norm. */
-  if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-       (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
-    tol = idals_mem->sqrtN * idals_mem->eplifac * IDA_mem->ida_epsNewt;
+  if (idals_mem->iterative) {
+    tol = idals_mem->nrmfac * idals_mem->eplifac * IDA_mem->ida_epsNewt;
   } else {
     tol = ZERO;
   }
@@ -1272,9 +1402,6 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
   idals_mem->ypcur = ypcur;
   idals_mem->rcur  = rescur;
 
-  /* Set initial guess x = 0 to LS */
-  N_VConst(ZERO, idals_mem->x);
-
   /* Set scaling vectors for LS to use (if applicable) */
   if (idals_mem->LS->ops->setscalingvectors) {
     retval = SUNLinSolSetScalingVectors(idals_mem->LS, weight, weight);
@@ -1298,16 +1425,22 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
        <=> w_mean^2 \sum_{i=0}^{n-1} (b - A x_i)^2 < tol^2
        <=> \sum_{i=0}^{n-1} (b - A x_i)^2 < tol^2 / w_mean^2
        <=> || b - A x ||_2 < tol / w_mean
-     So we compute w_mean = ||w||_RMS = ||w||_2 / sqrt(n), and scale
-     the desired tolerance accordingly. */
-  } else if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-              (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
+     So we compute w_mean = ||w||_RMS and scale the desired tolerance accordingly. */
+  } else if (idals_mem->iterative) {
 
-    w_mean = SUNRsqrt( N_VDotProd(weight, weight) ) / idals_mem->sqrtN;
+    N_VConst(ONE, idals_mem->x);
+    w_mean = N_VWrmsNorm(weight, idals_mem->x);
     tol /= w_mean;
 
   }
 
+  /* Set initial guess x = 0 to LS */
+  N_VConst(ZERO, idals_mem->x);
+
+  /* Set zero initial guess flag */
+  retval = SUNLinSolSetZeroGuess(idals_mem->LS, SUNTRUE);
+  if (retval != SUNLS_SUCCESS) return(-1);
+
   /* If a user-provided jtsetup routine is supplied, call that here */
   if (idals_mem->jtsetup) {
     idals_mem->last_flag = idals_mem->jtsetup(IDA_mem->ida_tn, ycur, ypcur, rescur,
@@ -1325,14 +1458,14 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
                           idals_mem->x, b, tol);
 
   /* Copy appropriate result to b (depending on solver type) */
-  if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-       (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
+  if (idals_mem->iterative) {
 
     /* Retrieve solver statistics */
     nli_inc = SUNLinSolNumIters(idals_mem->LS);
 
     /* Copy x (or preconditioned residual vector if no iterations required) to b */
-    if (nli_inc == 0) N_VScale(ONE, SUNLinSolResid(idals_mem->LS), b);
+    if ((nli_inc == 0) && (SUNLinSolGetType(idals_mem->LS) != SUNLINEARSOLVER_MATRIX_EMBEDDED))
+      N_VScale(ONE, SUNLinSolResid(idals_mem->LS), b);
     else N_VScale(ONE, idals_mem->x, b);
 
     /* Increment nli counter */
@@ -1347,9 +1480,7 @@ int idaLsSolve(IDAMem IDA_mem, N_Vector b, N_Vector weight,
 
   /* If using a direct or matrix-iterative solver, scale the correction to
      account for change in cj */
-  if ( ((LSType == SUNLINEARSOLVER_DIRECT) ||
-        (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE)) &&
-       (IDA_mem->ida_cjratio != ONE) )
+  if (idals_mem->scalesol && (IDA_mem->ida_cjratio != ONE))
     N_VScale(TWO/(ONE + IDA_mem->ida_cjratio), b, b);
 
   /* Increment ncfl counter */
@@ -1435,10 +1566,8 @@ int idaLsPerf(IDAMem IDA_mem, int perftask)
   nnid = IDA_mem->ida_nni - idals_mem->nni0;
   if (nstd == 0 || nnid == 0) return(0);
 
-  rcfn = (realtype) ( (IDA_mem->ida_ncfn - idals_mem->ncfn0) /
-                      ((realtype) nstd) );
-  rcfl = (realtype) ( (idals_mem->ncfl - idals_mem->ncfl0) /
-                      ((realtype) nnid) );
+  rcfn = ((realtype) (IDA_mem->ida_ncfn - idals_mem->ncfn0)) / ((realtype) nstd);
+  rcfl = ((realtype) (idals_mem->ncfl - idals_mem->ncfl0)) / ((realtype) nnid);
   lcfn = (rcfn > PT9);
   lcfl = (rcfl > PT9);
   if (!(lcfn || lcfl)) return(0);
