@@ -75,6 +75,14 @@ int KINSetLinearSolver(void *kinmem, SUNLinearSolver LS, SUNMatrix A)
   /* Retrieve the LS type */
   LSType = SUNLinSolGetType(LS);
 
+  /* Return with error if LS has 'matrix-embedded' type */
+  if (LSType == SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+    KINProcessError(kin_mem, KINLS_ILL_INPUT, "KINLS",
+                   "KINSetLinearSolver",
+                   "KINSOL is incompatible with MATRIX_EMBEDDED LS objects");
+    return(KINLS_ILL_INPUT);
+  }
+
   /* Set flags based on LS type */
   iterative   = (LSType != SUNLINEARSOLVER_DIRECT);
   matrixbased = (LSType != SUNLINEARSOLVER_ITERATIVE);
@@ -1189,6 +1197,10 @@ int kinLsSolve(KINMem kin_mem, N_Vector xx, N_Vector bb,
   /* Set initial guess x = 0 to LS */
   N_VConst(ZERO, xx);
 
+  /* Set zero initial guess flag */
+  retval = SUNLinSolSetZeroGuess(kinls_mem->LS, SUNTRUE);
+  if (retval != SUNLS_SUCCESS) return(-1);
+
   /* set flag required for user-supplied J*v routine */
   kinls_mem->new_uu = SUNTRUE;
 
