@@ -2,11 +2,11 @@
  * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  *                David Gardner @ LLNL
- * Based on code sundials_sparse.c by: Carol Woodward and 
+ * Based on code sundials_sparse.c by: Carol Woodward and
  *     Slaven Peles @ LLNL, and Daniel R. Reynolds @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -15,10 +15,10 @@
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the implementation file for the sparse implementation of 
+ * This is the implementation file for the sparse implementation of
  * the SUNMATRIX package.
  * -----------------------------------------------------------------
- */ 
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,8 +33,9 @@
 /* Private function prototypes */
 static booleantype SMCompatible_Sparse(SUNMatrix A, SUNMatrix B);
 static booleantype SMCompatible2_Sparse(SUNMatrix A, N_Vector x, N_Vector y);
-int Matvec_SparseCSC(SUNMatrix A, N_Vector x, N_Vector y);
-int Matvec_SparseCSR(SUNMatrix A, N_Vector x, N_Vector y);
+static int Matvec_SparseCSC(SUNMatrix A, N_Vector x, N_Vector y);
+static int Matvec_SparseCSR(SUNMatrix A, N_Vector x, N_Vector y);
+static int format_convert(const SUNMatrix A, SUNMatrix B);
 
 /*
  * -----------------------------------------------------------------
@@ -56,38 +57,35 @@ SUNMatrix SUNSparseMatrix(sunindextype M, sunindextype N,
                           sunindextype NNZ, int sparsetype)
 {
   SUNMatrix A;
-  SUNMatrix_Ops ops;
   SUNMatrixContent_Sparse content;
 
   /* return with NULL matrix on illegal input */
   if ( (M <= 0) || (N <= 0) || (NNZ < 0) ) return(NULL);
   if ( (sparsetype != CSC_MAT) && (sparsetype != CSR_MAT) ) return(NULL);
 
-  /* Create matrix */
+  /* Create an empty matrix object */
   A = NULL;
-  A = (SUNMatrix) malloc(sizeof *A);
+  A = SUNMatNewEmpty();
   if (A == NULL) return(NULL);
-  
-  /* Create matrix operation structure */
-  ops = NULL;
-  ops = (SUNMatrix_Ops) malloc(sizeof(struct _generic_SUNMatrix_Ops));
-  if (ops == NULL) { free(A); return(NULL); }
 
   /* Attach operations */
-  ops->getid       = SUNMatGetID_Sparse;
-  ops->clone       = SUNMatClone_Sparse;
-  ops->destroy     = SUNMatDestroy_Sparse;
-  ops->zero        = SUNMatZero_Sparse;
-  ops->copy        = SUNMatCopy_Sparse;
-  ops->scaleadd    = SUNMatScaleAdd_Sparse;
-  ops->scaleaddi   = SUNMatScaleAddI_Sparse;
-  ops->matvec      = SUNMatMatvec_Sparse;
-  ops->space       = SUNMatSpace_Sparse;
+  A->ops->getid     = SUNMatGetID_Sparse;
+  A->ops->clone     = SUNMatClone_Sparse;
+  A->ops->destroy   = SUNMatDestroy_Sparse;
+  A->ops->zero      = SUNMatZero_Sparse;
+  A->ops->copy      = SUNMatCopy_Sparse;
+  A->ops->scaleadd  = SUNMatScaleAdd_Sparse;
+  A->ops->scaleaddi = SUNMatScaleAddI_Sparse;
+  A->ops->matvec    = SUNMatMatvec_Sparse;
+  A->ops->space     = SUNMatSpace_Sparse;
 
   /* Create content */
   content = NULL;
-  content = (SUNMatrixContent_Sparse) malloc(sizeof(struct _SUNMatrixContent_Sparse));
-  if (content == NULL) { free(ops); free(A); return(NULL); }
+  content = (SUNMatrixContent_Sparse) malloc(sizeof *content);
+  if (content == NULL) { SUNMatDestroy(A); return(NULL); }
+
+  /* Attach content */
+  A->content = content;
 
   /* Fill content */
   content->sparsetype = sparsetype;
@@ -111,37 +109,30 @@ SUNMatrix SUNSparseMatrix(sunindextype M, sunindextype N,
     content->rowvals = NULL;
     content->colptrs = NULL;
   }
-  content->data = (realtype *) calloc(NNZ, sizeof(realtype));
-  if (content->data == NULL) {
-    free(content); free(ops); free(A); return(NULL);
-  }
-  content->indexvals = (sunindextype *) calloc(NNZ, sizeof(sunindextype));
-  if (content->indexvals == NULL) {
-    free(content->data); free(content); free(ops); free(A); return(NULL);
-  }
-  content->indexptrs = (sunindextype *) calloc((content->NP + 1), sizeof(sunindextype));
-  if (content->indexptrs == NULL) {
-    free(content->indexvals);
-    free(content->data);
-    free(content);
-    free(ops);
-    free(A);
-    return(NULL);
-  }
-  content->indexptrs[content->NP] = 0;
+  content->data      = NULL;
+  content->indexvals = NULL;
+  content->indexptrs = NULL;
 
-  /* Attach content and ops */
-  A->content = content;
-  A->ops     = ops;
+  /* Allocate content */
+  content->data = (realtype *) calloc(NNZ, sizeof(realtype));
+  if (content->data == NULL) { SUNMatDestroy(A); return(NULL); }
+
+  content->indexvals = (sunindextype *) calloc(NNZ, sizeof(sunindextype));
+  if (content->indexvals == NULL) { SUNMatDestroy(A); return(NULL); }
+
+  content->indexptrs = (sunindextype *) calloc((content->NP + 1), sizeof(sunindextype));
+  if (content->indexptrs == NULL) { SUNMatDestroy(A); return(NULL); }
+  content->indexptrs[content->NP] = 0;
 
   return(A);
 }
 
 
 
+
 /* ----------------------------------------------------------------------------
- * Function to create a new sparse matrix from an existing dense matrix 
- * by copying all nonzero values into the sparse matrix structure.  Returns NULL 
+ * Function to create a new sparse matrix from an existing dense matrix
+ * by copying all nonzero values into the sparse matrix structure.  Returns NULL
  * if the request for matrix storage cannot be satisfied.
  */
 
@@ -158,7 +149,7 @@ SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetyp
     return NULL;
   if (SUNMatGetID(Ad) != SUNMATRIX_DENSE)
     return NULL;
-  
+
   /* set size of new matrix */
   M = SM_ROWS_D(Ad);
   N = SM_COLUMNS_D(Ad);
@@ -168,18 +159,18 @@ SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetyp
   for (j=0; j<N; j++)
     for (i=0; i<M; i++)
       nnz += (SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol);
-    
+
   /* allocate sparse matrix */
   As = SUNSparseMatrix(M, N, nnz, sparsetype);
   if (As == NULL)  return NULL;
-  
+
   /* copy nonzeros from Ad into As, based on CSR/CSC type */
   nnz = 0;
   if (sparsetype == CSC_MAT) {
     for (j=0; j<N; j++) {
       (SM_INDEXPTRS_S(As))[j] = nnz;
       for (i=0; i<M; i++) {
-        if ( SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol ) { 
+        if ( SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol ) {
           (SM_INDEXVALS_S(As))[nnz] = i;
           (SM_DATA_S(As))[nnz++] = SM_ELEMENT_D(Ad,i,j);
         }
@@ -190,7 +181,7 @@ SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetyp
     for (i=0; i<M; i++) {
       (SM_INDEXPTRS_S(As))[i] = nnz;
       for (j=0; j<N; j++) {
-        if ( SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol ) { 
+        if ( SUNRabs(SM_ELEMENT_D(Ad,i,j)) > droptol ) {
           (SM_INDEXVALS_S(As))[nnz] = j;
           (SM_DATA_S(As))[nnz++] = SM_ELEMENT_D(Ad,i,j);
         }
@@ -198,14 +189,14 @@ SUNMatrix SUNSparseFromDenseMatrix(SUNMatrix Ad, realtype droptol, int sparsetyp
     }
     (SM_INDEXPTRS_S(As))[M] = nnz;
   }
-    
+
   return(As);
 }
 
 
 /* ----------------------------------------------------------------------------
- * Function to create a new sparse matrix from an existing band matrix 
- * by copying all nonzero values into the sparse matrix structure.  Returns NULL 
+ * Function to create a new sparse matrix from an existing band matrix
+ * by copying all nonzero values into the sparse matrix structure.  Returns NULL
  * if the request for matrix storage cannot be satisfied.
  */
 
@@ -222,7 +213,7 @@ SUNMatrix SUNSparseFromBandMatrix(SUNMatrix Ad, realtype droptol, int sparsetype
     return NULL;
   if (SUNMatGetID(Ad) != SUNMATRIX_BAND)
     return NULL;
-  
+
   /* set size of new matrix */
   M = SM_ROWS_B(Ad);
   N = SM_COLUMNS_B(Ad);
@@ -243,7 +234,7 @@ SUNMatrix SUNSparseFromBandMatrix(SUNMatrix Ad, realtype droptol, int sparsetype
     for (j=0; j<N; j++) {
       (SM_INDEXPTRS_S(As))[j] = nnz;
       for (i=SUNMAX(0,j-SM_UBAND_B(Ad)); i<=SUNMIN(M-1,j+SM_LBAND_B(Ad)); i++) {
-        if ( SUNRabs(SM_ELEMENT_B(Ad,i,j)) > droptol ) { 
+        if ( SUNRabs(SM_ELEMENT_B(Ad,i,j)) > droptol ) {
           (SM_INDEXVALS_S(As))[nnz] = i;
           (SM_DATA_S(As))[nnz++] = SM_ELEMENT_B(Ad,i,j);
         }
@@ -254,7 +245,7 @@ SUNMatrix SUNSparseFromBandMatrix(SUNMatrix Ad, realtype droptol, int sparsetype
     for (i=0; i<M; i++) {
       (SM_INDEXPTRS_S(As))[i] = nnz;
       for (j=SUNMAX(0,i-SM_LBAND_B(Ad)); j<=SUNMIN(N-1,i+SM_UBAND_B(Ad)); j++) {
-        if ( SUNRabs(SM_ELEMENT_B(Ad,i,j)) > droptol ) { 
+        if ( SUNRabs(SM_ELEMENT_B(Ad,i,j)) > droptol ) {
           (SM_INDEXVALS_S(As))[nnz] = j;
           (SM_DATA_S(As))[nnz++] = SM_ELEMENT_B(Ad,i,j);
         }
@@ -267,6 +258,36 @@ SUNMatrix SUNSparseFromBandMatrix(SUNMatrix Ad, realtype droptol, int sparsetype
 }
 
 
+/* ----------------------------------------------------------------------------
+ * Function to create a new CSR matrix from a CSC matrix.
+ */
+int SUNSparseMatrix_ToCSR(const SUNMatrix A, SUNMatrix* Bout)
+{
+    if (A == NULL) return(SUNMAT_ILL_INPUT);
+    if (SM_SPARSETYPE_S(A) != CSC_MAT) return(SUNMAT_ILL_INPUT);
+
+    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSR_MAT);
+    if (*Bout == NULL) return(SUNMAT_MEM_FAIL);
+
+    return format_convert(A, *Bout);
+}
+
+
+/* ----------------------------------------------------------------------------
+ * Function to create a new CSC matrix from a CSR matrix.
+ */
+int SUNSparseMatrix_ToCSC(const SUNMatrix A, SUNMatrix* Bout)
+{
+    if (A == NULL) return(SUNMAT_ILL_INPUT);
+    if (SM_SPARSETYPE_S(A) != CSR_MAT) return(SUNMAT_ILL_INPUT);
+
+    *Bout = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A), SM_NNZ_S(A), CSC_MAT);
+    if (*Bout == NULL) return(SUNMAT_MEM_FAIL);
+
+    return format_convert(A, *Bout);
+}
+
+
 /* ----------------------------------------------------------------------------
  * Function to reallocate internal sparse matrix storage arrays so that the
  * resulting sparse matrix holds indexptrs[NP] nonzeros.  Returns 0 on success
@@ -279,57 +300,56 @@ int SUNSparseMatrix_Realloc(SUNMatrix A)
 
   /* check for valid matrix type */
   if (SUNMatGetID(A) != SUNMATRIX_SPARSE)
-    return 1;
+    return SUNMAT_ILL_INPUT;
 
   /* get total number of nonzeros (return with failure if illegal) */
   nzmax = (SM_INDEXPTRS_S(A))[SM_NP_S(A)];
-  if (nzmax < 0)
-    return 1;
+  if (nzmax < 0) return SUNMAT_ILL_INPUT;
 
   /* perform reallocation */
   SM_INDEXVALS_S(A) = (sunindextype *) realloc(SM_INDEXVALS_S(A), nzmax*sizeof(sunindextype));
   SM_DATA_S(A) = (realtype *) realloc(SM_DATA_S(A), nzmax*sizeof(realtype));
   SM_NNZ_S(A) = nzmax;
 
-  return 0;
+  return SUNMAT_SUCCESS;
 }
 
 
 /* ----------------------------------------------------------------------------
  * Function to reallocate internal sparse matrix storage arrays so that the
- * resulting sparse matrix has storage for a specified number of nonzeros.  
- * Returns 0 on success and 1 on failure (e.g. if A does not have sparse type, 
+ * resulting sparse matrix has storage for a specified number of nonzeros.
+ * Returns 0 on success and 1 on failure (e.g. if A does not have sparse type,
  * or if nnz is negative)
  */
 
 int SUNSparseMatrix_Reallocate(SUNMatrix A, sunindextype NNZ)
 {
   /* check for valid matrix type */
-  if (SUNMatGetID(A) != SUNMATRIX_SPARSE)  return 1;
+  if (SUNMatGetID(A) != SUNMATRIX_SPARSE)  return SUNMAT_ILL_INPUT;
 
   /* check for valid nnz */
-  if (NNZ < 0)  return 1;
+  if (NNZ < 0)  return SUNMAT_ILL_INPUT;
 
   /* perform reallocation */
   SM_INDEXVALS_S(A) = (sunindextype *) realloc(SM_INDEXVALS_S(A), NNZ*sizeof(sunindextype));
   SM_DATA_S(A) = (realtype *) realloc(SM_DATA_S(A), NNZ*sizeof(realtype));
   SM_NNZ_S(A) = NNZ;
 
-  return 0;
+  return SUNMAT_SUCCESS;
 }
 
 
 /* ----------------------------------------------------------------------------
- * Function to print the sparse matrix 
+ * Function to print the sparse matrix
  */
- 
+
 void SUNSparseMatrix_Print(SUNMatrix A, FILE* outfile)
 {
   sunindextype i, j;
   char *matrixtype;
   char *indexname;
 
-  /* should not be called unless A is a sparse matrix; 
+  /* should not be called unless A is a sparse matrix;
      otherwise return immediately */
   if (SUNMatGetID(A) != SUNMATRIX_SPARSE)
     return;
@@ -344,7 +364,7 @@ void SUNSparseMatrix_Print(SUNMatrix A, FILE* outfile)
   }
   fprintf(outfile, "\n");
   fprintf(outfile, "%ld by %ld %s matrix, NNZ: %ld \n",
-          (long int) SM_ROWS_S(A), (long int) SM_COLUMNS_S(A), 
+          (long int) SM_ROWS_S(A), (long int) SM_COLUMNS_S(A),
           matrixtype, (long int) SM_NNZ_S(A));
   for (j=0; j<SM_NP_S(A); j++) {
     fprintf(outfile, "%s %ld : locations %ld to %ld\n", indexname,
@@ -379,7 +399,7 @@ sunindextype SUNSparseMatrix_Rows(SUNMatrix A)
   if (SUNMatGetID(A) == SUNMATRIX_SPARSE)
     return SM_ROWS_S(A);
   else
-    return -1;
+    return SUNMAT_ILL_INPUT;
 }
 
 sunindextype SUNSparseMatrix_Columns(SUNMatrix A)
@@ -387,7 +407,7 @@ sunindextype SUNSparseMatrix_Columns(SUNMatrix A)
   if (SUNMatGetID(A) == SUNMATRIX_SPARSE)
     return SM_COLUMNS_S(A);
   else
-    return -1;
+    return SUNMAT_ILL_INPUT;
 }
 
 sunindextype SUNSparseMatrix_NNZ(SUNMatrix A)
@@ -395,7 +415,7 @@ sunindextype SUNSparseMatrix_NNZ(SUNMatrix A)
   if (SUNMatGetID(A) == SUNMATRIX_SPARSE)
     return SM_NNZ_S(A);
   else
-    return -1;
+    return SUNMAT_ILL_INPUT;
 }
 
 sunindextype SUNSparseMatrix_NP(SUNMatrix A)
@@ -403,7 +423,7 @@ sunindextype SUNSparseMatrix_NP(SUNMatrix A)
   if (SUNMatGetID(A) == SUNMATRIX_SPARSE)
     return SM_NP_S(A);
   else
-    return -1;
+    return SUNMAT_ILL_INPUT;
 }
 
 int SUNSparseMatrix_SparseType(SUNMatrix A)
@@ -411,7 +431,7 @@ int SUNSparseMatrix_SparseType(SUNMatrix A)
   if (SUNMatGetID(A) == SUNMATRIX_SPARSE)
     return SM_SPARSETYPE_S(A);
   else
-    return -1;
+    return SUNMAT_ILL_INPUT;
 }
 
 realtype* SUNSparseMatrix_Data(SUNMatrix A)
@@ -459,28 +479,41 @@ SUNMatrix SUNMatClone_Sparse(SUNMatrix A)
 
 void SUNMatDestroy_Sparse(SUNMatrix A)
 {
-  /* perform operation */
-  if (SM_DATA_S(A)) {
-    free(SM_DATA_S(A));  SM_DATA_S(A) = NULL;
+  if (A == NULL) return;
+
+  /* free content */
+  if (A->content != NULL) {
+    /* free data array */
+    if (SM_DATA_S(A)) {
+      free(SM_DATA_S(A));
+      SM_DATA_S(A) = NULL;
+    }
+    /* free index values array */
+    if (SM_INDEXVALS_S(A)) {
+      free(SM_INDEXVALS_S(A));
+      SM_INDEXVALS_S(A) = NULL;
+      SM_CONTENT_S(A)->rowvals = NULL;
+      SM_CONTENT_S(A)->colvals = NULL;
+    }
+    /* free index pointers array */
+    if (SM_INDEXPTRS_S(A)) {
+      free(SM_INDEXPTRS_S(A));
+      SM_INDEXPTRS_S(A) = NULL;
+      SM_CONTENT_S(A)->colptrs = NULL;
+      SM_CONTENT_S(A)->rowptrs = NULL;
+    }
+    /* free content struct */
+    free(A->content);
+    A->content = NULL;
   }
-  if (SM_INDEXVALS_S(A)) {
-    free(SM_INDEXVALS_S(A));
-    SM_INDEXVALS_S(A) = NULL;
-    SM_CONTENT_S(A)->rowvals = NULL;
-    SM_CONTENT_S(A)->colvals = NULL;
-  }
-  if (SM_INDEXPTRS_S(A)) {
-    free(SM_INDEXPTRS_S(A));
-    SM_INDEXPTRS_S(A) = NULL;
-    SM_CONTENT_S(A)->colptrs = NULL;
-    SM_CONTENT_S(A)->rowptrs = NULL;
-  }
-  free(A->content); A->content = NULL;
-  free(A->ops);  A->ops = NULL;
+
+  /* free ops and matrix */
+  if (A->ops) { free(A->ops); A->ops = NULL; }
   free(A); A = NULL;
+
   return;
 }
-  
+
 int SUNMatZero_Sparse(SUNMatrix A)
 {
   sunindextype i;
@@ -490,10 +523,10 @@ int SUNMatZero_Sparse(SUNMatrix A)
     (SM_DATA_S(A))[i] = ZERO;
     (SM_INDEXVALS_S(A))[i] = 0;
   }
-  for (i=0; i<SM_NP_S(A); i++) 
+  for (i=0; i<SM_NP_S(A); i++)
     (SM_INDEXPTRS_S(A))[i] = 0;
   (SM_INDEXPTRS_S(A))[SM_NP_S(A)] = 0;
-  return 0;
+  return SUNMAT_SUCCESS;
 }
 
 int SUNMatCopy_Sparse(SUNMatrix A, SUNMatrix B)
@@ -502,12 +535,12 @@ int SUNMatCopy_Sparse(SUNMatrix A, SUNMatrix B)
 
   /* Verify that A and B are compatible */
   if (!SMCompatible_Sparse(A, B))
-    return 1;
+    return SUNMAT_ILL_INPUT;
 
   /* Perform operation */
   A_nz = (SM_INDEXPTRS_S(A))[SM_NP_S(A)];
-  
-  /* ensure that B is allocated with at least as 
+
+  /* ensure that B is allocated with at least as
      much memory as we have nonzeros in A */
   if (SM_NNZ_S(B) < A_nz) {
     SM_INDEXVALS_S(B) = (sunindextype *) realloc(SM_INDEXVALS_S(B), A_nz*sizeof(sunindextype));
@@ -516,7 +549,8 @@ int SUNMatCopy_Sparse(SUNMatrix A, SUNMatrix B)
   }
 
   /* zero out B so that copy works correctly */
-  SUNMatZero_Sparse(B);
+  if (SUNMatZero_Sparse(B) != SUNMAT_SUCCESS)
+    return SUNMAT_OPERATION_FAIL;
 
   /* copy the data and row indices over */
   for (i=0; i<A_nz; i++){
@@ -529,8 +563,8 @@ int SUNMatCopy_Sparse(SUNMatrix A, SUNMatrix B)
     (SM_INDEXPTRS_S(B))[i] = (SM_INDEXPTRS_S(A))[i];
   }
   (SM_INDEXPTRS_S(B))[SM_NP_S(A)] = A_nz;
-  
-  return 0;
+
+  return SUNMAT_SUCCESS;
 }
 
 int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
@@ -555,14 +589,14 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
   Ap = Ai = NULL;
   Ax = NULL;
   if (SM_INDEXPTRS_S(A))  Ap = SM_INDEXPTRS_S(A);
-  else  return (-1);
+  else  return (SUNMAT_MEM_FAIL);
   if (SM_INDEXVALS_S(A))  Ai = SM_INDEXVALS_S(A);
-  else  return (-1);
+  else  return (SUNMAT_MEM_FAIL);
   if (SM_DATA_S(A))       Ax = SM_DATA_S(A);
-  else  return (-1);
+  else  return (SUNMAT_MEM_FAIL);
 
-  
-  /* determine if A: contains values on the diagonal (so I can just be added in); 
+
+  /* determine if A: contains values on the diagonal (so I can just be added in);
      if not, then increment counter for extra storage that should be required. */
   newvals = 0;
   for (j=0; j < SUNMIN(M,N); j++) {
@@ -578,13 +612,13 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     if (!found)  newvals += 1;
   }
 
-  /* If extra nonzeros required, check whether matrix has sufficient storage space 
+  /* If extra nonzeros required, check whether matrix has sufficient storage space
      for new nonzero entries  (so I can be inserted into existing storage) */
   newmat = SUNFALSE;   /* no reallocation needed */
   if (newvals > (SM_NNZ_S(A) - Ap[N]))
     newmat = SUNTRUE;
 
-  
+
   /* perform operation based on existing/necessary structure */
 
   /*   case 1: A already contains a diagonal */
@@ -599,11 +633,11 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
           Ax[i] = c*Ax[i];
         }
 
-    
+
   /*   case 2: A has sufficient storage, but does not already contain a diagonal */
   } else if (!newmat) {
 
-   
+
     /* create work arrays for nonzero indices and values in a single column (row) */
     w = (sunindextype *) malloc(M * sizeof(sunindextype));
     x = (realtype *) malloc(M * sizeof(realtype));
@@ -611,11 +645,11 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     /* determine storage location where last column (row) should end */
     nz = Ap[N] + newvals;
 
-    /* store pointer past last column (row) from original A, 
-       and store updated value in revised A */ 
+    /* store pointer past last column (row) from original A,
+       and store updated value in revised A */
     cend = Ap[N];
     Ap[N] = nz;
-    
+
     /* iterate through columns (rows) backwards */
     for (j=N-1; j>=0; j--) {
 
@@ -639,8 +673,8 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
 
       /* fill entries of A with this column's (row's) data */
       for (i=M-1; i>=0; i--) {
-        if ( w[i] > 0 ) { 
-          Ai[--nz] = i;  
+        if ( w[i] > 0 ) {
+          Ai[--nz] = i;
           Ax[nz] = x[i];
         }
       }
@@ -648,14 +682,14 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
       /* store ptr past this col (row) from orig A, update value for new A */
       cend = Ap[j];
       Ap[j] = nz;
-      
+
     }
 
     /* clean up */
     free(w);
     free(x);
 
-    
+
   /*   case 3: A must be reallocated with sufficient storage */
   } else {
 
@@ -672,11 +706,11 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     Cp = Ci = NULL;
     Cx = NULL;
     if (SM_INDEXPTRS_S(C))  Cp = SM_INDEXPTRS_S(C);
-    else  return (-1);
+    else  return (SUNMAT_MEM_FAIL);
     if (SM_INDEXVALS_S(C))  Ci = SM_INDEXVALS_S(C);
-    else  return (-1);
+    else  return (SUNMAT_MEM_FAIL);
     if (SM_DATA_S(C))       Cx = SM_DATA_S(C);
-    else  return (-1);
+    else  return (SUNMAT_MEM_FAIL);
 
     /* initialize total nonzero count */
     nz = 0;
@@ -707,8 +741,8 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
 
       /* fill entries of C with this column's (row's) data */
       for (i=0; i<M; i++) {
-        if ( w[i] > 0 ) { 
-          Ci[nz] = i;  
+        if ( w[i] > 0 ) {
+          Ci[nz] = i;
           Cx[nz++] = x[i];
         }
       }
@@ -721,7 +755,7 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     SM_NNZ_S(A) = SM_NNZ_S(C);
 
     if (SM_DATA_S(A))
-      free(SM_DATA_S(A));  
+      free(SM_DATA_S(A));
     SM_DATA_S(A) = SM_DATA_S(C);
     SM_DATA_S(C) = NULL;
 
@@ -736,12 +770,12 @@ int SUNMatScaleAddI_Sparse(realtype c, SUNMatrix A)
     SM_INDEXPTRS_S(C) = NULL;
 
     /* clean up */
-    SUNMatDestroy_Sparse(C); 
+    SUNMatDestroy_Sparse(C);
     free(w);
     free(x);
 
   }
-  return 0;
+  return SUNMAT_SUCCESS;
 
 }
 
@@ -755,7 +789,7 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
 
   /* Verify that A and B are compatible */
   if (!SMCompatible_Sparse(A, B))
-    return 1;
+    return SUNMAT_ILL_INPUT;
 
   /* store shortcuts to matrix dimensions (M is inner dimension, N is outer) */
   if (SM_SPARSETYPE_S(A) == CSC_MAT) {
@@ -771,17 +805,17 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
   Ap = Ai = Bp = Bi = NULL;
   Ax = Bx = NULL;
   if (SM_INDEXPTRS_S(A))  Ap = SM_INDEXPTRS_S(A);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
   if (SM_INDEXVALS_S(A))  Ai = SM_INDEXVALS_S(A);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
   if (SM_DATA_S(A))       Ax = SM_DATA_S(A);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
   if (SM_INDEXPTRS_S(B))  Bp = SM_INDEXPTRS_S(B);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
   if (SM_INDEXVALS_S(B))  Bi = SM_INDEXVALS_S(B);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
   if (SM_DATA_S(B))       Bx = SM_DATA_S(B);
-  else  return(-1);
+  else  return(SUNMAT_MEM_FAIL);
 
   /* create work arrays for row indices and nonzero column values */
   w = (sunindextype *) malloc(M * sizeof(sunindextype));
@@ -802,18 +836,18 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
     for (i=Bp[j]; i<Bp[j+1]; i++)
       w[Bi[i]] -= 1;
 
-    /* if any entry of w is negative, A doesn't contain B's sparsity, 
+    /* if any entry of w is negative, A doesn't contain B's sparsity,
        so increment necessary storage counter */
     for (i=0; i<M; i++)
       if (w[i] < 0)  newvals += 1;
   }
 
-  /* If extra nonzeros required, check whether A has sufficient storage space 
+  /* If extra nonzeros required, check whether A has sufficient storage space
      for new nonzero entries (so B can be inserted into existing storage) */
   newmat = SUNFALSE;   /* no reallocation needed */
   if (newvals > (SM_NNZ_S(A) - Ap[N]))
     newmat = SUNTRUE;
-  
+
   /* perform operation based on existing/necessary structure */
 
   /*   case 1: A already contains sparsity pattern of B */
@@ -836,23 +870,21 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
 
     }
 
-    
+
   /*   case 2: A has sufficient storage, but does not already contain B's sparsity */
   } else if (!newmat) {
 
-
     /* determine storage location where last column (row) should end */
     nz = Ap[N] + newvals;
 
-    /* store pointer past last column (row) from original A, 
-       and store updated value in revised A */ 
+    /* store pointer past last column (row) from original A,
+       and store updated value in revised A */
     cend = Ap[N];
     Ap[N] = nz;
-    
+
     /* iterate through columns (rows) backwards */
     for (j=N-1; j>=0; j--) {
 
-
       /* clear out temporary arrays for this column (row) */
       for (i=0; i<M; i++) {
         w[i] = 0;
@@ -873,8 +905,8 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
 
       /* fill entries of A with this column's (row's) data */
       for (i=M-1; i>=0; i--) {
-        if ( w[i] > 0 ) { 
-          Ai[--nz] = i;  
+        if ( w[i] > 0 ) {
+          Ai[--nz] = i;
           Ax[nz] = x[i];
         }
       }
@@ -882,14 +914,14 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
       /* store ptr past this col (row) from orig A, update value for new A */
       cend = Ap[j];
       Ap[j] = nz;
-      
+
     }
-    
-    
+
+
   /*   case 3: A must be reallocated with sufficient storage */
   } else {
 
-    
+
     /* create new matrix for sum */
     C = SUNSparseMatrix(SM_ROWS_S(A), SM_COLUMNS_S(A),
                         Ap[N] + newvals, SM_SPARSETYPE_S(A));
@@ -898,11 +930,11 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
     Cp = Ci = NULL;
     Cx = NULL;
     if (SM_INDEXPTRS_S(C))  Cp = SM_INDEXPTRS_S(C);
-    else  return(-1);
+    else  return(SUNMAT_MEM_FAIL);
     if (SM_INDEXVALS_S(C))  Ci = SM_INDEXVALS_S(C);
-    else  return(-1);
+    else  return(SUNMAT_MEM_FAIL);
     if (SM_DATA_S(C))       Cx = SM_DATA_S(C);
-    else  return(-1);
+    else  return(SUNMAT_MEM_FAIL);
 
     /* initialize total nonzero count */
     nz = 0;
@@ -933,8 +965,8 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
 
       /* fill entries of C with this column's data */
       for (i=0; i<M; i++) {
-        if ( w[i] > 0 ) { 
-          Ci[nz] = i;  
+        if ( w[i] > 0 ) {
+          Ci[nz] = i;
           Cx[nz++] = x[i];
         }
       }
@@ -946,7 +978,7 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
     /* update A's structure with C's values; nullify C's pointers */
     SM_NNZ_S(A) = SM_NNZ_S(C);
 
-    free(SM_DATA_S(A));  
+    free(SM_DATA_S(A));
     SM_DATA_S(A) = SM_DATA_S(C);
     SM_DATA_S(C) = NULL;
 
@@ -959,7 +991,7 @@ int SUNMatScaleAdd_Sparse(realtype c, SUNMatrix A, SUNMatrix B)
     SM_INDEXPTRS_S(C) = NULL;
 
     /* clean up */
-    SUNMatDestroy_Sparse(C); 
+    SUNMatDestroy_Sparse(C);
 
   }
 
@@ -976,12 +1008,12 @@ int SUNMatMatvec_Sparse(SUNMatrix A, N_Vector x, N_Vector y)
 {
   /* Verify that A, x and y are compatible */
   if (!SMCompatible2_Sparse(A, x, y))
-    return 1;
+    return SUNMAT_ILL_INPUT;
 
   /* Perform operation */
   if(SM_SPARSETYPE_S(A) == CSC_MAT)
     return Matvec_SparseCSC(A, x, y);
-  else 
+  else
     return Matvec_SparseCSR(A, x, y);
 }
 
@@ -989,7 +1021,7 @@ int SUNMatSpace_Sparse(SUNMatrix A, long int *lenrw, long int *leniw)
 {
   *lenrw = SM_NNZ_S(A);
   *leniw = 10 + SM_NP_S(A) + SM_NNZ_S(A);
-  return 0;
+  return SUNMAT_SUCCESS;
 }
 
 
@@ -1029,26 +1061,26 @@ static booleantype SMCompatible_Sparse(SUNMatrix A, SUNMatrix B)
 
 static booleantype SMCompatible2_Sparse(SUNMatrix A, N_Vector x, N_Vector y)
 {
-
-  /*   vectors must be one of {SERIAL, OPENMP, PTHREADS} */ 
-  if ( (N_VGetVectorID(x) != SUNDIALS_NVEC_SERIAL) &&
-       (N_VGetVectorID(x) != SUNDIALS_NVEC_OPENMP) &&
-       (N_VGetVectorID(x) != SUNDIALS_NVEC_PTHREADS) )
+  /* vectors must implement N_VGetArrayPointer */
+  if ( (x->ops->nvgetarraypointer == NULL) ||
+       (y->ops->nvgetarraypointer == NULL) )
     return SUNFALSE;
 
-  /* Optimally we would verify that the dimensions of A, x and y agree, 
-   but since there is no generic 'length' routine for N_Vectors we cannot */
+  /* Verify that the dimensions of A, x, and y agree */
+  if ( (SUNSparseMatrix_Columns(A) != N_VGetLength(x)) ||
+       (SUNSparseMatrix_Rows(A) != N_VGetLength(y)) )
+    return SUNFALSE;
 
   return SUNTRUE;
 }
 
 
 /* -----------------------------------------------------------------
- * Computes y=A*x, where A is a CSC SUNMatrix_Sparse of dimension MxN, x is a 
- * compatible N_Vector object of length N, and y is a compatible 
+ * Computes y=A*x, where A is a CSC SUNMatrix_Sparse of dimension MxN, x is a
+ * compatible N_Vector object of length N, and y is a compatible
  * N_Vector object of length M.
- * 
- * Returns 0 if successful, 1 if unsuccessful (failed memory access, or both 
+ *
+ * Returns 0 if successful, 1 if unsuccessful (failed memory access, or both
  * x and y are the same vector).
  */
 int Matvec_SparseCSC(SUNMatrix A, N_Vector x, N_Vector y)
@@ -1062,13 +1094,13 @@ int Matvec_SparseCSC(SUNMatrix A, N_Vector x, N_Vector y)
   Ai = SM_INDEXVALS_S(A);
   Ax = SM_DATA_S(A);
   if ((Ap == NULL) || (Ai == NULL) || (Ax == NULL))
-    return 1;
+    return SUNMAT_MEM_FAIL;
 
   /* access vector data (return if failure) */
   xd = N_VGetArrayPointer(x);
   yd = N_VGetArrayPointer(y);
   if ((xd == NULL) || (yd == NULL) || (xd == yd) )
-    return 1;
+    return SUNMAT_MEM_FAIL;
 
   /* initialize result */
   for (i=0; i<SM_ROWS_S(A); i++)
@@ -1083,16 +1115,16 @@ int Matvec_SparseCSC(SUNMatrix A, N_Vector x, N_Vector y)
 
   }
 
-  return 0;
+  return SUNMAT_SUCCESS;
 }
 
 
 /* -----------------------------------------------------------------
- * Computes y=A*x, where A is a CSR SUNMatrix_Sparse of dimension MxN, x is a 
- * compatible N_Vector object of length N, and y is a compatible 
+ * Computes y=A*x, where A is a CSR SUNMatrix_Sparse of dimension MxN, x is a
+ * compatible N_Vector object of length N, and y is a compatible
  * N_Vector object of length M.
- * 
- * Returns 0 if successful, 1 if unsuccessful (failed memory access).
+ *
+ * Returns 0 if successful, -1 if unsuccessful (failed memory access).
  */
 int Matvec_SparseCSR(SUNMatrix A, N_Vector x, N_Vector y)
 {
@@ -1105,13 +1137,13 @@ int Matvec_SparseCSR(SUNMatrix A, N_Vector x, N_Vector y)
   Aj = SM_INDEXVALS_S(A);
   Ax = SM_DATA_S(A);
   if ((Ap == NULL) || (Aj == NULL) || (Ax == NULL))
-    return 1;
+    return SUNMAT_MEM_FAIL;
 
   /* access vector data (return if failure) */
   xd = N_VGetArrayPointer(x);
   yd = N_VGetArrayPointer(y);
   if ((xd == NULL) || (yd == NULL) || (xd == yd))
-    return 1;
+    return SUNMAT_MEM_FAIL;
 
   /* initialize result */
   for (i=0; i<SM_ROWS_S(A); i++)
@@ -1130,3 +1162,72 @@ int Matvec_SparseCSR(SUNMatrix A, N_Vector x, N_Vector y)
 }
 
 
+/* -----------------------------------------------------------------
+ * Copies A into a matrix B in the opposite format of A.
+ * Returns 0 if successful, nonzero if unsuccessful.
+ */
+int format_convert(const SUNMatrix A, SUNMatrix B)
+{
+    realtype *Ax, *Bx;
+    sunindextype *Ap, *Aj;
+    sunindextype *Bp, *Bi;
+    sunindextype n_row, n_col, nnz;
+    sunindextype n, col, csum, row, last;
+
+    if (SM_SPARSETYPE_S(A) == SM_SPARSETYPE_S(B))
+      return SUNMatCopy_Sparse(A, B);
+
+    Ap = SM_INDEXPTRS_S(A);
+    Aj = SM_INDEXVALS_S(A);
+    Ax = SM_DATA_S(A);
+    
+    n_row = (SM_SPARSETYPE_S(A) == CSR_MAT) ? SM_ROWS_S(A) : SM_COLUMNS_S(A);
+    n_col = (SM_SPARSETYPE_S(A) == CSR_MAT) ? SM_COLUMNS_S(A) : SM_ROWS_S(A);
+
+    Bp = SM_INDEXPTRS_S(B);
+    Bi = SM_INDEXVALS_S(B);
+    Bx = SM_DATA_S(B);
+
+    nnz = Ap[n_row];
+
+    SUNMatZero_Sparse(B);
+
+    /* compute number of non-zero entries per column (if CSR) or per row (if CSC) of A */
+    for (n = 0; n < nnz; n++)
+    {
+        Bp[Aj[n]]++;
+    }
+
+    /* cumualtive sum the nnz per column to get Bp[] */
+    for (col = 0, csum = 0; col < n_col; col++)
+    {
+        sunindextype temp  = Bp[col];
+        Bp[col] = csum;
+        csum += temp;
+    }
+    Bp[n_col] = nnz;
+
+    for (row = 0; row < n_row; row++)
+    {
+        sunindextype jj;
+        for (jj = Ap[row]; jj < Ap[row+1]; jj++)
+        {
+            sunindextype col  = Aj[jj];
+            sunindextype dest = Bp[col];
+
+            Bi[dest] = row;
+            Bx[dest] = Ax[jj];
+
+            Bp[col]++;
+        }
+    }
+
+    for (col = 0, last = 0; col <= n_col; col++)
+    {
+        sunindextype temp  = Bp[col];
+        Bp[col] = last;
+        last    = temp;
+    }
+
+    return 0;
+}
\ No newline at end of file
